@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008 Red Hat, Inc.
+ * Copyright (c) 2008, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -36,33 +36,57 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 
-#include <corosync/swab.h>
-#include <corosync/totem/totem.h>
-#include "util.h"
-#include <corosync/engine/logsys.h>
-#include "timer.h"
+#include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/lcr/lcr_ifact.h>
 #include <corosync/totem/totempg.h>
 #include <corosync/totem/totemip.h>
-#include "main.h"
-#include "coroipcs.h"
+#include <corosync/totem/totem.h>
+#include <corosync/engine/logsys.h>
+#include <corosync/coroipcs.h>
+#include "util.h"
+#include "timer.h"
 #include "sync.h"
 #include "quorum.h"
-#include <corosync/engine/coroapi.h>
 #include "service.h"
-#include <corosync/lcr/lcr_ifact.h>
+#include "schedwrk.h"
+#include "main.h"
+#include <corosync/engine/coroapi.h>
+#include "apidef.h"
 
-LOGSYS_DECLARE_SUBSYS ("APIDEF", LOG_INFO);
+LOGSYS_DECLARE_SUBSYS ("APIDEF");
 
 /*
- * Remove compile warnings about type name changes
+ * Remove compile warnings about type name changes in corosync_tpg_group
  */
-typedef int (*typedef_tpg_join) (hdb_handle_t, struct corosync_tpg_group *, int);
-typedef int (*typedef_tpg_leave) (hdb_handle_t, struct corosync_tpg_group *, int);
-typedef int (*typedef_tpg_groups_mcast) (hdb_handle_t, int, struct corosync_tpg_group *, int groups_cnt, struct iovec *, int);
-typedef int (*typedef_tpg_groups_send_ok) (hdb_handle_t, struct corosync_tpg_group *, int groups_cnt, struct iovec *, int);
+typedef int (*typedef_tpg_join) (
+	hdb_handle_t,
+	const struct corosync_tpg_group *,
+	size_t);
 
+typedef int (*typedef_tpg_leave) (hdb_handle_t,
+	const struct corosync_tpg_group *,
+	size_t);
+
+typedef int (*typedef_tpg_groups_mcast_groups) (
+	hdb_handle_t, int,
+	const struct corosync_tpg_group *,
+	size_t groups_cnt,
+	const struct iovec *,
+	unsigned int);
+
+typedef int (*typedef_tpg_groups_send_ok) (
+	hdb_handle_t,
+	const struct corosync_tpg_group *,
+	size_t groups_cnt,
+	struct iovec *,
+	int);
+
+static inline void _corosync_public_exit_error (cs_fatal_error_t err,
+						const char *file,
+						unsigned int line)
+  __attribute__((__noreturn__));
 static inline void _corosync_public_exit_error (
 	cs_fatal_error_t err, const char *file, unsigned int line)
 {
@@ -91,6 +115,7 @@ static struct corosync_api_v1 apidef_corosync_api_v1 = {
 	.totem_ifaces_get = totempg_ifaces_get,
 	.totem_ifaces_print = totempg_ifaces_print,
 	.totem_ip_print = totemip_print,
+	.totem_crypto_set = totempg_crypto_set,
 	.totem_callback_token_create = totempg_callback_token_create,
 	.tpg_init = totempg_groups_initialize,
 	.tpg_exit = NULL, /* missing from totempg api */
@@ -99,10 +124,12 @@ static struct corosync_api_v1 apidef_corosync_api_v1 = {
 	.tpg_joined_mcast = totempg_groups_mcast_joined,
 	.tpg_joined_reserve = totempg_groups_joined_reserve,
 	.tpg_joined_release = totempg_groups_joined_release,
-	.tpg_groups_mcast = (typedef_tpg_groups_mcast)totempg_groups_mcast_groups,
+	.tpg_groups_mcast = (typedef_tpg_groups_mcast_groups)totempg_groups_mcast_groups,
 	.tpg_groups_reserve = NULL,
 	.tpg_groups_release = NULL,
-	.sync_request = sync_request,
+	.schedwrk_create = schedwrk_create,
+	.schedwrk_destroy = schedwrk_destroy,
+	.sync_request = NULL, //sync_request,
 	.quorum_is_quorate = corosync_quorum_is_quorate,
 	.quorum_register_callback = corosync_quorum_register_callback,
 	.quorum_unregister_callback = corosync_quorum_unregister_callback,
@@ -112,7 +139,10 @@ static struct corosync_api_v1 apidef_corosync_api_v1 = {
 	.plugin_interface_reference = lcr_ifact_reference,
 	.plugin_interface_release = lcr_ifact_release,
 	.error_memory_failure = _corosync_out_of_memory_error,
-	.fatal_error = _corosync_public_exit_error
+	.fatal_error = _corosync_public_exit_error,
+	.shutdown_request = corosync_shutdown_request,
+	.state_dump = corosync_state_dump,
+	.poll_handle_get = corosync_poll_handle_get
 };
 
 void apidef_init (struct objdb_iface_ver0 *objdb) {
