@@ -50,6 +50,7 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <sys/wait.h>
+#include <sys/stat.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -63,16 +64,18 @@
 #if defined(HAVE_GETPEERUCRED)
 #include <ucred.h>
 #endif
+#include <string.h>
 
 #include <sys/shm.h>
 #include <sys/sem.h>
 #include <corosync/corotypes.h>
 #include <corosync/list.h>
 
-#include "coroipcs.h"
-#include <corosync/ipc_gen.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/coroipcs.h>
+#include <corosync/coroipc_ipc.h>
 
-#ifdef COROSYNC_SOLARIS
+#ifndef MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
 #endif
 
@@ -91,6 +94,12 @@ struct outq_item {
 	struct list_head list;
 };
 
+struct zcb_mapped {
+	struct list_head list;
+	void *addr;
+	size_t size;
+};
+
 #if defined(_SEM_SEMUN_UNDEFINED)
 union semun {
 	int val;
@@ -119,20 +128,27 @@ struct conn_info {
 	int refcount;
 	key_t shmkey;
 	key_t semkey;
-	int shmid;
 	int semid;
 	unsigned int pending_semops;
 	pthread_mutex_t mutex;
-	struct shared_memory *mem;
+	struct control_buffer *control_buffer;
+	char *request_buffer;
+	char *response_buffer;
+	char *dispatch_buffer;
+	size_t control_size;
+	size_t request_size;
+	size_t response_size;
+	size_t dispatch_size;
 	struct list_head outq_head;
 	void *private_data;
 	struct list_head list;
 	char setup_msg[sizeof (mar_req_setup_t)];
 	unsigned int setup_bytes_read;
+	struct list_head zcb_mapped_list_head;
 	char *sending_allowed_private_data[64];
 };
 
-static int shared_mem_dispatch_bytes_left (struct conn_info *conn_info);
+static int shared_mem_dispatch_bytes_left (const struct conn_info *conn_info);
 
 static void outq_flush (struct conn_info *conn_info);
 
@@ -140,9 +156,186 @@ static int priv_change (struct conn_info *conn_info);
 
 static void ipc_disconnect (struct conn_info *conn_info);
 
-static void msg_send (void *conn, struct iovec *iov, int iov_len, int locked);
+static void msg_send (void *conn, const struct iovec *iov, unsigned int iov_len,
+		      int locked);
 
-static int memcpy_dwrap (struct conn_info *conn_info, void *msg, int len);
+static int
+memory_map (
+	const char *path,
+	size_t bytes,
+	void **buf)
+{
+	int fd;
+	void *addr_orig;
+	void *addr;
+	int res;
+
+	fd = open (path, O_RDWR, 0600);
+
+	unlink (path);
+
+	res = ftruncate (fd, bytes);
+
+	addr_orig = mmap (NULL, bytes, PROT_NONE,
+		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+
+	if (addr_orig == MAP_FAILED) {
+		return (-1);
+	}
+
+	addr = mmap (addr_orig, bytes, PROT_READ | PROT_WRITE,
+		MAP_FIXED | MAP_SHARED, fd, 0);
+
+	if (addr != addr_orig) {
+		return (-1);
+	}
+#ifdef COROSYNC_BSD
+	madvise(addr, bytes, MADV_NOSYNC);
+#endif
+
+	res = close (fd);
+	if (res) {
+		return (-1);
+	}
+	*buf = addr_orig;
+	return (0);
+}
+
+static int
+circular_memory_map (
+	const char *path,
+	size_t bytes,
+	void **buf)
+{
+	int fd;
+	void *addr_orig;
+	void *addr;
+	int res;
+
+	fd = open (path, O_RDWR, 0600);
+
+	unlink (path);
+
+	res = ftruncate (fd, bytes);
+
+	addr_orig = mmap (NULL, bytes << 1, PROT_NONE,
+		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+
+	if (addr_orig == MAP_FAILED) {
+		return (-1);
+	}
+
+	addr = mmap (addr_orig, bytes, PROT_READ | PROT_WRITE,
+		MAP_FIXED | MAP_SHARED, fd, 0);
+
+	if (addr != addr_orig) {
+		return (-1);
+	}
+#ifdef COROSYNC_BSD
+	madvise(addr_orig, bytes, MADV_NOSYNC);
+#endif
+
+	addr = mmap (((char *)addr_orig) + bytes,
+                  bytes, PROT_READ | PROT_WRITE,
+                  MAP_FIXED | MAP_SHARED, fd, 0);
+#ifdef COROSYNC_BSD
+	madvise(((char *)addr_orig) + bytes, bytes, MADV_NOSYNC);
+#endif
+
+	res = close (fd);
+	if (res) {
+		return (-1);
+	}
+	*buf = addr_orig;
+	return (0);
+}
+
+static inline int
+circular_memory_unmap (void *buf, size_t bytes)
+{
+	int res;
+
+	res = munmap (buf, bytes << 1);
+
+	return (res);
+}
+
+static inline int zcb_free (struct zcb_mapped *zcb_mapped)
+{
+	unsigned int res;
+
+	res = munmap (zcb_mapped->addr, zcb_mapped->size);
+	list_del (&zcb_mapped->list);
+	free (zcb_mapped);
+	return (res);
+}
+
+static inline int zcb_by_addr_free (struct conn_info *conn_info, void *addr)
+{
+	struct list_head *list;
+	struct zcb_mapped *zcb_mapped;
+	unsigned int res = 0;
+
+	for (list = conn_info->zcb_mapped_list_head.next;
+		list != &conn_info->zcb_mapped_list_head; list = list->next) {
+
+		zcb_mapped = list_entry (list, struct zcb_mapped, list);
+
+		if (zcb_mapped->addr == addr) {
+			res = zcb_free (zcb_mapped);
+			break;
+		}
+
+	}
+	return (res);
+}
+
+static inline int zcb_all_free (
+	struct conn_info *conn_info)
+{
+	struct list_head *list;
+	struct zcb_mapped *zcb_mapped;
+
+	for (list = conn_info->zcb_mapped_list_head.next;
+		list != &conn_info->zcb_mapped_list_head;) {
+
+		zcb_mapped = list_entry (list, struct zcb_mapped, list);
+
+		list = list->next;
+
+		zcb_free (zcb_mapped);
+	}
+	return (0);
+}
+
+static inline int zcb_alloc (
+	struct conn_info *conn_info,
+	const char *path_to_file,
+	size_t size,
+	void **addr)
+{
+	struct zcb_mapped *zcb_mapped;
+	unsigned int res;
+
+	zcb_mapped = malloc (sizeof (struct zcb_mapped));
+	if (zcb_mapped == NULL) {
+		return (-1);
+	}
+
+	res = memory_map (
+		path_to_file,
+		size,
+		addr);
+	if (res == -1) {
+		return (-1);
+	}
+
+	list_init (&zcb_mapped->list);
+	zcb_mapped->addr = *addr;
+	zcb_mapped->size = size;
+	list_add_tail (&zcb_mapped->list, &conn_info->zcb_mapped_list_head);
+	return (0);
+}
 
 static int ipc_thread_active (void *conn)
 {
@@ -229,8 +422,9 @@ static inline int conn_info_destroy (struct conn_info *conn_info)
 	/*
 	 * Destroy shared memory segment and semaphore
 	 */
-	shmdt (conn_info->mem);
-	res = shmctl (conn_info->shmid, IPC_RMID, NULL);
+	res = munmap ((void *)conn_info->control_buffer, conn_info->control_size);
+	res = munmap ((void *)conn_info->request_buffer, conn_info->request_size);
+	res = munmap ((void *)conn_info->response_buffer, conn_info->response_size);
 	semctl (conn_info->semid, 0, IPC_RMID);
 
 	/*
@@ -240,24 +434,106 @@ static inline int conn_info_destroy (struct conn_info *conn_info)
 		api->free (conn_info->private_data);
 	}
 	close (conn_info->fd);
+	res = circular_memory_unmap (conn_info->dispatch_buffer, conn_info->dispatch_size);
+	zcb_all_free (conn_info);
 	api->free (conn_info);
 	api->serialize_unlock ();
 	return (-1);
 }
 
-struct res_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char buf[4096];
+union u {
+	uint64_t server_addr;
+	void *server_ptr;
 };
 
+static uint64_t void2serveraddr (void *server_ptr)
+{
+	union u u;
+
+	u.server_ptr = server_ptr;
+	return (u.server_addr);
+}
+
+static void *serveraddr2void (uint64_t server_addr)
+{
+	union u u;
+
+	u.server_addr = server_addr;
+	return (u.server_ptr);
+};
+
+static inline void zerocopy_operations_process (
+	struct conn_info *conn_info,
+	coroipc_request_header_t **header_out,
+	unsigned int *new_message)
+{
+	coroipc_request_header_t *header;
+
+	header = (coroipc_request_header_t *)conn_info->request_buffer;
+	if (header->id == ZC_ALLOC_HEADER) {
+		mar_req_coroipcc_zc_alloc_t *hdr = (mar_req_coroipcc_zc_alloc_t *)header;
+		coroipc_response_header_t res_header;
+		void *addr = NULL;
+		struct coroipcs_zc_header *zc_header;
+		unsigned int res;
+
+		res = zcb_alloc (conn_info, hdr->path_to_file, hdr->map_size,
+			&addr);
+
+		zc_header = (struct coroipcs_zc_header *)addr;
+		zc_header->server_address = void2serveraddr(addr);
+
+		res_header.size = sizeof (coroipc_response_header_t);
+		res_header.id = 0;
+		coroipcs_response_send (
+			conn_info, &res_header,
+			res_header.size);
+		*new_message = 0;
+		return;
+	} else
+	if (header->id == ZC_FREE_HEADER) {
+		mar_req_coroipcc_zc_free_t *hdr = (mar_req_coroipcc_zc_free_t *)header;
+		coroipc_response_header_t res_header;
+		void *addr = NULL;
+
+		addr = serveraddr2void (hdr->server_address);
+
+		zcb_by_addr_free (conn_info, addr);
+
+		res_header.size = sizeof (coroipc_response_header_t);
+		res_header.id = 0;
+		coroipcs_response_send (
+			conn_info, &res_header,
+			res_header.size);
+
+		*new_message = 0;
+		return;
+	} else
+	if (header->id == ZC_EXECUTE_HEADER) {
+		mar_req_coroipcc_zc_execute_t *hdr = (mar_req_coroipcc_zc_execute_t *)header;
+
+		header = (coroipc_request_header_t *)(((char *)serveraddr2void(hdr->server_address) + sizeof (struct coroipcs_zc_header)));
+	}
+	*header_out = header;
+	*new_message = 1;
+}
+
 static void *pthread_ipc_consumer (void *conn)
 {
 	struct conn_info *conn_info = (struct conn_info *)conn;
 	struct sembuf sop;
 	int res;
-	mar_req_header_t *header;
-	struct res_overlay res_overlay;
+	coroipc_request_header_t *header;
+	coroipc_response_header_t coroipc_response_header;
 	int send_ok;
+	unsigned int new_message;
+
+#if defined(HAVE_PTHREAD_SETSCHEDPARAM) && defined(HAVE_SCHED_GET_PRIORITY_MAX)
+	if (api->sched_policy != 0) {
+		res = pthread_setschedparam (conn_info->thread,
+			api->sched_policy, api->sched_param);
+	}
+#endif
 
 	for (;;) {
 		sop.sem_num = 0;
@@ -277,15 +553,21 @@ retry_semop:
 			pthread_exit (0);
 		}
 
-		coroipcs_refcount_inc (conn_info);
+		zerocopy_operations_process (conn_info, &header, &new_message);
+		/*
+		 * There is no new message to process, continue for loop
+		 */
+		if (new_message == 0) {
+			continue;
+		}
 
-                header = (mar_req_header_t *)conn_info->mem->req_buffer;
+		coroipcs_refcount_inc (conn);
 
 		send_ok = api->sending_allowed (conn_info->service,
 			header->id,
-			header,	
+			header,
 			conn_info->sending_allowed_private_data);
-	
+
 		if (send_ok) {
 			api->serialize_lock();
 			api->handler_fn_get (conn_info->service, header->id) (conn_info, header);
@@ -294,13 +576,12 @@ retry_semop:
 			/*
 			 * Overload, tell library to retry
 			 */
-			res_overlay.header.size =
-				api->response_size_get (conn_info->service, header->id);
-			res_overlay.header.id =
-				api->response_id_get (conn_info->service, header->id);
-			res_overlay.header.error = CS_ERR_TRY_AGAIN;
-			coroipcs_response_send (conn_info, &res_overlay, 
-				res_overlay.header.size);
+			coroipc_response_header.size = sizeof (coroipc_response_header_t);
+			coroipc_response_header.id = 0;
+			coroipc_response_header.error = CS_ERR_TRY_AGAIN;
+			coroipcs_response_send (conn_info,
+				&coroipc_response_header,
+				sizeof (coroipc_response_header_t));
 		}
 
 		api->sending_allowed_release (conn_info->sending_allowed_private_data);
@@ -337,12 +618,14 @@ req_setup_recv (
 	int res;
 	struct msghdr msg_recv;
 	struct iovec iov_recv;
+	int authenticated = 0;
+
 #ifdef COROSYNC_LINUX
 	struct cmsghdr *cmsg;
 	char cmsg_cred[CMSG_SPACE (sizeof (struct ucred))];
-	struct ucred *cred;
 	int off = 0;
 	int on = 1;
+	struct ucred *cred;
 #endif
 
 	msg_recv.msg_iov = &iov_recv;
@@ -353,55 +636,14 @@ req_setup_recv (
 	msg_recv.msg_control = (void *)cmsg_cred;
 	msg_recv.msg_controllen = sizeof (cmsg_cred);
 #endif
-
-#ifdef PORTABILITY_WORK_TODO
 #ifdef COROSYNC_SOLARIS
-	msg_recv.msg_flags = 0;
-	uid_t euid;
-	gid_t egid;
-		                
-	euid = -1;
-	egid = -1;
-	if (getpeereid(conn_info->fd, &euid, &egid) != -1 &&
-	    (api->security_valid (euid, egid)) {
-		if (conn_info->state == CONN_IO_STATE_INITIALIZING) {
-			api->log_printf ("Invalid security authentication\n");
-			return (-1);
-		}
-	}
 	msg_recv.msg_accrights = 0;
 	msg_recv.msg_accrightslen = 0;
-#else /* COROSYNC_SOLARIS */
-
-#ifdef HAVE_GETPEERUCRED
-	ucred_t *uc;
-	uid_t euid = -1;
-	gid_t egid = -1;
-
-	if (getpeerucred (conn_info->fd, &uc) == 0) {
-		euid = ucred_geteuid (uc);
-		egid = ucred_getegid (uc);
-		if (api->security_valid (euid, egid) {
-			conn_info->authenticated = 1;
-		}
-		ucred_free(uc);
-	}
-	if (conn_info->authenticated == 0) {
-		api->log_printf ("Invalid security authentication\n");
- 	}
-#else /* HAVE_GETPEERUCRED */
- 	api->log_printf (LOG_LEVEL_SECURITY, "Connection not authenticated "
- 		"because platform does not support "
- 		"authentication with sockets, continuing "
- 		"with a fake authentication\n");
-#endif /* HAVE_GETPEERUCRED */
 #endif /* COROSYNC_SOLARIS */
 
-#endif
-
-#ifdef COROSYNC_LINUX
 	iov_recv.iov_base = &conn_info->setup_msg[conn_info->setup_bytes_read];
 	iov_recv.iov_len = sizeof (mar_req_setup_t) - conn_info->setup_bytes_read;
+#ifdef COROSYNC_LINUX
 	setsockopt(conn_info->fd, SOL_SOCKET, SO_PASSCRED, &on, sizeof (on));
 #endif
 
@@ -419,25 +661,77 @@ retry_recv:
 		 * EOF is detected when recvmsg return 0.
 		 */
 		ipc_disconnect (conn_info);
-#endif
+		return 0;
+#else
 		return (-1);
+#endif
 	}
 	conn_info->setup_bytes_read += res;
 
-#ifdef COROSYNC_LINUX
+/*
+ * currently support getpeerucred, getpeereid, and SO_PASSCRED credential
+ * retrieval mechanisms for various Platforms
+ */
+#ifdef HAVE_GETPEERUCRED
+/*
+ * Solaris and some BSD systems
+ */
+	{
+		ucred_t *uc = NULL;
+		uid_t euid = -1;
+		gid_t egid = -1;
 
+		if (getpeerucred (conn_info->fd, &uc) == 0) {
+			euid = ucred_geteuid (uc);
+			egid = ucred_getegid (uc);
+			if (api->security_valid (euid, egid)) {
+				authenticated = 1;
+			}
+			ucred_free(uc);
+		}
+	}
+#elif HAVE_GETPEEREID
+/*
+ * Usually MacOSX systems
+ */
+
+	{
+		uid_t euid;
+		gid_t egid;
+
+		euid = -1;
+		egid = -1;
+		if (getpeereid (conn_info->fd, &euid, &egid) == 0) {
+			if (api->security_valid (euid, egid)) {
+				authenticated = 1;
+			}
+		}
+	}
+
+#elif SO_PASSCRED
+/*
+ * Usually Linux systems
+ */
 	cmsg = CMSG_FIRSTHDR (&msg_recv);
 	assert (cmsg);
 	cred = (struct ucred *)CMSG_DATA (cmsg);
 	if (cred) {
 		if (api->security_valid (cred->uid, cred->gid)) {
-		} else {
-			ipc_disconnect (conn_info);
-			api->log_printf ("Invalid security authentication\n");
-			return (-1);
+			authenticated = 1;
 		}
 	}
-#endif
+
+#else /* no credentials */
+	authenticated = 1;
+ 	api->log_printf ("Platform does not support IPC authentication.  Using no authentication\n");
+#endif /* no credentials */
+
+	if (authenticated == 0) {
+		api->log_printf ("Invalid IPC credentials.\n");
+		ipc_disconnect (conn_info);
+		return (-1);
+ 	}
+
 	if (conn_info->setup_bytes_read == sizeof (mar_req_setup_t)) {
 #ifdef COROSYNC_LINUX
 		setsockopt(conn_info->fd, SOL_SOCKET, SO_PASSCRED,
@@ -479,6 +773,7 @@ static int conn_info_create (int fd)
 	conn_info->state = CONN_STATE_THREAD_INACTIVE;
 	list_init (&conn_info->outq_head);
 	list_init (&conn_info->list);
+	list_init (&conn_info->zcb_mapped_list_head);
 	list_add (&conn_info->list, &conn_info_list_head);
 
         api->poll_dispatch_add (fd, conn_info);
@@ -510,7 +805,11 @@ extern void coroipcs_ipc_init (
 	/*
 	 * Create socket for IPC clients, name socket, listen for connections
 	 */
+#if defined(COROSYNC_SOLARIS)
 	server_fd = socket (PF_UNIX, SOCK_STREAM, 0);
+#else
+	server_fd = socket (PF_LOCAL, SOCK_STREAM, 0);
+#endif
 	if (server_fd == -1) {
 		api->log_printf ("Cannot create client connections socket.\n");
 		api->fatal_error ("Can't create library listen socket");
@@ -525,21 +824,37 @@ extern void coroipcs_ipc_init (
 	memset (&un_addr, 0, sizeof (struct sockaddr_un));
 	un_addr.sun_family = AF_UNIX;
 #if defined(COROSYNC_BSD) || defined(COROSYNC_DARWIN)
-	un_addr.sun_len = sizeof(struct sockaddr_un);
+	un_addr.sun_len = SUN_LEN(&un_addr);
 #endif
 
 #if defined(COROSYNC_LINUX)
 	sprintf (un_addr.sun_path + 1, "%s", api->socket_name);
 #else
-	sprintf (un_addr.sun_path, "%s%s", SOCKETDIR, api->socket_name);
-	unlink (un_addr.sun_path);
+	{
+		struct stat stat_out;
+		res = stat (SOCKETDIR, &stat_out);
+		if (res == -1 || (res == 0 && !S_ISDIR(stat_out.st_mode))) {
+			api->log_printf ("Required directory not present %s\n", SOCKETDIR);
+			api->fatal_error ("Please create required directory.");
+		}
+		sprintf (un_addr.sun_path, "%s/%s", SOCKETDIR, api->socket_name);
+		unlink (un_addr.sun_path);
+	}
 #endif
 
 	res = bind (server_fd, (struct sockaddr *)&un_addr, COROSYNC_SUN_LEN(&un_addr));
 	if (res) {
-		api->log_printf ("Could not bind AF_UNIX: %s.\n", strerror (errno));
+		api->log_printf ("Could not bind AF_UNIX (%s): %s.\n", un_addr.sun_path, strerror (errno));
 		api->fatal_error ("Could not bind to AF_UNIX socket\n");
 	}
+
+	/*
+	 * Allow eveyrone to write to the socket since the IPC layer handles
+	 * security automatically
+	 */
+#if !defined(COROSYNC_LINUX)
+	res = chmod (un_addr.sun_path, S_IRWXU|S_IRWXG|S_IRWXO);
+#endif
 	listen (server_fd, SERVER_BACKLOG);
 
         /*
@@ -552,16 +867,27 @@ void coroipcs_ipc_exit (void)
 {
 	struct list_head *list;
 	struct conn_info *conn_info;
+	unsigned int res;
 
 	for (list = conn_info_list_head.next; list != &conn_info_list_head;
 		list = list->next) {
 
 		conn_info = list_entry (list, struct conn_info, list);
 
-		shmdt (conn_info->mem);
-		shmctl (conn_info->shmid, IPC_RMID, NULL);
+		/*
+		 * Unmap memory segments
+		 */
+		res = munmap ((void *)conn_info->control_buffer,
+			conn_info->control_size);
+		res = munmap ((void *)conn_info->request_buffer,
+			conn_info->request_size);
+		res = munmap ((void *)conn_info->response_buffer,
+			conn_info->response_size);
+		res = circular_memory_unmap (conn_info->dispatch_buffer,
+			conn_info->dispatch_size);
+
 		semctl (conn_info->semid, 0, IPC_RMID);
-	
+
 		pthread_kill (conn_info->thread, SIGUSR1);
 	}
 }
@@ -576,13 +902,13 @@ void *coroipcs_private_data_get (void *conn)
 	return (conn_info->private_data);
 }
 
-int coroipcs_response_send (void *conn, void *msg, int mlen)
+int coroipcs_response_send (void *conn, const void *msg, size_t mlen)
 {
 	struct conn_info *conn_info = (struct conn_info *)conn;
 	struct sembuf sop;
 	int res;
 
-	memcpy (conn_info->mem->res_buffer, msg, mlen);
+	memcpy (conn_info->response_buffer, msg, mlen);
 	sop.sem_num = 1;
 	sop.sem_op = 1;
 	sop.sem_flg = 0;
@@ -598,7 +924,7 @@ retry_semop:
 	return (0);
 }
 
-int coroipcs_response_iov_send (void *conn, struct iovec *iov, int iov_len)
+int coroipcs_response_iov_send (void *conn, const struct iovec *iov, unsigned int iov_len)
 {
 	struct conn_info *conn_info = (struct conn_info *)conn;
 	struct sembuf sop;
@@ -607,7 +933,8 @@ int coroipcs_response_iov_send (void *conn, struct iovec *iov, int iov_len)
 	int i;
 
 	for (i = 0; i < iov_len; i++) {
-		memcpy (&conn_info->mem->res_buffer[write_idx], iov[i].iov_base, iov[i].iov_len);
+		memcpy (&conn_info->response_buffer[write_idx],
+			iov[i].iov_base, iov[i].iov_len);
 		write_idx += iov[i].iov_len;
 	}
 
@@ -626,45 +953,35 @@ retry_semop:
 	return (0);
 }
 
-static int shared_mem_dispatch_bytes_left (struct conn_info *conn_info)
+static int shared_mem_dispatch_bytes_left (const struct conn_info *conn_info)
 {
-	unsigned int read;
-	unsigned int write;
+	unsigned int n_read;
+	unsigned int n_write;
 	unsigned int bytes_left;
 
-	read = conn_info->mem->read;
-	write = conn_info->mem->write;
+	n_read = conn_info->control_buffer->read;
+	n_write = conn_info->control_buffer->write;
 
-	if (read <= write) {
-		bytes_left = DISPATCH_SIZE - write + read;
+	if (n_read <= n_write) {
+		bytes_left = conn_info->dispatch_size - n_write + n_read;
 	} else {
-		bytes_left = read - write;
+		bytes_left = n_read - n_write;
 	}
 	return (bytes_left);
 }
 
-static int memcpy_dwrap (struct conn_info *conn_info, void *msg, int len)
+static void memcpy_dwrap (struct conn_info *conn_info, void *msg, unsigned int len)
 {
-	char *dest_char = (char *)conn_info->mem->dispatch_buffer;
-	char *src_char = (char *)msg;
-	unsigned int first_write;
-	unsigned int second_write;
+	unsigned int write_idx;
 
-	first_write = len;
-	second_write = 0;
-	if (len + conn_info->mem->write >= DISPATCH_SIZE) {
-		first_write = DISPATCH_SIZE - conn_info->mem->write;
-		second_write = len - first_write;
-	}
-	memcpy (&dest_char[conn_info->mem->write], src_char, first_write);
-	if (second_write) {
-		memcpy (dest_char, &src_char[first_write], second_write);
-	}
-	conn_info->mem->write = (conn_info->mem->write + len) % DISPATCH_SIZE;
-	return (0);
+	write_idx = conn_info->control_buffer->write;
+
+	memcpy (&conn_info->dispatch_buffer[write_idx], msg, len);
+	conn_info->control_buffer->write = (write_idx + len) % conn_info->dispatch_size;
 }
 
-static void msg_send (void *conn, struct iovec *iov, int iov_len, int locked)
+static void msg_send (void *conn, const struct iovec *iov, unsigned int iov_len,
+		      int locked)
 {
 	struct conn_info *conn_info = (struct conn_info *)conn;
 	struct sembuf sop;
@@ -785,7 +1102,7 @@ retry_recv:
 	return (0);
 }
 
-static void msg_send_or_queue (void *conn, struct iovec *iov, int iov_len)
+static void msg_send_or_queue (void *conn, const struct iovec *iov, unsigned int iov_len)
 {
 	struct conn_info *conn_info = (struct conn_info *)conn;
 	unsigned int bytes_left;
@@ -856,18 +1173,18 @@ void coroipcs_refcount_dec (void *conn)
 	pthread_mutex_unlock (&conn_info->mutex);
 }
 
-int coroipcs_dispatch_send (void *conn, void *msg, int mlen)
+int coroipcs_dispatch_send (void *conn, const void *msg, size_t mlen)
 {
 	struct iovec iov;
 
-	iov.iov_base = msg;
+	iov.iov_base = (void *)msg;
 	iov.iov_len = mlen;
 
 	msg_send_or_queue (conn, &iov, 1);
 	return (0);
 }
 
-int coroipcs_dispatch_iov_send (void *conn, struct iovec *iov, int iov_len)
+int coroipcs_dispatch_iov_send (void *conn, const struct iovec *iov, unsigned int iov_len)
 {
 	msg_send_or_queue (conn, iov, iov_len);
 	return (0);
@@ -954,12 +1271,12 @@ int coroipcs_handler_dispatch (
 	if (conn_info->service == SOCKET_SERVICE_INIT && (revent & POLLIN)) {
 		/*
 		 * Receive in a nonblocking fashion the request
-		 * IF security invalid, send TRY_AGAIN, otherwise
+		 * IF security invalid, send ERR_SECURITY, otherwise
 		 * send OK
 		 */
 		res = req_setup_recv (conn_info);
 		if (res == -1) {
-			req_setup_send (conn_info, CS_ERR_TRY_AGAIN);
+			req_setup_send (conn_info, CS_ERR_SECURITY);
 		}
 		if (res != 1) {
 			return (0);
@@ -976,23 +1293,43 @@ int coroipcs_handler_dispatch (
 			return (0);
 		}
 
-		conn_info->shmkey = req_setup->shmkey;
 		conn_info->semkey = req_setup->semkey;
+		res = memory_map (
+			req_setup->control_file,
+			req_setup->control_size,
+			(void *)&conn_info->control_buffer);
+		conn_info->control_size = req_setup->control_size;
+
+		res = memory_map (
+			req_setup->request_file,
+			req_setup->request_size,
+			(void *)&conn_info->request_buffer);
+		conn_info->request_size = req_setup->request_size;
+
+		res = memory_map (
+			req_setup->response_file,
+			req_setup->response_size,
+			(void *)&conn_info->response_buffer);
+		conn_info->response_size = req_setup->response_size;
+
+		res = circular_memory_map (
+			req_setup->dispatch_file,
+			req_setup->dispatch_size,
+			(void *)&conn_info->dispatch_buffer);
+		conn_info->dispatch_size = req_setup->dispatch_size;
+
 		conn_info->service = req_setup->service;
 		conn_info->refcount = 0;
 		conn_info->notify_flow_control_enabled = 0;
 		conn_info->setup_bytes_read = 0;
 
-		conn_info->shmid = shmget (conn_info->shmkey,
-			sizeof (struct shared_memory), 0600);
-		conn_info->mem = shmat (conn_info->shmid, NULL, 0);
 		conn_info->semid = semget (conn_info->semkey, 3, 0600);
 		conn_info->pending_semops = 0;
 
 		/*
 		 * ipc thread is the only reference at startup
 		 */
-		conn_info->refcount = 1; 
+		conn_info->refcount = 1;
 		conn_info->state = CONN_STATE_THREAD_ACTIVE;
 
 		conn_info->private_data = api->malloc (api->private_data_size_get (conn_info->service));
@@ -1042,7 +1379,6 @@ int coroipcs_handler_dispatch (
 				res = 0;
 				break;
 			}
-			coroipcs_refcount_dec (conn_info);
 		}
 #if defined(COROSYNC_SOLARIS) || defined(COROSYNC_BSD) || defined(COROSYNC_DARWIN)
 		/* On many OS poll never return POLLHUP or POLLERR.
@@ -1050,9 +1386,11 @@ int coroipcs_handler_dispatch (
 		 */
 		if (res == 0) {
 			ipc_disconnect (conn_info);
+			coroipcs_refcount_dec (conn_info);
 			return (0);
 		}
 #endif
+		coroipcs_refcount_dec (conn_info);
 	}
 
 	coroipcs_refcount_inc (conn_info);
@@ -1086,4 +1424,3 @@ int coroipcs_handler_dispatch (
 
 	return (0);
 }
-
