@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006 Red Hat, Inc.
+ * Copyright (c) 2006, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -37,6 +37,7 @@
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -45,23 +46,24 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
-#include <signal.h>
 #include <string.h>
+#include <dirent.h>
+#include <limits.h>
 
 #include <corosync/lcr/lcr_comp.h>
 #include <corosync/engine/objdb.h>
 #include <corosync/engine/config.h>
 
-#include "mempool.h"
 #include "util.h"
 
 static int read_config_file_into_objdb(
 	struct objdb_iface_ver0 *objdb,
-	char **error_string);
+	const char **error_string);
 static char error_string_response[512];
 
 
-static int aisparser_readconfig (struct objdb_iface_ver0 *objdb, char **error_string)
+static int aisparser_readconfig (struct objdb_iface_ver0 *objdb,
+				 const char **error_string)
 {
 	if (read_config_file_into_objdb(objdb, error_string)) {
 		return -1;
@@ -84,18 +86,32 @@ static char *remove_whitespace(char *string)
 	return start;
 }
 
+#define PCHECK_ADD_SUBSECTION 1
+#define PCHECK_ADD_ITEM       2
+
+typedef int (*parser_check_item_f)(struct objdb_iface_ver0 *objdb,
+				hdb_handle_t parent_handle,
+				int type,
+				const char *name,
+				const char **error_string);
+
 static int parse_section(FILE *fp,
 			 struct objdb_iface_ver0 *objdb,
 			 hdb_handle_t parent_handle,
-			 char **error_string)
+			 const char **error_string,
+			 parser_check_item_f parser_check_item_call)
 {
 	char line[512];
 	int i;
 	char *loc;
 
-	while (fgets (line, 255, fp)) {
-		if (strlen(line) > 0)
-			line[strlen(line) - 1] = '\0';
+	while (fgets (line, sizeof (line), fp)) {
+		if (strlen(line) > 0) {
+			if (line[strlen(line) - 1] == '\n')
+				line[strlen(line) - 1] = '\0';
+			if (strlen (line) > 0 && line[strlen(line) - 1] == '\r')
+				line[strlen(line) - 1] = '\0';
+		}
 		/*
 		 * Clear out white space and tabs
 		 */
@@ -114,32 +130,43 @@ static int parse_section(FILE *fp,
 		}
 
 		/* New section ? */
-		if ((loc = strstr_rs (line, "{"))) {
+		if ((loc = strchr_rs (line, '{'))) {
 			hdb_handle_t new_parent;
 			char *section = remove_whitespace(line);
 
 			loc--;
 			*loc = '\0';
+			if (parser_check_item_call) {
+				if (!parser_check_item_call(objdb, parent_handle, PCHECK_ADD_SUBSECTION,
+				    section, error_string))
+					    return -1;
+			}
+
 			objdb->object_create (parent_handle, &new_parent,
 					      section, strlen (section));
-			if (parse_section(fp, objdb, new_parent, error_string))
+			if (parse_section(fp, objdb, new_parent, error_string, parser_check_item_call))
 				return -1;
 		}
 
 		/* New key/value */
-		if ((loc = strstr_rs (line, ":"))) {
+		if ((loc = strchr_rs (line, ':'))) {
 			char *key;
 			char *value;
 
 			*(loc-1) = '\0';
 			key = remove_whitespace(line);
 			value = remove_whitespace(loc);
+			if (parser_check_item_call) {
+				if (!parser_check_item_call(objdb, parent_handle, PCHECK_ADD_ITEM,
+				    key, error_string))
+					    return -1;
+			}
 			objdb->object_key_create (parent_handle, key,
 				strlen (key),
 				value, strlen (value) + 1);
 		}
 
-		if ((loc = strstr_rs (line, "}"))) {
+		if ((loc = strchr_rs (line, '}'))) {
 			return 0;
 		}
 	}
@@ -152,36 +179,112 @@ static int parse_section(FILE *fp,
 	return 0;
 }
 
+static int parser_check_item_uidgid(struct objdb_iface_ver0 *objdb,
+			hdb_handle_t parent_handle,
+			int type,
+			const char *name,
+			const char **error_string)
+{
+	if (type == PCHECK_ADD_SUBSECTION) {
+		if (parent_handle != OBJECT_PARENT_HANDLE) {
+			*error_string = "uidgid: Can't add second level subsection";
+			return 0;
+		}
 
+		if (strcmp (name, "uidgid") != 0) {
+			*error_string = "uidgid: Can't add subsection different then uidgid";
+			return 0;
+		}
+	}
+
+	if (type == PCHECK_ADD_ITEM) {
+		if (!(strcmp (name, "uid") == 0 || strcmp (name, "gid") == 0)) {
+			*error_string = "uidgid: Only uid and gid are allowed items";
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
+static int read_uidgid_files_into_objdb(
+	struct objdb_iface_ver0 *objdb,
+	const char **error_string)
+{
+	FILE *fp;
+	const char *dirname;
+	DIR *dp;
+	struct dirent *dirent;
+	char filename[PATH_MAX + FILENAME_MAX + 1];
+	int res = 0;
+	struct stat stat_buf;
+
+	dirname = COROSYSCONFDIR "/uidgid.d";
+	dp = opendir (dirname);
+
+	if (dp == NULL)
+		return 0;
+
+	while ((dirent = readdir (dp))) {
+		snprintf(filename, sizeof (filename), "%s/%s", dirname, dirent->d_name);
+		stat (filename, &stat_buf);
+		if (S_ISREG(stat_buf.st_mode)) {
+
+			fp = fopen (filename, "r");
+			if (fp == NULL) continue;
+
+			res = parse_section(fp, objdb, OBJECT_PARENT_HANDLE, error_string, parser_check_item_uidgid);
+
+			fclose (fp);
+
+			if (res != 0) {
+				goto error_exit;
+			}
+		}
+	}
+
+error_exit:
+	closedir(dp);
+
+	return res;
+}
 
 /* Read config file and load into objdb */
 static int read_config_file_into_objdb(
 	struct objdb_iface_ver0 *objdb,
-	char **error_string)
+	const char **error_string)
 {
 	FILE *fp;
-	char *filename;
+	const char *filename;
 	char *error_reason = error_string_response;
 	int res;
 
 	filename = getenv ("COROSYNC_MAIN_CONFIG_FILE");
 	if (!filename)
-		filename = SYSCONFDIR "/corosync.conf";
+		filename = COROSYSCONFDIR "/corosync.conf";
 
 	fp = fopen (filename, "r");
-	if (fp == 0) {
-		sprintf (error_reason, "Can't read file %s reason = (%s)\n",
+	if (fp == NULL) {
+		snprintf (error_reason, sizeof(error_string_response),
+			"Can't read file %s reason = (%s)\n",
 			 filename, strerror (errno));
 		*error_string = error_reason;
 		return -1;
 	}
 
-	res = parse_section(fp, objdb, OBJECT_PARENT_HANDLE, error_string);
+	res = parse_section(fp, objdb, OBJECT_PARENT_HANDLE, error_string, NULL);
 
 	fclose(fp);
 
-	sprintf (error_reason, "Successfully read main configuration file '%s'.\n", filename);
-	*error_string = error_reason;
+	if (res == 0) {
+	        res = read_uidgid_files_into_objdb(objdb, error_string);
+	}
+
+	if (res == 0) {
+		snprintf (error_reason, sizeof(error_string_response),
+			"Successfully read main configuration file '%s'.\n", filename);
+		*error_string = error_reason;
+	}
 
 	return res;
 }
@@ -215,10 +318,13 @@ struct lcr_comp aisparser_comp_ver0 = {
 	.ifaces					= corosync_aisparser_ver0
 };
 
+#ifdef COROSYNC_SOLARIS
+void corosync_lcr_component_register (void);
 
-__attribute__ ((constructor)) static void aisparser_comp_register (void) {
+void corosync_lcr_component_register (void) {
+#else
+__attribute__ ((constructor)) static void corosync_lcr_component_register (void) {
+#endif
         lcr_interfaces_set (&corosync_aisparser_ver0[0], &aisparser_iface_ver0);
 	lcr_component_register (&aisparser_comp_ver0);
 }
-
-
