@@ -1,13 +1,13 @@
 /*
  * Copyright (c) 2003-2004 MontaVista Software, Inc.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -47,7 +47,7 @@
 #include <corosync/list.h>
 #include "tlist.h"
 
-typedef int (*dispatch_fn_t) (hdb_handle_t hdb_handle_t, int fd, int revents, void *data);
+typedef int (*dispatch_fn_t) (hdb_handle_t hdb_handle, int fd, int revents, void *data);
 
 struct poll_entry {
 	struct pollfd ufd;
@@ -63,14 +63,7 @@ struct poll_instance {
 	int stop_requested;
 };
 
-/*
- * All instances in one database
- */
-static struct hdb_handle_database poll_instance_database = {
-	.handle_count	= 0,
-	.handles	= 0,
-	.iterator	= 0
-};
+DECLARE_HDB_DATABASE (poll_instance_database,NULL);
 
 hdb_handle_t poll_create (void)
 {
@@ -88,7 +81,7 @@ hdb_handle_t poll_create (void)
 	if (res != 0) {
 		goto error_destroy;
 	}
-	
+
 	poll_instance->poll_entries = 0;
 	poll_instance->ufds = 0;
 	poll_instance->poll_entry_count = 0;
@@ -99,7 +92,7 @@ hdb_handle_t poll_create (void)
 
 error_destroy:
 	hdb_handle_destroy (&poll_instance_database, handle);
-	
+
 error_exit:
 	return (-1);
 }
@@ -116,12 +109,8 @@ int poll_destroy (hdb_handle_t handle)
 		goto error_exit;
 	}
 
-	if (poll_instance->poll_entries) {
-		free (poll_instance->poll_entries);
-	}
-	if (poll_instance->ufds) {
-		free (poll_instance->ufds);
-	}
+	free (poll_instance->poll_entries);
+	free (poll_instance->ufds);
 
 	hdb_handle_destroy (&poll_instance_database, handle);
 
@@ -175,7 +164,7 @@ int poll_dispatch_add (
 			goto error_put;
 		}
 		poll_instance->poll_entries = poll_entries;
-	
+
 		ufds = (struct pollfd *)realloc (poll_instance->ufds,
 			(poll_instance->poll_entry_count + 1) *
 			sizeof (struct pollfd));
@@ -188,7 +177,7 @@ int poll_dispatch_add (
 		poll_instance->poll_entry_count += 1;
 		install_pos = poll_instance->poll_entry_count - 1;
 	}
-	
+
 	/*
 	 * Install new dispatch handler
 	 */
@@ -233,7 +222,7 @@ int poll_dispatch_modify (
 		if (poll_instance->poll_entries[i].ufd.fd == fd) {
 			poll_instance->poll_entries[i].ufd.events = events;
 			poll_instance->poll_entries[i].dispatch_fn = dispatch_fn;
-			
+
 			goto error_put;
 		}
 	}
@@ -312,12 +301,12 @@ error_exit:
 
 int poll_timer_delete (
 	hdb_handle_t handle,
-	poll_timer_handle timer_handle)
+	poll_timer_handle th)
 {
 	struct poll_instance *poll_instance;
 	int res = 0;
 
-	if (timer_handle == 0) {
+	if (th == 0) {
 		return (0);
 	}
 	res = hdb_handle_get (&poll_instance_database, handle,
@@ -327,7 +316,7 @@ int poll_timer_delete (
 		goto error_exit;
 	}
 
-	timerlist_del (&poll_instance->timerlist, (void *)timer_handle);
+	timerlist_del (&poll_instance->timerlist, (void *)th);
 
 	hdb_handle_put (&poll_instance_database, handle);
 
@@ -347,7 +336,7 @@ int poll_stop (
 		res = -ENOENT;
 		goto error_exit;
 	}
-	
+
 	poll_instance->stop_requested = 1;
 
 	hdb_handle_put (&poll_instance_database, handle);
@@ -387,7 +376,6 @@ retry_poll:
 		res = poll (poll_instance->ufds,
 			poll_instance->poll_entry_count, expire_timeout_msec);
 		if (poll_instance->stop_requested) {
-			printf ("poll should stop\n");
 			return (0);
 		}
 		if (errno == EINTR && res == -1) {
@@ -403,7 +391,7 @@ retry_poll:
 				poll_instance->ufds[i].revents) {
 
 				res = poll_instance->poll_entries[i].dispatch_fn (handle,
-					poll_instance->ufds[i].fd, 
+					poll_instance->ufds[i].fd,
 					poll_instance->ufds[i].revents,
 					poll_instance->poll_entries[i].data);
 
@@ -446,5 +434,5 @@ void poll_print_state (
 		}
 	}
 }
-	
+
 #endif
