@@ -45,6 +45,7 @@
 #include <sys/un.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <sys/stat.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -58,41 +59,49 @@
 
 #include <corosync/swab.h>
 #include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/corodefs.h>
 #include <corosync/list.h>
-#include <corosync/queue.h>
 #include <corosync/lcr/lcr_ifact.h>
 #include <corosync/totem/coropoll.h>
 #include <corosync/totem/totempg.h>
 #include <corosync/engine/objdb.h>
 #include <corosync/engine/config.h>
 #include <corosync/engine/logsys.h>
+#include <corosync/coroipcs.h>
 
 #include "quorum.h"
 #include "totemsrp.h"
-#include "mempool.h"
 #include "mainconfig.h"
 #include "totemconfig.h"
 #include "main.h"
 #include "sync.h"
+#include "syncv2.h"
 #include "tlist.h"
-#include "coroipcs.h"
 #include "timer.h"
 #include "util.h"
 #include "apidef.h"
 #include "service.h"
+#include "schedwrk.h"
 #include "version.h"
+#include "evil.h"
 
 LOGSYS_DECLARE_SYSTEM ("corosync",
-	LOG_MODE_OUTPUT_STDERR | LOG_MODE_THREADED | LOG_MODE_FORK,
+	LOGSYS_MODE_OUTPUT_STDERR | LOGSYS_MODE_THREADED | LOGSYS_MODE_FORK,
+	0,
 	NULL,
+	LOG_INFO,
 	LOG_DAEMON,
+	LOG_INFO,
 	NULL,
 	1000000);
 
-LOGSYS_DECLARE_SUBSYS ("MAIN", LOG_INFO);
+LOGSYS_DECLARE_SUBSYS ("MAIN");
 
 #define SERVER_BACKLOG 5
 
+static int sched_priority = 0;
+
 static unsigned int service_count = 32;
 
 #if defined(HAVE_PTHREAD_SPIN_LOCK)
@@ -103,8 +112,6 @@ static pthread_mutex_t serialize_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static struct totem_logging_configuration totem_logging_configuration;
 
-static char delivery_data[MESSAGE_SIZE_MAX];
-
 static int num_config_modules;
 
 static struct config_iface_ver0 *config_modules[MAX_DYNAMIC_SERVICES];
@@ -113,71 +120,71 @@ static struct objdb_iface_ver0 *objdb = NULL;
 
 static struct corosync_api_v1 *api = NULL;
 
-static struct main_config main_config;
+static enum cs_sync_mode minimum_sync_mode;
 
-unsigned long long *(*main_clm_get_by_nodeid) (unsigned int node_id);
+static enum cs_sync_mode minimum_sync_mode;
 
-hdb_handle_t corosync_poll_handle;
+static int sync_in_process = 1;
 
-static void sigusr2_handler (int num)
+static hdb_handle_t corosync_poll_handle;
+
+struct sched_param global_sched_param;
+
+hdb_handle_t corosync_poll_handle_get (void)
+{
+	return (corosync_poll_handle);
+}
+
+void corosync_state_dump (void)
 {
 	int i;
 
-	for (i = 0; ais_service[i]; i++) {
-		if (ais_service[i]->exec_dump_fn) {
+	for (i = 0; i < SERVICE_HANDLER_MAXIMUM_COUNT; i++) {
+		if (ais_service[i] && ais_service[i]->exec_dump_fn) {
 			ais_service[i]->exec_dump_fn ();
 		}
 	}
 }
 
-static void *corosync_exit (void *arg)
+static void sigusr2_handler (int num)
+{
+	/*
+	 * TODO remove this from sigusr2 handler and access via cfg service
+	 * engine api - corosync-cfgtool
+	 */
+	corosync_state_dump ();
+}
+
+/*
+ * TODO this function needs some love
+ */
+void corosync_shutdown_request (void)
 {
 	if (api) {
 		corosync_service_unlink_all (api);
 	}
 
-#ifdef DEBUG_MEMPOOL
-	int stats_inuse[MEMPOOL_GROUP_SIZE];
-	int stats_avail[MEMPOOL_GROUP_SIZE];
-	int stats_memoryused[MEMPOOL_GROUP_SIZE];
-	int i;
-
-	mempool_getstats (stats_inuse, stats_avail, stats_memoryused);
-	log_printf (LOG_LEVEL_DEBUG, "Memory pools:\n");
-	for (i = 0; i < MEMPOOL_GROUP_SIZE; i++) {
-	log_printf (LOG_LEVEL_DEBUG, "order %d size %d inuse %d avail %d memory used %d\n",
-		i, 1<<i, stats_inuse[i], stats_avail[i], stats_memoryused[i]);
-	}
-#endif
-
 	poll_stop (0);
 	totempg_finalize ();
 	coroipcs_ipc_exit ();
+
 	corosync_exit_error (AIS_DONE_EXIT);
-
-	/* never reached */
-	return NULL;
 }
 
-pthread_t corosync_exit_thread;
-static void init_shutdown(void *data) 
-{
-	pthread_create (&corosync_exit_thread, NULL, corosync_exit, NULL);
-}
-
-
-static poll_timer_handle shutdown_handle;
 static void sigquit_handler (int num)
 {
-	/* avoid creating threads from within the interrupt context */
-	poll_timer_add (corosync_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
+	corosync_shutdown_request ();
 }
 
+static void sigintr_handler (int num)
+{
+	corosync_shutdown_request ();
+}
 
 static void sigsegv_handler (int num)
 {
 	(void)signal (SIGSEGV, SIG_DFL);
-	logsys_atsegv();
+	logsys_atexit();
 	logsys_log_rec_store (LOCALSTATEDIR "/lib/corosync/fdata");
 	raise (SIGSEGV);
 }
@@ -185,78 +192,74 @@ static void sigsegv_handler (int num)
 static void sigabrt_handler (int num)
 {
 	(void)signal (SIGABRT, SIG_DFL);
-	logsys_atsegv();
+	logsys_atexit();
 	logsys_log_rec_store (LOCALSTATEDIR "/lib/corosync/fdata");
 	raise (SIGABRT);
 }
 
 #define LOCALHOST_IP inet_addr("127.0.0.1")
 
-hdb_handle_t corosync_group_handle;
+static hdb_handle_t corosync_group_handle;
 
-struct totempg_group corosync_group = {
+static struct totempg_group corosync_group = {
 	.group		= "a",
 	.group_len	= 1
 };
 
-void sigintr_handler (int signum)
-{
-	poll_timer_add (corosync_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
-}
 
 
-static int pool_sizes[] = { 0, 0, 0, 0, 0, 4096, 0, 1, 0, /* 256 */
-					1024, 0, 1, 4096, 0, 0, 0, 0, /* 65536 */
-					1, 1, 1, 1, 1, 1, 1, 1, 1 };
-
 #if defined(HAVE_PTHREAD_SPIN_LOCK)
-void serialize_lock (void)
+static void serialize_lock (void)
 {
 	pthread_spin_lock (&serialize_spin);
 }
 
-void serialize_unlock (void)
+static void serialize_unlock (void)
 {
 	pthread_spin_unlock (&serialize_spin);
 }
 #else
-void serialize_lock (void)
+static void serialize_lock (void)
 {
 	pthread_mutex_lock (&serialize_mutex);
 }
 
-void serialize_unlock (void)
+static void serialize_unlock (void)
 {
 	pthread_mutex_unlock (&serialize_mutex);
 }
 #endif
 
-
-
 static void corosync_sync_completed (void)
 {
+	log_printf (LOGSYS_LEVEL_NOTICE,
+		"Completed service synchronization, ready to provide service.\n");
+	sync_in_process = 0;
 }
 
 static int corosync_sync_callbacks_retrieve (int sync_id,
 	struct sync_callbacks *callbacks)
 {
 	unsigned int ais_service_index;
-	unsigned int ais_services_found = 0;
-	
+	int res;
+
 	for (ais_service_index = 0;
 		ais_service_index < SERVICE_HANDLER_MAXIMUM_COUNT;
 		ais_service_index++) {
 
-		if (ais_service[ais_service_index] != NULL) {
-			if (ais_services_found == sync_id) {
+		if (ais_service[ais_service_index] != NULL
+			&& ais_service[ais_service_index]->sync_mode == CS_SYNC_V1) {
+			if (ais_service_index == sync_id) {
 				break;
 			}
-			ais_services_found += 1;
 		}
 	}
+	/*
+	 * Try to load backwards compat sync engines
+	 */
 	if (ais_service_index == SERVICE_HANDLER_MAXIMUM_COUNT) {
-		memset (callbacks, 0, sizeof (struct sync_callbacks));
-		return (-1);
+		res = evil_callbacks_load (sync_id, callbacks);
+		return (res);
 	}
 	callbacks->name = ais_service[ais_service_index]->name;
 	callbacks->sync_init = ais_service[ais_service_index]->sync_init;
@@ -266,17 +269,51 @@ static int corosync_sync_callbacks_retrieve (int sync_id,
 	return (0);
 }
 
+static int corosync_sync_v2_callbacks_retrieve (
+	int service_id,
+	struct sync_callbacks *callbacks)
+{
+	int res;
+
+	if (minimum_sync_mode == CS_SYNC_V2 && service_id == CLM_SERVICE && ais_service[CLM_SERVICE] == NULL) {
+		res = evil_callbacks_load (service_id, callbacks);
+		return (res);
+	}
+	if (minimum_sync_mode == CS_SYNC_V2 && service_id == EVT_SERVICE && ais_service[EVT_SERVICE] == NULL) {
+		res = evil_callbacks_load (service_id, callbacks);
+		return (res);
+	}
+	if (ais_service[service_id] == NULL) {
+		return (-1);
+	}
+	if (minimum_sync_mode == CS_SYNC_V1 && ais_service[service_id]->sync_mode != CS_SYNC_V2) {
+		return (-1);
+	}
+
+	callbacks->name = ais_service[service_id]->name;
+	callbacks->sync_init = ais_service[service_id]->sync_init;
+	callbacks->sync_process = ais_service[service_id]->sync_process;
+	callbacks->sync_activate = ais_service[service_id]->sync_activate;
+	callbacks->sync_abort = ais_service[service_id]->sync_abort;
+	return (0);
+}
+
 static struct memb_ring_id corosync_ring_id;
 
 static void confchg_fn (
 	enum totem_configuration_type configuration_type,
-	unsigned int *member_list, int member_list_entries,
-	unsigned int *left_list, int left_list_entries,
-	unsigned int *joined_list, int joined_list_entries,
-	struct memb_ring_id *ring_id)
+	const unsigned int *member_list, size_t member_list_entries,
+	const unsigned int *left_list, size_t left_list_entries,
+	const unsigned int *joined_list, size_t joined_list_entries,
+	const struct memb_ring_id *ring_id)
 {
 	int i;
+	int abort_activate = 0;
 
+	if (sync_in_process == 1) {
+		abort_activate = 1;
+	}
+	sync_in_process = 1;
 	serialize_lock ();
 	memcpy (&corosync_ring_id, ring_id, sizeof (struct memb_ring_id));
 
@@ -292,24 +329,18 @@ static void confchg_fn (
 		}
 	}
 	serialize_unlock ();
+
+	if (abort_activate) {
+		sync_v2_abort ();
+	}
+	if (minimum_sync_mode == CS_SYNC_V2 && configuration_type == TOTEM_CONFIGURATION_REGULAR) {
+		sync_v2_start (member_list, member_list_entries, ring_id);
+	}
 }
 
 static void priv_drop (void)
 {
-return; /* TODO: we are still not dropping privs */
-	setuid (main_config.uid);
-	setegid (main_config.gid);
-}
-
-static void corosync_mempool_init (void)
-{
-	int res;
-
-	res = mempool_init (pool_sizes);
-	if (res == ENOMEM) {
-		log_printf (LOG_LEVEL_ERROR, "Couldn't allocate memory pools, not enough memory");
-		corosync_exit_error (AIS_DONE_MEMPOOL_INIT);
-	}
+	return; /* TODO: we are still not dropping privs */
 }
 
 static void corosync_tty_detach (void)
@@ -342,7 +373,7 @@ static void corosync_tty_detach (void)
 	/* Create new session */
 	(void)setsid();
 
-	/* 
+	/*
 	 * Map stdin/out/err to /dev/null.
 	 */
 	fd = open("/dev/null", O_RDWR);
@@ -353,32 +384,11 @@ static void corosync_tty_detach (void)
 		dup2(fd, STDERR_FILENO); /* 2 */
 
 		/* Should be 0, but just in case it isn't... */
-		if (fd > 2) 
+		if (fd > 2)
 			close(fd);
 	}
 }
 
-static void corosync_setscheduler (void)
-{
-#if ! defined(TS_CLASS) && (defined(COROSYNC_BSD) || defined(COROSYNC_LINUX) || defined(COROSYNC_SOLARIS))
-	struct sched_param sched_param;
-	int res;
-
-	res = sched_get_priority_max (SCHED_RR);
-	if (res != -1) {
-		sched_param.sched_priority = 1;//res;
-		res = sched_setscheduler (0, SCHED_RR, &sched_param);
-		if (res == -1) {
-			log_printf (LOG_LEVEL_WARNING, "Could not set SCHED_RR at priority %d: %s\n",
-				sched_param.sched_priority, strerror (errno));
-		}
-	} else
-		log_printf (LOG_LEVEL_WARNING, "Could not get maximum scheduler priority: %s\n", strerror (errno));
-#else
-	log_printf(LOG_LEVEL_WARNING, "Scheduler priority left to default value (no OS support)\n");
-#endif
-}
-
 static void corosync_mlockall (void)
 {
 #if !defined(COROSYNC_BSD)
@@ -398,67 +408,62 @@ static void corosync_mlockall (void)
 	/* under FreeBSD a process with locked page cannot call dlopen
 	 * code disabled until FreeBSD bug i386/93396 was solved
 	 */
-	log_printf (LOG_LEVEL_WARNING, "Could not lock memory of service to avoid page faults\n");
+	log_printf (LOGSYS_LEVEL_WARNING, "Could not lock memory of service to avoid page faults\n");
 #else
 	res = mlockall (MCL_CURRENT | MCL_FUTURE);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_WARNING, "Could not lock memory of service to avoid page faults: %s\n", strerror (errno));
+		log_printf (LOGSYS_LEVEL_WARNING, "Could not lock memory of service to avoid page faults: %s\n", strerror (errno));
 	};
 #endif
 }
 
 static void deliver_fn (
 	unsigned int nodeid,
-	struct iovec *iovec,
-	int iov_len,
+	const void *msg,
+	unsigned int msg_len,
 	int endian_conversion_required)
 {
-	mar_req_header_t *header;
-	int pos = 0;
-	int i;
+	const coroipc_request_header_t *header;
 	int service;
 	int fn_id;
+	unsigned int id;
+	unsigned int size;
 
-	/*
-	 * Build buffer without iovecs to make processing easier
-	 * This is only used for messages which are multicast with iovecs
-	 * and self-delivered.  All other mechanisms avoid the copy.
-	 */
-	if (iov_len > 1) {
-		for (i = 0; i < iov_len; i++) {
-			memcpy (&delivery_data[pos], iovec[i].iov_base, iovec[i].iov_len);
-			pos += iovec[i].iov_len;
-			assert (pos < MESSAGE_SIZE_MAX);
-		}
-		header = (mar_req_header_t *)delivery_data;
-	} else {
-		header = (mar_req_header_t *)iovec[0].iov_base;
-	}
+	header = msg;
 	if (endian_conversion_required) {
-		header->id = swab32 (header->id);
-		header->size = swab32 (header->size);
+		id = swab32 (header->id);
+		size = swab32 (header->size);
+	} else {
+		id = header->id;
+		size = header->size;
 	}
 
-//	assert(iovec->iov_len == header->size);
-
 	/*
 	 * Call the proper executive handler
 	 */
-	service = header->id >> 16;
-	fn_id = header->id & 0xffff;
-	if (!ais_service[service])
-		return;
+	service = id >> 16;
+	fn_id = id & 0xffff;
 
 	serialize_lock();
 
+	if (ais_service[service] == NULL && service == EVT_SERVICE) {
+		evil_deliver_fn (nodeid, service, fn_id, msg,
+			endian_conversion_required);
+	}
+
+	if (!ais_service[service]) {
+		serialize_unlock();
+		return;
+	}
+
 	if (endian_conversion_required) {
 		assert(ais_service[service]->exec_engine[fn_id].exec_endian_convert_fn != NULL);
 		ais_service[service]->exec_engine[fn_id].exec_endian_convert_fn
-			(header);
+			((void *)msg);
 	}
 
 	ais_service[service]->exec_engine[fn_id].exec_handler_fn
-		(header, nodeid);
+		(msg, nodeid);
 
 	serialize_unlock();
 }
@@ -470,14 +475,14 @@ void main_get_config_modules(struct config_iface_ver0 ***modules, int *num)
 }
 
 int main_mcast (
-        struct iovec *iovec,
-        int iov_len,
+        const struct iovec *iovec,
+        unsigned int iov_len,
         unsigned int guarantee)
 {
 	return (totempg_groups_mcast_joined (corosync_group_handle, iovec, iov_len, guarantee));
 }
 
-int message_source_is_local (mar_message_source_t *source)
+int message_source_is_local (const mar_message_source_t *source)
 {
 	int ret = 0;
 
@@ -524,46 +529,43 @@ static coroipcs_handler_fn_lvalue corosync_handler_fn_get (unsigned int service,
 
 static int corosync_security_valid (int euid, int egid)
 {
+	struct list_head *iter;
 	if (euid == 0 || egid == 0) {
 		return (1);
 	}
-	if (euid == main_config.uid || egid == main_config.gid) {
-		return (1);
+
+	for (iter = uidgid_list_head.next; iter != &uidgid_list_head;
+		iter = iter->next) {
+
+		struct uidgid_item *ugi = list_entry (iter, struct uidgid_item,
+			list);
+
+		if (euid == ugi->uid || egid == ugi->gid)
+			return (1);
 	}
+
 	return (0);
 }
 
 static int corosync_service_available (unsigned int service)
 {
-	return (ais_service[service]);
+	return (ais_service[service] != NULL);
 }
 
-static int corosync_response_size_get (unsigned int service, unsigned int id)
-{
-	return (ais_service[service]->lib_engine[id].response_size);
-
-}
-
-static int corosync_response_id_get (unsigned int service, unsigned int id)
-{
-	return (ais_service[service]->lib_engine[id].response_id);
-}
-
-
 struct sending_allowed_private_data_struct {
 	int reserved_msgs;
 };
 
 static int corosync_sending_allowed (
 	unsigned int service,
-	unsigned int id, 
-	void *msg,
+	unsigned int id,
+	const void *msg,
 	void *sending_allowed_private_data)
 {
 	struct sending_allowed_private_data_struct *pd =
 		(struct sending_allowed_private_data_struct *)sending_allowed_private_data;
 	struct iovec reserve_iovec;
-	mar_req_header_t *header = (mar_req_header_t *)msg;
+	coroipc_request_header_t *header = (coroipc_request_header_t *)msg;
 	int sending_allowed;
 
 	reserve_iovec.iov_base = (char *)header;
@@ -579,7 +581,7 @@ static int corosync_sending_allowed (
 		((ais_service[service]->lib_engine[id].flow_control == CS_LIB_FLOW_CONTROL_NOT_REQUIRED) ||
 		((ais_service[service]->lib_engine[id].flow_control == CS_LIB_FLOW_CONTROL_REQUIRED) &&
 		(pd->reserved_msgs) &&
-		(sync_in_process() == 0)));
+		(sync_in_process == 0)));
 
 	return (sending_allowed);
 }
@@ -594,19 +596,34 @@ static void corosync_sending_allowed_release (void *sending_allowed_private_data
 
 static int ipc_subsys_id = -1;
 
-static void ipc_log_printf (char *format, ...) {
-        va_list ap;
+static void ipc_log_printf (const char *format, ...) __attribute__((format(printf, 1, 2)));
+static void ipc_log_printf (const char *format, ...) {
+	va_list ap;
 
-        va_start (ap, format);
-	
-       _logsys_log_printf (ipc_subsys_id, __FUNCTION__,	
-                __FILE__, __LINE__, LOG_LEVEL_ERROR, format, ap);
+	va_start (ap, format);
 
-        va_end (ap);
+	_logsys_log_vprintf (
+		LOGSYS_ENCODE_RECID(LOGSYS_LEVEL_ERROR,
+				    ipc_subsys_id,
+				    LOGSYS_RECID_LOG),
+		__FUNCTION__, __FILE__, __LINE__,
+		format, ap);
+
+	va_end (ap);
+}
+
+static void ipc_fatal_error(const char *error_msg) {
+       _logsys_log_printf (
+		LOGSYS_ENCODE_RECID(LOGSYS_LEVEL_ERROR,
+				    ipc_subsys_id,
+				    LOGSYS_RECID_LOG),
+		__FUNCTION__, __FILE__, __LINE__,
+                "%s", error_msg);
+	exit(EXIT_FAILURE);
 }
 
 static int corosync_poll_handler_accept (
-	hdb_handle_t handle,	
+	hdb_handle_t handle,
 	int fd,
 	int revent,
 	void *context)
@@ -615,7 +632,7 @@ static int corosync_poll_handler_accept (
 }
 
 static int corosync_poll_handler_dispatch (
-	hdb_handle_t handle,	
+	hdb_handle_t handle,
 	int fd,
 	int revent,
 	void *context)
@@ -647,11 +664,14 @@ static void corosync_poll_dispatch_modify (
 		corosync_poll_handler_dispatch);
 }
 
-struct coroipcs_init_state ipc_init_state = {
-	.socket_name			= IPC_SOCKET_NAME,
+static struct coroipcs_init_state ipc_init_state = {
+	.socket_name			= COROSYNC_SOCKET_NAME,
+	.sched_policy			= SCHED_OTHER,
+	.sched_param			= &global_sched_param,
 	.malloc				= malloc,
 	.free				= free,
 	.log_printf			= ipc_log_printf,
+	.fatal_error			= ipc_fatal_error,
 	.security_valid			= corosync_security_valid,
 	.service_available		= corosync_service_available,
 	.private_data_size_get		= corosync_private_data_size_get,
@@ -659,8 +679,6 @@ struct coroipcs_init_state ipc_init_state = {
 	.serialize_unlock		= serialize_unlock,
 	.sending_allowed		= corosync_sending_allowed,
 	.sending_allowed_release	= corosync_sending_allowed_release,
-	.response_size_get		= corosync_response_size_get,
-	.response_id_get		= corosync_response_id_get,
 	.poll_accept_add		= corosync_poll_accept_add,
 	.poll_dispatch_add		= corosync_poll_dispatch_add,
 	.poll_dispatch_modify		= corosync_poll_dispatch_modify,
@@ -669,6 +687,35 @@ struct coroipcs_init_state ipc_init_state = {
 	.handler_fn_get			= corosync_handler_fn_get
 };
 
+static void corosync_setscheduler (void)
+{
+#if defined(HAVE_PTHREAD_SETSCHEDPARAM) && defined(HAVE_SCHED_GET_PRIORITY_MAX)
+	int res;
+
+	sched_priority = sched_get_priority_max (SCHED_RR);
+	if (sched_priority != -1) {
+		global_sched_param.sched_priority = sched_priority;
+		res = sched_setscheduler (0, SCHED_RR, &global_sched_param);
+		if (res == -1) {
+			global_sched_param.sched_priority = 0;
+			log_printf (LOGSYS_LEVEL_WARNING, "Could not set SCHED_RR at priority %d: %s\n",
+				global_sched_param.sched_priority, strerror (errno));
+		} else {
+			/*
+			 * Turn on SCHED_RR in ipc system
+			 */
+			ipc_init_state.sched_policy = SCHED_RR;
+		}
+	} else {
+		log_printf (LOGSYS_LEVEL_WARNING, "Could not get maximum scheduler priority: %s\n", strerror (errno));
+		sched_priority = 0;
+	}
+#else
+	log_printf(LOGSYS_LEVEL_WARNING,
+		"The Platform is missing process priority setting features.  Leaving at default.");
+#endif
+}
+
 int main (int argc, char **argv)
 {
 	const char *error_string;
@@ -679,10 +726,13 @@ int main (int argc, char **argv)
 	void *objdb_p;
 	struct config_iface_ver0 *config;
 	void *config_p;
-	const char *config_iface;
+	const char *config_iface_init;
+	char *config_iface;
 	char *iface;
 	int res, ch;
 	int background, setprio;
+	struct stat stat_out;
+	char corosync_lib_dir[PATH_MAX];
 
 #if defined(HAVE_PTHREAD_SPIN_LOCK)
 	pthread_spin_init (&serialize_spin, 0);
@@ -692,13 +742,13 @@ int main (int argc, char **argv)
 	 */
 	background = 1;
 	setprio = 1;
- 	
+
  	while ((ch = getopt (argc, argv, "fp")) != EOF) {
- 	
+
 		switch (ch) {
 			case 'f':
 				background = 0;
-				logsys_config_mode_set (LOG_MODE_OUTPUT_STDERR|LOG_MODE_THREADED|LOG_MODE_FORK);
+				logsys_config_mode_set (NULL, LOGSYS_MODE_OUTPUT_STDERR|LOGSYS_MODE_THREADED|LOGSYS_MODE_FORK);
 				break;
 			case 'p':
 				setprio = 0;
@@ -710,26 +760,37 @@ int main (int argc, char **argv)
 					"        -p     : Do not set process priority.    \n");
 				return EXIT_FAILURE;
 		}
-	}	
+	}
 
 	if (background)
 		corosync_tty_detach ();
 
-	log_printf (LOG_LEVEL_NOTICE, "Corosync Executive Service RELEASE '%s'\n", RELEASE_VERSION);
-	log_printf (LOG_LEVEL_NOTICE, "Copyright (C) 2002-2006 MontaVista Software, Inc and contributors.\n");
-	log_printf (LOG_LEVEL_NOTICE, "Copyright (C) 2006-2008 Red Hat, Inc.\n");
+	/*
+	 * Set round robin realtime scheduling with priority 99
+	 * Lock all memory to avoid page faults which may interrupt
+	 * application healthchecking
+	 */
+	if (setprio) {
+		corosync_setscheduler ();
+	}
+
+	corosync_mlockall ();
+
+	log_printf (LOGSYS_LEVEL_NOTICE, "Corosync Cluster Engine ('%s'): started and ready to provide service.\n", RELEASE_VERSION);
 
 	(void)signal (SIGINT, sigintr_handler);
 	(void)signal (SIGUSR2, sigusr2_handler);
 	(void)signal (SIGSEGV, sigsegv_handler);
 	(void)signal (SIGABRT, sigabrt_handler);
 	(void)signal (SIGQUIT, sigquit_handler);
-	
+#if MSG_NOSIGNAL == 0
+	(void)signal (SIGPIPE, SIG_IGN);
+#endif
+
 	corosync_timer_init (
 		serialize_lock,
-		serialize_unlock);
-
-	log_printf (LOG_LEVEL_NOTICE, "Corosync Executive Service: started and ready to provide service.\n");
+		serialize_unlock,
+		sched_priority);
 
 	corosync_poll_handle = poll_create ();
 
@@ -743,7 +804,7 @@ int main (int argc, char **argv)
 		&objdb_p,
 		0);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "Corosync Executive couldn't open configuration object database component.\n");
+		log_printf (LOGSYS_LEVEL_ERROR, "Corosync Executive couldn't open configuration object database component.\n");
 		corosync_exit_error (AIS_DONE_OBJDB);
 	}
 
@@ -764,13 +825,16 @@ int main (int argc, char **argv)
 	 * the corosync default built in parser if the configuration parser
 	 * isn't overridden
 	 */
-	config_iface = getenv("COROSYNC_DEFAULT_CONFIG_IFACE");
-	if (!config_iface) {
-		config_iface = "corosync_parser";
+	config_iface_init = getenv("COROSYNC_DEFAULT_CONFIG_IFACE");
+	if (!config_iface_init) {
+		config_iface_init = "corosync_parser";
 	}
 
 	/* Make a copy so we can deface it with strtok */
-	config_iface = strdup(config_iface);
+	if ((config_iface = strdup(config_iface_init)) == NULL) {
+		log_printf (LOGSYS_LEVEL_ERROR, "exhausted virtual memory");
+		corosync_exit_error (AIS_DONE_OBJDB);
+	}
 
 	iface = strtok(config_iface, ":");
 	while (iface)
@@ -784,67 +848,109 @@ int main (int argc, char **argv)
 
 		config = (struct config_iface_ver0 *)config_p;
 		if (res == -1) {
-			log_printf (LOG_LEVEL_ERROR, "Corosync Executive couldn't open configuration component '%s'\n", iface);
+			log_printf (LOGSYS_LEVEL_ERROR, "Corosync Executive couldn't open configuration component '%s'\n", iface);
 			corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 		}
 
 		res = config->config_readconfig(objdb, &error_string);
 		if (res == -1) {
-			log_printf (LOG_LEVEL_ERROR, "%s", error_string);
+			log_printf (LOGSYS_LEVEL_ERROR, "%s", error_string);
 			corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 		}
-		log_printf (LOG_LEVEL_NOTICE, "%s", error_string);
+		log_printf (LOGSYS_LEVEL_NOTICE, "%s", error_string);
 		config_modules[num_config_modules++] = config;
 
 		iface = strtok(NULL, ":");
 	}
-	if (config_iface)
-		free(config_iface);
+	free(config_iface);
 
-	res = corosync_main_config_read (objdb, &error_string, &main_config);
+	res = corosync_main_config_read (objdb, &error_string);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
+		/*
+		 * if we are here, we _must_ flush the logsys queue
+		 * and try to inform that we couldn't read the config.
+		 * this is a desperate attempt before certain death
+		 * and there is no guarantee that we can print to stderr
+		 * nor that logsys is sending the messages where we expect.
+		 */
+		log_printf (LOGSYS_LEVEL_ERROR, "%s", error_string);
+		fprintf(stderr, "%s", error_string);
+		syslog (LOGSYS_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
+	logsys_fork_completed();
+	if (setprio) {
+		res = logsys_thread_priority_set (SCHED_RR, &global_sched_param, 10);
+		if (res == -1) {
+			log_printf (LOGSYS_LEVEL_ERROR,
+				"Could not set logsys thread priority.  Can't continue because of priority inversions.");
+			corosync_exit_error (AIS_DONE_LOGSETUP);
+		}
+	} else {
+		res = logsys_thread_priority_set (SCHED_OTHER, NULL, 1);
+	}
+
+	/*
+	 * Make sure required directory is present
+	 */
+	sprintf (corosync_lib_dir, "%s/lib/corosync", LOCALSTATEDIR);
+	res = stat (corosync_lib_dir, &stat_out);
+	if ((res == -1) || (res == 0 && !S_ISDIR(stat_out.st_mode))) {
+		log_printf (LOGSYS_LEVEL_ERROR, "Required directory not present %s.  Please create it.\n", corosync_lib_dir);
+		corosync_exit_error (AIS_DONE_DIR_NOT_PRESENT);
+	}
 
 	res = totem_config_read (objdb, &totem_config, &error_string);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
+		log_printf (LOGSYS_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
 	res = totem_config_keyread (objdb, &totem_config, &error_string);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
+		log_printf (LOGSYS_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
 	res = totem_config_validate (&totem_config, &error_string);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "%s", error_string);
+		log_printf (LOGSYS_LEVEL_ERROR, "%s", error_string);
 		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
-	/*
-	 * Set round robin realtime scheduling with priority 99
-	 * Lock all memory to avoid page faults which may interrupt
-	 * application healthchecking
-	 */
-	if (setprio)
-		corosync_setscheduler ();
-
-	corosync_mlockall ();
-
 	totem_config.totem_logging_configuration = totem_logging_configuration;
 	totem_config.totem_logging_configuration.log_subsys_id =
-		_logsys_subsys_create ("TOTEM", LOG_INFO);
-  	totem_config.totem_logging_configuration.log_level_security = LOG_LEVEL_SECURITY;
-	totem_config.totem_logging_configuration.log_level_error = LOG_LEVEL_ERROR;
-	totem_config.totem_logging_configuration.log_level_warning = LOG_LEVEL_WARNING;
-	totem_config.totem_logging_configuration.log_level_notice = LOG_LEVEL_NOTICE;
-	totem_config.totem_logging_configuration.log_level_debug = LOG_LEVEL_DEBUG;
+		_logsys_subsys_create ("TOTEM");
+
+	if (totem_config.totem_logging_configuration.log_subsys_id < 0) {
+		log_printf (LOGSYS_LEVEL_ERROR,
+			"Unable to initialize TOTEM logging subsystem\n");
+		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
+	}
+
+  	totem_config.totem_logging_configuration.log_level_security = LOGSYS_LEVEL_WARNING;
+	totem_config.totem_logging_configuration.log_level_error = LOGSYS_LEVEL_ERROR;
+	totem_config.totem_logging_configuration.log_level_warning = LOGSYS_LEVEL_WARNING;
+	totem_config.totem_logging_configuration.log_level_notice = LOGSYS_LEVEL_NOTICE;
+	totem_config.totem_logging_configuration.log_level_debug = LOGSYS_LEVEL_DEBUG;
 	totem_config.totem_logging_configuration.log_printf = _logsys_log_printf;
 
+	res = corosync_main_config_compatibility_read (objdb,
+		&minimum_sync_mode,
+		&error_string);
+	if (res == -1) {
+		log_printf (LOGSYS_LEVEL_ERROR, "%s", error_string);
+		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
+	}
+
+	res = corosync_main_config_compatibility_read (objdb,
+		&minimum_sync_mode,
+		&error_string);
+	if (res == -1) {
+		log_printf (LOGSYS_LEVEL_ERROR, "%s", error_string);
+		corosync_exit_error (AIS_DONE_MAINCONFIGREAD);
+	}
+
 	/*
 	 * Sleep for a while to let other nodes in the cluster
 	 * understand that this node has been away (if it was
@@ -882,12 +988,30 @@ int main (int argc, char **argv)
 	 */
 	res = corosync_service_defaults_link_and_init (api);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "Could not initialize default services\n");
+		log_printf (LOGSYS_LEVEL_ERROR, "Could not initialize default services\n");
 		corosync_exit_error (AIS_DONE_INIT_SERVICES);
 	}
+	evil_init (api);
 
+	if (minimum_sync_mode == CS_SYNC_V2) {
+		log_printf (LOGSYS_LEVEL_NOTICE, "Compatibility mode set to none.  Using V2 of the synchronization engine.\n");
+		sync_v2_init (
+			corosync_sync_v2_callbacks_retrieve,
+			corosync_sync_completed);
+	} else
+	if (minimum_sync_mode == CS_SYNC_V1) {
+		log_printf (LOGSYS_LEVEL_NOTICE, "Compatibility mode set to whitetank.  Using V1 and V2 of the synchronization engine.\n");
+		sync_register (
+			corosync_sync_callbacks_retrieve,
+			sync_v2_memb_list_determine,
+			sync_v2_memb_list_abort,
+			sync_v2_start);
+
+		sync_v2_init (
+			corosync_sync_v2_callbacks_retrieve,
+			corosync_sync_completed);
+	}
 
-	sync_register (corosync_sync_callbacks_retrieve, corosync_sync_completed);
 
 	/*
 	 * Drop root privleges to user 'ais'
@@ -899,9 +1023,16 @@ int main (int argc, char **argv)
 	 */
 	priv_drop ();
 
-	corosync_mempool_init ();
+ 	schedwrk_init (
+ 		serialize_lock,
+ 		serialize_unlock);
 
-	ipc_subsys_id = _logsys_subsys_create ("IPC", LOG_INFO);
+	ipc_subsys_id = _logsys_subsys_create ("IPC");
+	if (ipc_subsys_id < 0) {
+		log_printf (LOGSYS_LEVEL_ERROR,
+			"Could not initialize IPC logging subsystem\n");
+		corosync_exit_error (AIS_DONE_INIT_SERVICES);
+	}
 
 	coroipcs_ipc_init (&ipc_init_state);
 
