@@ -43,14 +43,16 @@
 #include <corosync/lcr/lcr_comp.h>
 #include <corosync/engine/objdb.h>
 #include <corosync/engine/config.h>
+#include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
 
 #include "main.h"
 
 struct object_key {
 	void *key_name;
-	int key_len;
+	size_t key_len;
 	void *value;
-	int value_len;
+	size_t value_len;
 	struct list_head list;
 };
 
@@ -68,7 +70,7 @@ struct object_tracker {
 
 struct object_instance {
 	void *object_name;
-	int object_name_len;
+	size_t object_name_len;
 	hdb_handle_t object_handle;
 	hdb_handle_t parent_handle;
 	struct list_head key_head;
@@ -89,7 +91,7 @@ struct object_find_instance {
 	struct list_head *find_child_list;
 	struct list_head *child_head;
 	void *object_name;
-	int object_len;
+	size_t object_len;
 };
 
 struct objdb_iface_ver0 objdb_iface;
@@ -98,22 +100,11 @@ static pthread_rwlock_t reload_lock;
 static pthread_t lock_thread;
 static pthread_mutex_t meta_lock;
 
-static struct hdb_handle_database object_instance_database = {
-	.handle_count	= 0,
-	.handles	= 0,
-	.iterator	= 0,
-	.mutex		= PTHREAD_MUTEX_INITIALIZER
-};
+DECLARE_HDB_DATABASE (object_instance_database,NULL);
 
-static struct hdb_handle_database object_find_instance_database = {
-	.handle_count	= 0,
-	.handles	= 0,
-	.iterator	= 0,
-	.mutex		= PTHREAD_MUTEX_INITIALIZER
-};
+DECLARE_HDB_DATABASE (object_find_instance_database,NULL);
 
-
-static void objdb_wrlock()
+static void objdb_wrlock(void)
 {
 	pthread_mutex_lock(&meta_lock);
 	pthread_rwlock_wrlock(&reload_lock);
@@ -121,7 +112,7 @@ static void objdb_wrlock()
 	pthread_mutex_unlock(&meta_lock);
 }
 
-static void objdb_rdlock()
+static void objdb_rdlock(void)
 {
 	pthread_mutex_lock(&meta_lock);
 	if (lock_thread != pthread_self())
@@ -129,7 +120,7 @@ static void objdb_rdlock()
 	pthread_mutex_unlock(&meta_lock);
 }
 
-static void objdb_rdunlock()
+static void objdb_rdunlock(void)
 {
 	pthread_mutex_lock(&meta_lock);
 	if (lock_thread != pthread_self())
@@ -137,7 +128,7 @@ static void objdb_rdunlock()
 	pthread_mutex_unlock(&meta_lock);
 }
 
-static void objdb_wrunlock()
+static void objdb_wrunlock(void)
 {
 	pthread_mutex_lock(&meta_lock);
 	pthread_rwlock_unlock(&reload_lock);
@@ -162,7 +153,7 @@ static int objdb_init (void)
 		goto error_destroy;
 	}
 	instance->find_child_list = &instance->child_head;
-	instance->object_name = "parent";
+	instance->object_name = (char *)"parent";
 	instance->object_name_len = strlen ("parent");
 	instance->object_handle = handle;
 	instance->parent_handle = OBJECT_PARENT_HANDLE;
@@ -225,7 +216,7 @@ static int _object_notify_deleted_children(struct object_instance *parent_pt)
 static void object_created_notification(
 	hdb_handle_t object_handle,
 	hdb_handle_t parent_object_handle,
-	void *name_pt, int name_len)
+	const void *name_pt, size_t name_len)
 {
 	struct list_head * list;
 	struct object_instance * obj_pt;
@@ -258,7 +249,7 @@ static void object_created_notification(
 
 static void object_pre_deletion_notification(hdb_handle_t object_handle,
 	hdb_handle_t parent_object_handle,
-	void *name_pt, int name_len)
+	const void *name_pt, size_t name_len)
 	{
 	struct list_head * list;
 	struct object_instance * obj_pt;
@@ -294,8 +285,8 @@ static void object_pre_deletion_notification(hdb_handle_t object_handle,
 }
 
 static void object_key_changed_notification(hdb_handle_t object_handle,
-	void *name_pt,	int name_len,
-	void *value_pt, int value_len,
+	const void *name_pt, size_t name_len,
+	const void *value_pt, size_t value_len,
 	object_change_type_t type)
 {
 	struct list_head * list;
@@ -363,8 +354,8 @@ static void object_reload_notification(int startstop, int flush)
 static int object_create (
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t *object_handle,
-	void *object_name,
-	unsigned int object_name_len)
+	const void *object_name,
+	size_t object_name_len)
 {
 	struct object_instance *object_instance;
 	struct object_instance *parent_instance;
@@ -490,10 +481,10 @@ error_exit:
 
 static int object_key_create (
 	hdb_handle_t object_handle,
-	void *key_name,
-	int key_len,
-	void *value,
-	int value_len)
+	const void *key_name,
+	size_t key_len,
+	const void *value,
+	size_t value_len)
 {
 	struct object_instance *instance;
 	struct object_key *object_key;
@@ -582,7 +573,7 @@ static int object_key_create (
 	object_key->value_len = value_len;
 
 	object_key_changed_notification(object_handle, key_name, key_len,
-								value, value_len, OBJECT_KEY_CREATED);
+					value, value_len, OBJECT_KEY_CREATED);
 	objdb_rdunlock();
 	return (0);
 
@@ -673,7 +664,7 @@ static int object_destroy (
 static int object_valid_set (
 	hdb_handle_t object_handle,
 	struct object_valid *object_valid_list,
-	unsigned int object_valid_list_entries)
+	size_t object_valid_list_entries)
 {
 	struct object_instance *instance;
 	unsigned int res;
@@ -701,7 +692,7 @@ error_exit:
 static int object_key_valid_set (
 		hdb_handle_t object_handle,
 		struct object_key_valid *object_key_valid_list,
-		unsigned int object_key_valid_list_entries)
+		size_t object_key_valid_list_entries)
 {
 	struct object_instance *instance;
 	unsigned int res;
@@ -731,8 +722,8 @@ error_exit:
  */
 static int object_find_create (
 	hdb_handle_t object_handle,
-	void *object_name,
-	int object_len,
+	const void *object_name,
+	size_t object_len,
 	hdb_handle_t *object_find_handle)
 {
 	unsigned int res;
@@ -759,7 +750,7 @@ static int object_find_create (
 
 	object_find_instance->find_child_list = &object_instance->child_head;
 	object_find_instance->child_head = &object_instance->child_head;
-	object_find_instance->object_name = object_name;
+	object_find_instance->object_name = (char *)object_name;
 	object_find_instance->object_len = object_len;
 
 	hdb_handle_put (&object_instance_database, object_handle);
@@ -853,10 +844,10 @@ error_exit:
 
 static int object_key_get (
 	hdb_handle_t object_handle,
-	void *key_name,
-	int key_len,
+	const void *key_name,
+	size_t key_len,
 	void **value,
-	int *value_len)
+	size_t *value_len)
 {
 	unsigned int res = 0;
 	struct object_instance *instance;
@@ -902,8 +893,8 @@ error_exit:
 
 static int object_key_increment (
 	hdb_handle_t object_handle,
-	void *key_name,
-	int key_len,
+	const void *key_name,
+	size_t key_len,
 	unsigned int *value)
 {
 	unsigned int res = 0;
@@ -948,8 +939,8 @@ error_exit:
 
 static int object_key_decrement (
 	hdb_handle_t object_handle,
-	void *key_name,
-	int key_len,
+	const void *key_name,
+	size_t key_len,
 	unsigned int *value)
 {
 	unsigned int res = 0;
@@ -994,8 +985,8 @@ error_exit:
 
 static int object_key_delete (
 	hdb_handle_t object_handle,
-	void *key_name,
-	int key_len)
+	const void *key_name,
+	size_t key_len)
 {
 	unsigned int res;
 	int ret = 0;
@@ -1046,10 +1037,10 @@ error_exit:
 
 static int object_key_replace (
 	hdb_handle_t object_handle,
-	void *key_name,
-	int key_len,
-	void *new_value,
-	int new_value_len)
+	const void *key_name,
+	size_t key_len,
+	const void *new_value,
+	size_t new_value_len)
 {
 	unsigned int res;
 	int ret = 0;
@@ -1248,9 +1239,9 @@ error_exit:
 
 static int object_key_iter(hdb_handle_t parent_object_handle,
 			   void **key_name,
-			   int *key_len,
+			   size_t *key_len,
 			   void **value,
-			   int *value_len)
+			   size_t *value_len)
 {
 	unsigned int res;
 	struct object_instance *instance;
@@ -1297,9 +1288,9 @@ error_exit:
 static int object_key_iter_from(hdb_handle_t parent_object_handle,
 				hdb_handle_t start_pos,
 				void **key_name,
-				int *key_len,
+				size_t *key_len,
 				void **value,
-				int *value_len)
+				size_t *value_len)
 {
 	unsigned int pos = 0;
 	unsigned int res;
@@ -1379,7 +1370,7 @@ static int object_parent_get(hdb_handle_t object_handle,
 
 static int object_name_get(hdb_handle_t object_handle,
 			   char *object_name,
-			   int *object_name_len)
+			   size_t *object_name_len)
 {
 	struct object_instance *instance;
 	unsigned int res;
@@ -1511,7 +1502,7 @@ static int object_dump(hdb_handle_t object_handle,
 	return (res);
 }
 
-static int object_write_config(char **error_string)
+static int object_write_config(const char **error_string)
 {
 	struct config_iface_ver0 **modules;
 	int num_modules;
@@ -1535,7 +1526,7 @@ static int object_write_config(char **error_string)
 	return 0;
 }
 
-static int object_reload_config(int flush, char **error_string)
+static int object_reload_config(int flush, const char **error_string)
 {
 	struct config_iface_ver0 **modules;
 	int num_modules;
@@ -1610,9 +1601,13 @@ struct lcr_comp objdb_comp_ver0 = {
 	.ifaces				= objdb_iface_ver0
 };
 
+#ifdef COROSYNC_SOLARIS
+void corosync_lcr_component_register (void);
 
-
-__attribute__ ((constructor)) static void objdb_comp_register (void) {
+void corosync_lcr_component_register (void) {
+#else
+__attribute__ ((constructor)) static void corosync_lcr_component_register (void) {
+#endif
         lcr_interfaces_set (&objdb_iface_ver0[0], &objdb_iface);
 
 	lcr_component_register (&objdb_comp_ver0);
