@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2002-2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -39,7 +39,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <assert.h>
 #include <unistd.h>
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -56,6 +55,13 @@
 #include <corosync/engine/config.h>
 #include <corosync/engine/logsys.h>
 
+#ifdef HAVE_LIBNSS
+#include <nss.h>
+#include <pk11pub.h>
+#include <pkcs11.h>
+#include <prerror.h>
+#endif
+
 #include "util.h"
 #include "totemconfig.h"
 #include "tlist.h" /* for HZ */
@@ -89,9 +95,9 @@ static void add_totem_config_notification(
 
 /* These just makes the code below a little neater */
 static inline int objdb_get_string (
-	struct objdb_iface_ver0 *objdb,
+	const struct objdb_iface_ver0 *objdb,
 	hdb_handle_t object_service_handle,
-	char *key, char **value)
+	const char *key, const char **value)
 {
 	int res;
 
@@ -110,9 +116,9 @@ static inline int objdb_get_string (
 }
 
 static inline void objdb_get_int (
-	struct objdb_iface_ver0 *objdb,
+	const struct objdb_iface_ver0 *objdb,
 	hdb_handle_t object_service_handle,
-	char *key, unsigned int *intvalue)
+	const char *key, unsigned int *intvalue)
 {
 	char *value = NULL;
 
@@ -218,6 +224,46 @@ static void totem_volatile_config_read (
 }
 
 
+static void totem_get_crypto_type(
+	const struct objdb_iface_ver0 *objdb,
+	hdb_handle_t object_totem_handle,
+	struct totem_config *totem_config)
+{
+	const char *str;
+
+	totem_config->crypto_accept = TOTEM_CRYPTO_ACCEPT_OLD;
+	if (!objdb_get_string (objdb, object_totem_handle, "crypto_accept", &str)) {
+		if (strcmp(str, "new") == 0) {
+			totem_config->crypto_accept = TOTEM_CRYPTO_ACCEPT_NEW;
+		}
+	}
+
+	totem_config->crypto_type = TOTEM_CRYPTO_SOBER;
+
+#ifdef HAVE_LIBNSS
+	/*
+	 * We must set these even if the key does not exist.
+	 * Encryption type can be set on-the-fly using CFG
+	 */
+	totem_config->crypto_crypt_type = CKM_AES_CBC_PAD;
+	totem_config->crypto_sign_type = CKM_SHA256_RSA_PKCS;
+#endif
+
+	if (!objdb_get_string (objdb, object_totem_handle, "crypto_type", &str)) {
+		if (strcmp(str, "sober") == 0) {
+			return;
+		}
+#ifdef HAVE_LIBNSS
+		if (strcmp(str, "nss") == 0) {
+			totem_config->crypto_type = TOTEM_CRYPTO_NSS;
+
+		}
+#endif
+	}
+}
+
+
+
 extern int totem_config_read (
 	struct objdb_iface_ver0 *objdb,
 	struct totem_config *totem_config,
@@ -226,7 +272,7 @@ extern int totem_config_read (
 	int res = 0;
 	hdb_handle_t object_totem_handle;
 	hdb_handle_t object_interface_handle;
-	char *str;
+	const char *str;
 	unsigned int ringnumber = 0;
 	hdb_handle_t object_find_interface_handle;
 
@@ -263,6 +309,11 @@ printf ("couldn't find totem handle\n");
 			totem_config->secauth = 0;
 		}
 	}
+
+	if (totem_config->secauth == 1) {
+		totem_get_crypto_type(objdb, object_totem_handle, totem_config);
+	}
+
 	if (!objdb_get_string (objdb, object_totem_handle, "rrp_mode", &str)) {
 		strcpy (totem_config->rrp_mode, str);
 	}
@@ -272,6 +323,13 @@ printf ("couldn't find totem handle\n");
 	 */
 	objdb_get_int (objdb, object_totem_handle, "nodeid", &totem_config->node_id);
 
+	totem_config->clear_node_high_bit = 0;
+	if (!objdb_get_string (objdb,object_totem_handle, "clear_node_high_bit", &str)) {
+		if (strcmp (str, "yes") == 0) {
+			totem_config->clear_node_high_bit = 1;
+		}
+	}
+
 	objdb_get_int (objdb,object_totem_handle, "threads", &totem_config->threads);
 
 
@@ -300,6 +358,17 @@ printf ("couldn't find totem handle\n");
 		if (!objdb_get_string (objdb, object_interface_handle, "mcastaddr", &str)) {
 			res = totemip_parse (&totem_config->interfaces[ringnumber].mcast_addr, str, 0);
 		}
+		totem_config->broadcast_use = 0;
+		if (!objdb_get_string (objdb, object_interface_handle, "broadcast", &str)) {
+			if (strcmp (str, "yes") == 0) {
+				totem_config->broadcast_use = 1;
+				totemip_parse (
+					&totem_config->interfaces[ringnumber].mcast_addr,
+					"255.255.255.255", 0);
+			}
+
+
+		}
 
 		/*
 		 * Get mcast port
@@ -332,7 +401,7 @@ int totem_config_validate (
 {
 	static char local_error_reason[512];
 	char parse_error[512];
-	char *error_reason = local_error_reason;
+	const char *error_reason = local_error_reason;
 	int i;
 	unsigned int interface_max = INTERFACE_MAX;
 
@@ -362,14 +431,16 @@ int totem_config_validate (
 			goto parse_error;
 		}
 
-		if (totem_config->interfaces[i].mcast_addr.family != totem_config->interfaces[i].bindnet.family) {
-			error_reason = "Multicast address family does not match bind address family";
-			goto parse_error;
-		}
+		if (totem_config->broadcast_use == 0) {
+			if (totem_config->interfaces[i].mcast_addr.family != totem_config->interfaces[i].bindnet.family) {
+				error_reason = "Multicast address family does not match bind address family";
+				goto parse_error;
+			}
 
-		if (totem_config->interfaces[i].mcast_addr.family != totem_config->interfaces[i].bindnet.family) {
-			error_reason =  "Not all bind address belong to the same IP family";
-			goto parse_error;
+			if (totem_config->interfaces[i].mcast_addr.family != totem_config->interfaces[i].bindnet.family) {
+				error_reason =  "Not all bind address belong to the same IP family";
+				goto parse_error;
+			}
 		}
 	}
 
@@ -410,7 +481,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->max_network_delay < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The max_network_delay parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The max_network_delay parameter (%d ms) may not be less then (%d ms).",
 			totem_config->max_network_delay, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -424,7 +496,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->token_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The token timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The token timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->token_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -440,7 +513,8 @@ int totem_config_validate (
 			(1000/HZ));
 	}
 	if (totem_config->token_retransmit_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The token retransmit timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The token retransmit timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->token_retransmit_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -450,7 +524,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->token_hold_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The token hold timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The token hold timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->token_hold_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -460,7 +535,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->join_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The join timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The join timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->join_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -470,7 +546,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->consensus_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The consensus timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The consensus timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->consensus_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -480,7 +557,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->merge_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The merge timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The merge timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->merge_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -490,7 +568,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->downcheck_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The downcheck timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The downcheck timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->downcheck_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -501,14 +580,16 @@ int totem_config_validate (
 	if (strcmp (totem_config->rrp_mode, "none") &&
 		strcmp (totem_config->rrp_mode, "active") &&
 		strcmp (totem_config->rrp_mode, "passive")) {
-		sprintf (local_error_reason, "The RRP mode \"%s\" specified is invalid.  It must be none, active, or passive.\n", totem_config->rrp_mode);
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The RRP mode \"%s\" specified is invalid.  It must be none, active, or passive.\n", totem_config->rrp_mode);
 		goto parse_error;
 	}
 	if (totem_config->rrp_problem_count_timeout == 0) {
 		totem_config->rrp_problem_count_timeout = RRP_PROBLEM_COUNT_TIMEOUT;
 	}
 	if (totem_config->rrp_problem_count_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The RRP problem count timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The RRP problem count timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->rrp_problem_count_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -516,7 +597,8 @@ int totem_config_validate (
 		totem_config->rrp_problem_count_threshold = RRP_PROBLEM_COUNT_THRESHOLD_DEFAULT;
 	}
 	if (totem_config->rrp_problem_count_threshold < RRP_PROBLEM_COUNT_THRESHOLD_MIN) {
-		sprintf (local_error_reason, "The RRP problem count threshold (%d problem count) may not be less then (%d problem count).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The RRP problem count threshold (%d problem count) may not be less then (%d problem count).",
 			totem_config->rrp_problem_count_threshold, RRP_PROBLEM_COUNT_THRESHOLD_MIN);
 		goto parse_error;
 	}
@@ -526,7 +608,8 @@ int totem_config_validate (
 	}
 
 	if (totem_config->rrp_token_expired_timeout < MINIMUM_TIMEOUT) {
-		sprintf (local_error_reason, "The RRP token expired timeout parameter (%d ms) may not be less then (%d ms).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The RRP token expired timeout parameter (%d ms) may not be less then (%d ms).",
 			totem_config->rrp_token_expired_timeout, MINIMUM_TIMEOUT);
 		goto parse_error;
 	}
@@ -535,7 +618,7 @@ int totem_config_validate (
 		interface_max = 1;
 	}
 	if (interface_max < totem_config->interface_count) {
-		sprintf (parse_error,
+		snprintf (parse_error, sizeof(parse_error),
 			"%d is too many configured interfaces for the rrp_mode setting %s.",
 			totem_config->interface_count,
 			totem_config->rrp_mode);
@@ -555,7 +638,8 @@ int totem_config_validate (
 	}
 
 	if ((MESSAGE_QUEUE_MAX) < totem_config->max_messages) {
-		sprintf (local_error_reason, "The max_messages parameter (%d messages) may not be greater then (%d messages).",
+		snprintf (local_error_reason, sizeof(local_error_reason),
+			"The max_messages parameter (%d messages) may not be greater then (%d messages).",
 			totem_config->max_messages, MESSAGE_QUEUE_MAX);
 		goto parse_error;
 	}
@@ -577,43 +661,50 @@ int totem_config_validate (
 	return (0);
 
 parse_error:
-	sprintf (error_string_response,
+	snprintf (error_string_response, sizeof(error_string_response),
 		 "parse error in config: %s\n", error_reason);
 	*error_string = error_string_response;
 	return (-1);
 }
 
 static int read_keyfile (
-	char *key_location,
+	const char *key_location,
 	struct totem_config *totem_config,
 	const char **error_string)
 {
 	int fd;
 	int res;
+	ssize_t expected_key_len = sizeof (totem_config->private_key);
+	int saved_errno;
 
 	fd = open (key_location, O_RDONLY);
 	if (fd == -1) {
-		sprintf (error_string_response, "Could not open %s: %s\n",
+		snprintf (error_string_response, sizeof(error_string_response),
+			"Could not open %s: %s\n",
 			 key_location, strerror (errno));
 		goto parse_error;
 	}
 
-	res = read (fd, totem_config->private_key, 128);
+	res = read (fd, totem_config->private_key, expected_key_len);
+	saved_errno = errno;
+	close (fd);
+
 	if (res == -1) {
-		close (fd);
-		sprintf (error_string_response, "Could not read %s: %s\n",
-			 key_location, strerror (errno));
+		snprintf (error_string_response, sizeof(error_string_response),
+			"Could not read %s: %s\n",
+			 key_location, strerror (saved_errno));
 		goto parse_error;
 	}
 
-	totem_config->private_key_len = 128;
+	totem_config->private_key_len = expected_key_len;
 
-	if (res != 128) {
-		close (fd);
-		sprintf (error_string_response, "Could only read %d bits of 1024 bits from %s.\n",
+	if (res != expected_key_len) {
+		snprintf (error_string_response, sizeof(error_string_response),
+			"Could only read %d bits of 1024 bits from %s.\n",
 			 res * 8, key_location);
 		goto parse_error;
 	}
+
 	return 0;
 
 parse_error:
@@ -627,7 +718,7 @@ int totem_config_keyread (
 	const char **error_string)
 {
 	int got_key = 0;
-	char *key_location = NULL;
+	const char *key_location = NULL;
 	hdb_handle_t object_totem_handle;
 	int res;
 
@@ -652,7 +743,7 @@ int totem_config_keyread (
 		got_key = 1;
 	} else { /* Or the key itself may be in the objdb */
 		char *key = NULL;
-		int key_len;
+		size_t key_len;
 		res = objdb->object_key_get (object_totem_handle,
 			"key",
 			strlen ("key"),
@@ -660,6 +751,9 @@ int totem_config_keyread (
 			&key_len);
 
 		if (res == 0 && key) {
+			if (key_len > sizeof (totem_config->private_key)) {
+				goto key_error;
+			}
 			memcpy(totem_config->private_key, key, key_len);
 			totem_config->private_key_len = key_len;
 			got_key = 1;
@@ -668,9 +762,9 @@ int totem_config_keyread (
 
 	/* In desperation we read the default filename */
 	if (!got_key) {
-		char *filename = getenv("COROSYNC_TOTEM_AUTHKEY_FILE");
+		const char *filename = getenv("COROSYNC_TOTEM_AUTHKEY_FILE");
 		if (!filename)
-			filename = SYSCONFDIR "/ais/authkey";
+			filename = COROSYSCONFDIR "/authkey";
 		res = read_keyfile(filename, totem_config, error_string);
 		if (res)
 			goto key_error;
@@ -688,9 +782,9 @@ key_error:
 static void totem_key_change_notify(object_change_type_t change_type,
 			      hdb_handle_t parent_object_handle,
 			      hdb_handle_t object_handle,
-			      void *object_name_pt, int object_name_len,
-			      void *key_name_pt, int key_len,
-			      void *key_value_pt, int key_value_len,
+			      const void *object_name_pt, size_t object_name_len,
+			      const void *key_name_pt, size_t key_len,
+			      const void *key_value_pt, size_t key_value_len,
 			      void *priv_data_pt)
 {
 	struct totem_config *totem_config = priv_data_pt;
@@ -740,7 +834,7 @@ static void totem_objdb_reload_notify(objdb_reload_notify_type_t type, int flush
 
 		}
 		else {
-			log_printf(LOG_LEVEL_ERROR, "totem objdb tracking stopped, cannot find totem{} handle on objdb\n");
+			log_printf(LOGSYS_LEVEL_ERROR, "totem objdb tracking stopped, cannot find totem{} handle on objdb\n");
 		}
 	}
 }
