@@ -1,13 +1,13 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006-2007, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -59,12 +59,12 @@ extern int totemnet_initialize (
 
 	void (*deliver_fn) (
 		void *context,
-		void *msg,
-		int msg_len),
+		const void *msg,
+		unsigned int msg_len),
 
 	void (*iface_change_fn) (
 		void *context,
-		struct totem_ip_address *iface_address));
+		const struct totem_ip_address *iface_address));
 
 extern int totemnet_processor_count_set (
 	hdb_handle_t handle,
@@ -72,18 +72,18 @@ extern int totemnet_processor_count_set (
 
 extern int totemnet_token_send (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 extern int totemnet_mcast_flush_send (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 extern int totemnet_mcast_noflush_send (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 extern int totemnet_recv_flush (hdb_handle_t handle);
 
@@ -103,6 +103,13 @@ extern int totemnet_iface_get (
 
 extern int totemnet_token_target_set (
 	hdb_handle_t handle,
-	struct totem_ip_address *token_target);
+	const struct totem_ip_address *token_target);
+
+extern int totemnet_crypto_set (
+	hdb_handle_t handle,
+	unsigned int type);
+
+extern int totemnet_recv_mcast_empty (
+	hdb_handle_t handle);
 
 #endif /* TOTEMNET_H_DEFINED */
