@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2003-2005 MontaVista Software, Inc.
  * Copyright (c) 2005 OSDL.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -9,7 +9,7 @@
  * Author: Mark Haverkamp (markh@osdl.org)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -66,7 +66,7 @@
 
 /*
  * ASSEMBLY AND UNPACKING ALGORITHM:
- * 
+ *
  * copy incoming packet into assembly data buffer indexed by current
  * location of end of fragment
  *
@@ -79,11 +79,14 @@
  * else
  * if msg_count = 1 and fragmented
  *	do nothing
- *	
+ *
  */
 
 #include <config.h>
 
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 #include <netinet/in.h>
 #include <sys/uio.h>
 #include <stdio.h>
@@ -92,12 +95,15 @@
 #include <assert.h>
 #include <pthread.h>
 #include <errno.h>
+#include <limits.h>
 
 #include <corosync/swab.h>
 #include <corosync/hdb.h>
 #include <corosync/list.h>
 #include <corosync/totem/coropoll.h>
 #include <corosync/totem/totempg.h>
+#define LOGSYS_UTILS_ONLY 1
+#include <corosync/engine/logsys.h>
 
 #include "totemmrp.h"
 #include "totemsrp.h"
@@ -123,16 +129,16 @@ struct totempg_mcast_header {
  */
 struct totempg_mcast {
 	struct totempg_mcast_header header;
-	unsigned char fragmented; 
-	unsigned char continuation; 
+	unsigned char fragmented;
+	unsigned char continuation;
 	unsigned short msg_count;
 	/*
 	 * short msg_len[msg_count];
-	 */ 
-	/* 
+	 */
+	/*
 	 * data for messages
 	 */
-}; 
+};
 
 /*
  * Maximum packet size for totem pg messages
@@ -158,31 +164,33 @@ static int totempg_log_level_warning;
 static int totempg_log_level_notice;
 static int totempg_log_level_debug;
 static int totempg_subsys_id;
-static void (*totempg_log_printf) (int subsys_id, const char *function,
-	const char *file, int line, unsigned int level,
-	const char *format, ...) __attribute__((format(printf, 6, 7)));
+static void (*totempg_log_printf) (
+	unsigned int rec_ident,
+	const char *function,
+	const char *file,
+	int line,
+	const char *format, ...) __attribute__((format(printf, 5, 6)));
 
 struct totem_config *totempg_totem_config;
 
+enum throw_away_mode {
+	THROW_AWAY_INACTIVE,
+	THROW_AWAY_ACTIVE
+};
+
 struct assembly {
 	unsigned int nodeid;
 	unsigned char data[MESSAGE_SIZE_MAX];
 	int index;
 	unsigned char last_frag_num;
+	enum throw_away_mode throw_away_mode;
 	struct list_head list;
 };
 
 static void assembly_deref (struct assembly *assembly);
 
 static int callback_token_received_fn (enum totem_callback_token_type type,
-	void *data);
-
-enum throw_away_mode_t {
-	THROW_AWAY_INACTIVE,
-	THROW_AWAY_ACTIVE
-};
-
-static enum throw_away_mode_t throw_away_mode = THROW_AWAY_INACTIVE;
+	const void *data);
 
 DECLARE_LIST_INIT(assembly_list_inuse);
 
@@ -190,12 +198,12 @@ DECLARE_LIST_INIT(assembly_list_free);
 
 /*
  * Staging buffer for packed messages.  Messages are staged in this buffer
- * before sending.  Multiple messages may fit which cuts down on the 
- * number of mcasts sent.  If a message doesn't completely fit, then 
- * the mcast header has a fragment bit set that says that there are more 
+ * before sending.  Multiple messages may fit which cuts down on the
+ * number of mcasts sent.  If a message doesn't completely fit, then
+ * the mcast header has a fragment bit set that says that there are more
  * data to follow.  fragment_size is an index into the buffer.  It indicates
- * the size of message data and where to place new message data.  
- * fragment_contuation indicates whether the first packed message in 
+ * the size of message data and where to place new message data.
+ * fragment_contuation indicates whether the first packed message in
  * the buffer is a continuation of a previously packed fragment.
  */
 static unsigned char *fragmentation_data;
@@ -211,28 +219,23 @@ static unsigned int totempg_max_handle = 0;
 struct totempg_group_instance {
 	void (*deliver_fn) (
 		unsigned int nodeid,
-		struct iovec *iovec,
-		int iov_len,
+		const void *msg,
+		unsigned int msg_len,
 		int endian_conversion_required);
 
 	void (*confchg_fn) (
 		enum totem_configuration_type configuration_type,
-		unsigned int *member_list, int member_list_entries,
-		unsigned int *left_list, int left_list_entries,
-		unsigned int *joined_list, int joined_list_entries,
-		struct memb_ring_id *ring_id);
+		const unsigned int *member_list, size_t member_list_entries,
+		const unsigned int *left_list, size_t left_list_entries,
+		const unsigned int *joined_list, size_t joined_list_entries,
+		const struct memb_ring_id *ring_id);
 
 	struct totempg_group *groups;
 
 	int groups_cnt;
 };
 
-static struct hdb_handle_database totempg_groups_instance_database = {
-	.handle_count	= 0,
-	.handles	= 0,
-	.iterator	= 0,
-	.mutex		= PTHREAD_MUTEX_INITIALIZER
-};
+DECLARE_HDB_DATABASE (totempg_groups_instance_database,NULL);
 
 static unsigned char next_fragment = 1;
 
@@ -244,8 +247,12 @@ static pthread_mutex_t mcast_msg_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 #define log_printf(level, format, args...)				\
 do {									\
-        totempg_log_printf (totempg_subsys_id, __FUNCTION__,		\
-		__FILE__, __LINE__, level, format, ##args);		\
+        totempg_log_printf (						\
+		LOGSYS_ENCODE_RECID(level,				\
+				    totempg_subsys_id,			\
+				    LOGSYS_RECID_LOG),			\
+		__FUNCTION__, __FILE__, __LINE__,			\
+		format, ##args);					\
 } while (0);
 
 static int msg_count_send_ok (int msg_count);
@@ -262,7 +269,7 @@ static struct assembly *assembly_ref (unsigned int nodeid)
 	 */
 	for (list = assembly_list_inuse.next;
 		list != &assembly_list_inuse;
-		list = list->next) { 
+		list = list->next) {
 
 		assembly = list_entry (list, struct assembly, list);
 
@@ -279,6 +286,9 @@ static struct assembly *assembly_ref (unsigned int nodeid)
 		list_del (&assembly->list);
 		list_add (&assembly->list, &assembly_list_inuse);
 		assembly->nodeid = nodeid;
+		assembly->index = 0;
+		assembly->last_frag_num = 0;
+		assembly->throw_away_mode = THROW_AWAY_INACTIVE;
 		return (assembly);
 	}
 
@@ -286,12 +296,15 @@ static struct assembly *assembly_ref (unsigned int nodeid)
 	 * Nothing available in inuse or free list, so allocate a new one
 	 */
 	assembly = malloc (sizeof (struct assembly));
-	memset (assembly, 0, sizeof (struct assembly));
 	/*
 	 * TODO handle memory allocation failure here
 	 */
 	assert (assembly);
 	assembly->nodeid = nodeid;
+	assembly->data[0] = 0;
+	assembly->index = 0;
+	assembly->last_frag_num = 0;
+	assembly->throw_away_mode = THROW_AWAY_INACTIVE;
 	list_init (&assembly->list);
 	list_add (&assembly->list, &assembly_list_inuse);
 
@@ -306,15 +319,26 @@ static void assembly_deref (struct assembly *assembly)
 
 static inline void app_confchg_fn (
 	enum totem_configuration_type configuration_type,
-	unsigned int *member_list, int member_list_entries,
-	unsigned int *left_list, int left_list_entries,
-	unsigned int *joined_list, int joined_list_entries,
-	struct memb_ring_id *ring_id)
+	const unsigned int *member_list, size_t member_list_entries,
+	const unsigned int *left_list, size_t left_list_entries,
+	const unsigned int *joined_list, size_t joined_list_entries,
+	const struct memb_ring_id *ring_id)
 {
 	int i;
 	struct totempg_group_instance *instance;
+	struct assembly *assembly;
 	unsigned int res;
 
+	/*
+	 * For every leaving processor, add to free list
+	 * This also has the side effect of clearing out the dataset
+	 * In the leaving processor's assembly buffer.
+	 */
+	for (i = 0; i < left_list_entries; i++) {
+		assembly = assembly_ref (left_list[i]);
+		list_del (&assembly->list);
+		list_add (&assembly->list, &assembly_list_free);
+	}
 	for (i = 0; i <= totempg_max_handle; i++) {
 		res = hdb_handle_get (&totempg_groups_instance_database,
 			hdb_nocheck_convert (i), (void *)&instance);
@@ -339,33 +363,31 @@ static inline void app_confchg_fn (
 }
 
 static inline void group_endian_convert (
-	struct iovec *iovec)
+	void *msg,
+	int msg_len)
 {
 	unsigned short *group_len;
 	int i;
-	struct iovec iovec_aligned = { NULL, 0 };
-	struct iovec *iovec_swab;
+	char *aligned_msg;
 
 	/*
 	 * Align data structure for sparc and ia64
 	 */
-	if ((size_t)iovec->iov_base % 4 != 0) {
-		iovec_aligned.iov_base = alloca(iovec->iov_len);
-		memcpy(iovec_aligned.iov_base, iovec->iov_base, iovec->iov_len);
-		iovec_aligned.iov_len = iovec->iov_len;
-		iovec_swab = &iovec_aligned;
+	if ((size_t)msg % 4 != 0) {
+		aligned_msg = alloca(msg_len);
+		memcpy(aligned_msg, msg, msg_len);
 	} else {
-		iovec_swab = iovec;
+		aligned_msg = msg;
 	}
 
-	group_len = (unsigned short *)iovec_swab->iov_base;
+	group_len = (unsigned short *)aligned_msg;
 	group_len[0] = swab16(group_len[0]);
 	for (i = 1; i < group_len[0] + 1; i++) {
 		group_len[i] = swab16(group_len[i]);
 	}
 
-	if (iovec_swab == &iovec_aligned) {
-		memcpy(iovec->iov_base, iovec_aligned.iov_base, iovec->iov_len);
+	if (aligned_msg != msg) {
+		memcpy(msg, aligned_msg, msg_len);
 	}
 }
 
@@ -381,7 +403,7 @@ static inline int group_matches (
 	int i;
 	int j;
         struct iovec iovec_aligned = { NULL, 0 };
-	
+
 	assert (iov_len == 1);
 
 	/*
@@ -422,12 +444,12 @@ static inline int group_matches (
 	}
 	return (0);
 }
-	
+
 
 static inline void app_deliver_fn (
 	unsigned int nodeid,
-	struct iovec *iovec,
-	unsigned int iov_len,
+	void *msg,
+	unsigned int msg_len,
 	int endian_conversion_required)
 {
 	int i;
@@ -435,18 +457,23 @@ static inline void app_deliver_fn (
 	struct iovec stripped_iovec;
 	unsigned int adjust_iovec;
 	unsigned int res;
+	struct iovec *iovec;
+
         struct iovec aligned_iovec = { NULL, 0 };
 
 	if (endian_conversion_required) {
-		group_endian_convert (iovec);
+		group_endian_convert (msg, msg_len);
 	}
 
+/*
+ * TODO This function needs to be rewritten for proper alignment to avoid 3+ memory copies
+ */
 	/*
 	 * Align data structure for sparc and ia64
 	 */
-	aligned_iovec.iov_base = alloca(iovec->iov_len);
-	aligned_iovec.iov_len = iovec->iov_len;
-	memcpy(aligned_iovec.iov_base, iovec->iov_base, iovec->iov_len);
+	aligned_iovec.iov_base = alloca(msg_len);
+	aligned_iovec.iov_len = msg_len;
+	memcpy(aligned_iovec.iov_base, msg, msg_len);
 	iovec = &aligned_iovec;
 
 	for (i = 0; i <= totempg_max_handle; i++) {
@@ -454,10 +481,9 @@ static inline void app_deliver_fn (
 			hdb_nocheck_convert (i), (void *)&instance);
 
 		if (res == 0) {
-			assert (iov_len == 1);
-			if (group_matches (iovec, iov_len, instance->groups, instance->groups_cnt, &adjust_iovec)) {
+			if (group_matches (iovec, 1, instance->groups, instance->groups_cnt, &adjust_iovec)) {
 				stripped_iovec.iov_len = iovec->iov_len - adjust_iovec;
-//				stripped_iovec.iov_base = (char *)iovec->iov_base + adjust_iovec;
+				stripped_iovec.iov_base = (char *)iovec->iov_base + adjust_iovec;
 
 				/*
 				 * Align data structure for sparc and ia64
@@ -475,8 +501,8 @@ static inline void app_deliver_fn (
 
 				instance->deliver_fn (
 					nodeid,
-					&stripped_iovec,
-					iov_len,
+					stripped_iovec.iov_base,
+					stripped_iovec.iov_len,
 					endian_conversion_required);
 			}
 
@@ -487,10 +513,10 @@ static inline void app_deliver_fn (
 
 static void totempg_confchg_fn (
 	enum totem_configuration_type configuration_type,
-	unsigned int *member_list, int member_list_entries,
-	unsigned int *left_list, int left_list_entries,
-	unsigned int *joined_list, int joined_list_entries,
-	struct memb_ring_id *ring_id)
+	const unsigned int *member_list, size_t member_list_entries,
+	const unsigned int *left_list, size_t left_list_entries,
+	const unsigned int *joined_list, size_t joined_list_entries,
+	const struct memb_ring_id *ring_id)
 {
 // TODO optimize this
 	app_confchg_fn (configuration_type,
@@ -502,8 +528,8 @@ static void totempg_confchg_fn (
 
 static void totempg_deliver_fn (
 	unsigned int nodeid,
-	struct iovec *iovec,
-	int iov_len,
+	const void *msg,
+	unsigned int msg_len,
 	int endian_conversion_required)
 {
 	struct totempg_mcast *mcast;
@@ -511,11 +537,11 @@ static void totempg_deliver_fn (
 	int i;
 	struct assembly *assembly;
 	char header[FRAME_SIZE_MAX];
-	int h_index;
-	int a_i = 0;
 	int msg_count;
 	int continuation;
 	int start;
+	const char *data;
+	int datasize;
 
 	assembly = assembly_ref (nodeid);
 	assert (assembly);
@@ -524,70 +550,38 @@ static void totempg_deliver_fn (
 	 * Assemble the header into one block of data and
 	 * assemble the packet contents into one block of data to simplify delivery
 	 */
-	if (iov_len == 1) {
-		/* 
-		 * This message originated from external processor 
-		 * because there is only one iovec for the full msg.
-		 */
-		char *data;
-		int datasize;
 
-		mcast = (struct totempg_mcast *)iovec[0].iov_base;
-		if (endian_conversion_required) {
-			mcast->msg_count = swab16 (mcast->msg_count);
-		}
-
-		msg_count = mcast->msg_count;
-		datasize = sizeof (struct totempg_mcast) +
-			msg_count * sizeof (unsigned short);
-		
-		memcpy (header, iovec[0].iov_base, datasize);
-		assert(iovec);
-		data = iovec[0].iov_base;
-
-		msg_lens = (unsigned short *) (header + sizeof (struct totempg_mcast));
-		if (endian_conversion_required) {
-			for (i = 0; i < mcast->msg_count; i++) {
-				msg_lens[i] = swab16 (msg_lens[i]);
-			}
-		}
-
-		memcpy (&assembly->data[assembly->index], &data[datasize],
-			iovec[0].iov_len - datasize);
-	} else {
-		/* 
-		 * The message originated from local processor  
-		 * becasue there is greater than one iovec for then full msg.
-		 */
-		h_index = 0;
-		for (i = 0; i < 2; i++) {
-			memcpy (&header[h_index], iovec[i].iov_base, iovec[i].iov_len);
-			h_index += iovec[i].iov_len;
-		}
-
-		mcast = (struct totempg_mcast *)header;
-// TODO make sure we are using a copy of mcast not the actual data itself
-
-		msg_lens = (unsigned short *) (header + sizeof (struct totempg_mcast));
-
-		for (i = 2; i < iov_len; i++) {
-			a_i = assembly->index;
-			assert (iovec[i].iov_len + a_i <= MESSAGE_SIZE_MAX);
-			memcpy (&assembly->data[a_i], iovec[i].iov_base, iovec[i].iov_len);
-			a_i += msg_lens[i - 2];
-		}
-		iov_len -= 2;
+	mcast = (struct totempg_mcast *)msg;
+	if (endian_conversion_required) {
+		mcast->msg_count = swab16 (mcast->msg_count);
 	}
 
+	msg_count = mcast->msg_count;
+	datasize = sizeof (struct totempg_mcast) +
+		msg_count * sizeof (unsigned short);
+
+	memcpy (header, msg, datasize);
+	data = msg;
+
+	msg_lens = (unsigned short *) (header + sizeof (struct totempg_mcast));
+	if (endian_conversion_required) {
+		for (i = 0; i < mcast->msg_count; i++) {
+			msg_lens[i] = swab16 (msg_lens[i]);
+		}
+	}
+
+	memcpy (&assembly->data[assembly->index], &data[datasize],
+		msg_len - datasize);
+
 	/*
 	 * If the last message in the buffer is a fragment, then we
 	 * can't deliver it.  We'll first deliver the full messages
-	 * then adjust the assembly buffer so we can add the rest of the 
+	 * then adjust the assembly buffer so we can add the rest of the
 	 * fragment when it arrives.
 	 */
 	msg_count = mcast->fragmented ? mcast->msg_count - 1 : mcast->msg_count;
 	continuation = mcast->continuation;
-	iov_delv.iov_base = &assembly->data[0];
+	iov_delv.iov_base = (void *)&assembly->data[0];
 	iov_delv.iov_len = assembly->index + msg_lens[0];
 
 	/*
@@ -596,37 +590,37 @@ static void totempg_deliver_fn (
 	 * Also, if the first packed message is a continuation
 	 * of a previous message, but the assembly buffer
 	 * is empty, then we need to discard it since we can't
-	 * assemble a complete message. Likewise, if this message isn't a 
+	 * assemble a complete message. Likewise, if this message isn't a
 	 * continuation and the assembly buffer is empty, we have to discard
 	 * the continued message.
 	 */
 	start = 0;
 
-	if (throw_away_mode == THROW_AWAY_ACTIVE) {
+	if (assembly->throw_away_mode == THROW_AWAY_ACTIVE) {
 		 /* Throw away the first msg block */
 		if (mcast->fragmented == 0 || mcast->fragmented == 1) {
-			throw_away_mode = THROW_AWAY_INACTIVE;
+			assembly->throw_away_mode = THROW_AWAY_INACTIVE;
 
 			assembly->index += msg_lens[0];
-			iov_delv.iov_base = &assembly->data[assembly->index];
+			iov_delv.iov_base = (void *)&assembly->data[assembly->index];
 			iov_delv.iov_len = msg_lens[1];
 			start = 1;
 		}
-	} else 
-	if (throw_away_mode == THROW_AWAY_INACTIVE) {
+	} else
+	if (assembly->throw_away_mode == THROW_AWAY_INACTIVE) {
 		if (continuation == assembly->last_frag_num) {
 			assembly->last_frag_num = mcast->fragmented;
 			for  (i = start; i < msg_count; i++) {
-				app_deliver_fn(nodeid, &iov_delv, 1,
+				app_deliver_fn(nodeid, iov_delv.iov_base, iov_delv.iov_len,
 					endian_conversion_required);
 				assembly->index += msg_lens[i];
-				iov_delv.iov_base = &assembly->data[assembly->index];
+				iov_delv.iov_base = (void *)&assembly->data[assembly->index];
 				if (i < (msg_count - 1)) {
 					iov_delv.iov_len = msg_lens[i + 1];
 				}
 			}
 		} else {
-			throw_away_mode = THROW_AWAY_ACTIVE;
+			assembly->throw_away_mode = THROW_AWAY_ACTIVE;
 		}
 	}
 
@@ -660,7 +654,7 @@ static void totempg_deliver_fn (
 void *callback_token_received_handle;
 
 int callback_token_received_fn (enum totem_callback_token_type type,
-	void *data)
+				const void *data)
 {
 	struct totempg_mcast mcast;
 	struct iovec iovecs[3];
@@ -675,6 +669,7 @@ int callback_token_received_fn (enum totem_callback_token_type type,
 		pthread_mutex_unlock (&mcast_msg_mutex);
 		return (0);
 	}
+	mcast.header.version = 0;
 	mcast.fragmented = 0;
 
 	/*
@@ -686,11 +681,11 @@ int callback_token_received_fn (enum totem_callback_token_type type,
 
 	mcast.msg_count = mcast_packed_msg_count;
 
-	iovecs[0].iov_base = &mcast;
+	iovecs[0].iov_base = (void *)&mcast;
 	iovecs[0].iov_len = sizeof (struct totempg_mcast);
-	iovecs[1].iov_base = mcast_packed_msg_lens;
+	iovecs[1].iov_base = (void *)mcast_packed_msg_lens;
 	iovecs[1].iov_len = mcast_packed_msg_count * sizeof (unsigned short);
-	iovecs[2].iov_base = &fragmentation_data[0];
+	iovecs[2].iov_base = (void *)&fragmentation_data[0];
 	iovecs[2].iov_len = fragment_size;
 	res = totemmrp_mcast (iovecs, 3, 0);
 
@@ -731,7 +726,7 @@ int totempg_initialize (
 		totempg_confchg_fn);
 
 	totemmrp_callback_token_create (
-		&callback_token_received_handle, 
+		&callback_token_received_handle,
 		TOTEM_CALLBACK_TOKEN_RECEIVED,
 		0,
 		callback_token_received_fn,
@@ -754,7 +749,7 @@ void totempg_finalize (void)
  */
 static int mcast_msg (
 	struct iovec *iovec_in,
-	int iov_len,
+	unsigned int iov_len,
 	int guarantee)
 {
 	int res = 0;
@@ -764,7 +759,7 @@ static int mcast_msg (
 	int i;
 	int dest, src;
 	int max_packet_size = 0;
-	int copy_len = 0; 
+	int copy_len = 0;
 	int copy_base = 0;
 	int total_size = 0;
 
@@ -802,6 +797,7 @@ static int mcast_msg (
 		return(-1);
 	}
 
+	mcast.header.version = 0;
 	for (i = 0; i < iov_len; ) {
 		mcast.fragmented = 0;
 		mcast.continuation = fragment_continuation;
@@ -813,7 +809,7 @@ static int mcast_msg (
 		 * fragment_buffer on exit so that max_packet_size + fragment_size
 		 * doesn't exceed the size of the fragment_buffer on the next call.
 		 */
-		if ((copy_len + fragment_size) < 
+		if ((copy_len + fragment_size) <
 			(max_packet_size - sizeof (unsigned short))) {
 
 			memcpy (&fragmentation_data[fragment_size],
@@ -850,7 +846,7 @@ static int mcast_msg (
 			 * fit, then indicate a fragment. This also means that the next
 			 * message will have the continuation of this one.
 			 */
-			if ((i < (iov_len - 1)) || 
+			if ((i < (iov_len - 1)) ||
 					((copy_base + copy_len) < iovec[i].iov_len)) {
 				if (!next_fragment) {
 					next_fragment++;
@@ -867,12 +863,12 @@ static int mcast_msg (
 			 * assemble the message and send it
 			 */
 			mcast.msg_count = ++mcast_packed_msg_count;
-			iovecs[0].iov_base = &mcast;
+			iovecs[0].iov_base = (void *)&mcast;
 			iovecs[0].iov_len = sizeof(struct totempg_mcast);
-			iovecs[1].iov_base = mcast_packed_msg_lens;
-			iovecs[1].iov_len = mcast_packed_msg_count * 
+			iovecs[1].iov_base = (void *)mcast_packed_msg_lens;
+			iovecs[1].iov_len = mcast_packed_msg_count *
 				sizeof(unsigned short);
-			iovecs[2].iov_base = data_ptr;
+			iovecs[2].iov_base = (void *)data_ptr;
 			iovecs[2].iov_len = max_packet_size;
 			assert (totemmrp_avail() > 0);
 			res = totemmrp_mcast (iovecs, 3, guarantee);
@@ -892,7 +888,7 @@ static int mcast_msg (
 				copy_len = 0;
 				copy_base = 0;
 				i++;
-			
+
 			/*
 			 * Continue with the rest of the current iovec.
 			 */
@@ -924,7 +920,7 @@ static int msg_count_send_ok (
 	int avail = 0;
 
 	avail = totemmrp_avail () - totempg_reserved - 1;
-	
+
 	return (avail > msg_count);
 }
 
@@ -936,7 +932,7 @@ static int byte_count_send_ok (
 
 	avail = totemmrp_avail () - 1;
 
-	msg_count = (byte_count / (totempg_totem_config->net_mtu - 25)) + 1; 
+	msg_count = (byte_count / (totempg_totem_config->net_mtu - 25)) + 1;
 
 	return (avail > msg_count);
 }
@@ -946,7 +942,7 @@ static int send_reserve (
 {
 	unsigned int msg_count = 0;
 
-	msg_count = (msg_size / (totempg_totem_config->net_mtu - 25)) + 1; 
+	msg_count = (msg_size / (totempg_totem_config->net_mtu - 25)) + 1;
 	totempg_reserved += msg_count;
 
 	return (msg_count);
@@ -962,8 +958,8 @@ int totempg_callback_token_create (
 	void **handle_out,
 	enum totem_callback_token_type type,
 	int delete,
-	int (*callback_fn) (enum totem_callback_token_type type, void *),
-	void *data)
+	int (*callback_fn) (enum totem_callback_token_type type, const void *),
+	const void *data)
 {
 	unsigned int res;
 	pthread_mutex_lock (&callback_token_mutex);
@@ -990,16 +986,16 @@ int totempg_groups_initialize (
 
 	void (*deliver_fn) (
 		unsigned int nodeid,
-		struct iovec *iovec,
-		int iov_len,
+		const void *msg,
+		unsigned int msg_len,
 		int endian_conversion_required),
 
 	void (*confchg_fn) (
 		enum totem_configuration_type configuration_type,
-		unsigned int *member_list, int member_list_entries,
-		unsigned int *left_list, int left_list_entries,
-		unsigned int *joined_list, int joined_list_entries,
-		struct memb_ring_id *ring_id))
+		const unsigned int *member_list, size_t member_list_entries,
+		const unsigned int *left_list, size_t left_list_entries,
+		const unsigned int *joined_list, size_t joined_list_entries,
+		const struct memb_ring_id *ring_id))
 {
 	struct totempg_group_instance *instance;
 	unsigned int res;
@@ -1041,8 +1037,8 @@ error_exit:
 
 int totempg_groups_join (
 	hdb_handle_t handle,
-	struct totempg_group *groups,
-	int group_cnt)
+	const struct totempg_group *groups,
+	size_t group_cnt)
 {
 	struct totempg_group_instance *instance;
 	struct totempg_group *new_groups;
@@ -1062,10 +1058,10 @@ int totempg_groups_join (
 		res = ENOMEM;
 		goto error_exit;
 	}
-	memcpy (&new_groups[instance->groups_cnt], 
+	memcpy (&new_groups[instance->groups_cnt],
 		groups, group_cnt * sizeof (struct totempg_group));
 	instance->groups = new_groups;
-	instance->groups_cnt = instance->groups_cnt = group_cnt;
+	instance->groups_cnt += group_cnt;
 
 	hdb_handle_put (&totempg_groups_instance_database, handle);
 
@@ -1076,8 +1072,8 @@ error_exit:
 
 int totempg_groups_leave (
 	hdb_handle_t handle,
-	struct totempg_group *groups,
-	int group_cnt)
+	const struct totempg_group *groups,
+	size_t group_cnt)
 {
 	struct totempg_group_instance *instance;
 	unsigned int res;
@@ -1101,8 +1097,8 @@ error_exit:
 
 int totempg_groups_mcast_joined (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	int iov_len,
+	const struct iovec *iovec,
+	unsigned int iov_len,
 	int guarantee)
 {
 	struct totempg_group_instance *instance;
@@ -1125,7 +1121,7 @@ int totempg_groups_mcast_joined (
 	for (i = 0; i < instance->groups_cnt; i++) {
 		group_len[i + 1] = instance->groups[i].group_len;
 		iovec_mcast[i + 1].iov_len = instance->groups[i].group_len;
-		iovec_mcast[i + 1].iov_base = instance->groups[i].group;
+		iovec_mcast[i + 1].iov_base = (void *) instance->groups[i].group;
 	}
 	iovec_mcast[0].iov_len = (instance->groups_cnt + 1) * sizeof (unsigned short);
 	iovec_mcast[0].iov_base = group_len;
@@ -1144,8 +1140,8 @@ error_exit:
 
 int totempg_groups_joined_reserve (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	int iov_len)
+	const struct iovec *iovec,
+	unsigned int iov_len)
 {
 	struct totempg_group_instance *instance;
 	unsigned int size = 0;
@@ -1183,22 +1179,23 @@ error_exit:
 }
 
 
-void totempg_groups_joined_release (int msg_count)
+int totempg_groups_joined_release (int msg_count)
 {
 	pthread_mutex_lock (&totempg_mutex);
 	pthread_mutex_lock (&mcast_msg_mutex);
 	send_release (msg_count);
 	pthread_mutex_unlock (&mcast_msg_mutex);
 	pthread_mutex_unlock (&totempg_mutex);
+	return 0;
 }
 
 int totempg_groups_mcast_groups (
 	hdb_handle_t handle,
 	int guarantee,
-	struct totempg_group *groups,
-	int groups_cnt,
-	struct iovec *iovec,
-	int iov_len)
+	const struct totempg_group *groups,
+	size_t groups_cnt,
+	const struct iovec *iovec,
+	unsigned int iov_len)
 {
 	struct totempg_group_instance *instance;
 	unsigned short group_len[MAX_GROUPS_PER_MSG + 1];
@@ -1220,7 +1217,7 @@ int totempg_groups_mcast_groups (
 	for (i = 0; i < groups_cnt; i++) {
 		group_len[i + 1] = groups[i].group_len;
 		iovec_mcast[i + 1].iov_len = groups[i].group_len;
-		iovec_mcast[i + 1].iov_base = groups[i].group;
+		iovec_mcast[i + 1].iov_base = (void *) groups[i].group;
 	}
 	iovec_mcast[0].iov_len = (groups_cnt + 1) * sizeof (unsigned short);
 	iovec_mcast[0].iov_base = group_len;
@@ -1243,10 +1240,10 @@ error_exit:
  */
 int totempg_groups_send_ok_groups (
 	hdb_handle_t handle,
-	struct totempg_group *groups,
-	int groups_cnt,
-	struct iovec *iovec,
-	int iov_len)
+	const struct totempg_group *groups,
+	size_t groups_cnt,
+	const struct iovec *iovec,
+	unsigned int iov_len)
 {
 	struct totempg_group_instance *instance;
 	unsigned int size = 0;
@@ -1268,7 +1265,7 @@ int totempg_groups_send_ok_groups (
 	}
 
 	res = msg_count_send_ok (size);
-	 
+
 	hdb_handle_put (&totempg_groups_instance_database, handle);
 error_exit:
 	pthread_mutex_unlock (&totempg_mutex);
@@ -1292,6 +1289,17 @@ int totempg_ifaces_get (
 	return (res);
 }
 
+int totempg_crypto_set (
+	unsigned int type)
+{
+	int res;
+
+	res = totemmrp_crypto_set (
+		type);
+
+	return (res);
+}
+
 int totempg_ring_reenable (void)
 {
 	int res;
