@@ -1,13 +1,13 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -52,30 +52,31 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
-#include <signal.h>
 #include <sched.h>
 #include <time.h>
 #include <sys/time.h>
 #include <sys/poll.h>
+#include <limits.h>
 
-#include <corosync/queue.h>
 #include <corosync/sq.h>
 #include <corosync/list.h>
 #include <corosync/hdb.h>
 #include <corosync/swab.h>
 #include <corosync/totem/coropoll.h>
+#define LOGSYS_UTILS_ONLY 1
+#include <corosync/engine/logsys.h>
 
 #include "totemnet.h"
 #include "totemrrp.h"
 
 void rrp_deliver_fn (
 	void *context,
-	void *msg,
-	int msg_len);
+	const void *msg,
+	unsigned int msg_len);
 
 void rrp_iface_change_fn (
 	void *context,
-	struct totem_ip_address *iface_addr);
+	const struct totem_ip_address *iface_addr);
 
 struct totemrrp_instance;
 struct passive_instance {
@@ -116,31 +117,31 @@ struct rrp_algo {
 		struct totemrrp_instance *instance,
 		unsigned int iface_no,
 		void *context,
-		void *msg,
+		const void *msg,
 		unsigned int msg_len);
 
 	void (*mcast_noflush_send) (
 		struct totemrrp_instance *instance,
-		struct iovec *iovec,
-		unsigned int iov_len);
+		const void *msg,
+		unsigned int msg_len);
 
 	void (*mcast_flush_send) (
 		struct totemrrp_instance *instance,
-		struct iovec *iovec,
-		unsigned int iov_len);
+		const void *msg,
+		unsigned int msg_len);
 
 	void (*token_recv) (
 		struct totemrrp_instance *instance,
 		unsigned int iface_no,
 		void *context,
-		void *msg,
+		const void *msg,
 		unsigned int msg_len,
 		unsigned int token_seqid);
 
 	void (*token_send) (
 		struct totemrrp_instance *instance,
-		struct iovec *iovec,
-		unsigned int iov_len);	
+		const void *msg,
+		unsigned int msg_len);
 
 	void (*recv_flush) (
 		struct totemrrp_instance *instance);
@@ -162,6 +163,9 @@ struct rrp_algo {
 
 	void (*ring_reenable) (
 		struct totemrrp_instance *instance);
+
+	int (*mcast_recv_empty) (
+		struct totemrrp_instance *instance);
 };
 
 struct totemrrp_instance {
@@ -172,21 +176,21 @@ struct totemrrp_instance {
 	struct rrp_algo *rrp_algo;
 
 	void *context;
-	
+
 	char *status[INTERFACE_MAX];
 
 	void (*totemrrp_deliver_fn) (
 		void *context,
-		void *msg,
-		int msg_len);
+		const void *msg,
+		unsigned int msg_len);
 
 	void (*totemrrp_iface_change_fn) (
 		void *context,
-		struct totem_ip_address *iface_addr,
+		const struct totem_ip_address *iface_addr,
 		unsigned int iface_no);
 
 	void (*totemrrp_token_seqid_get) (
-		void *msg,
+		const void *msg,
 		unsigned int *seqid,
 		unsigned int *token_is);
 
@@ -207,9 +211,12 @@ struct totemrrp_instance {
 
 	int totemrrp_subsys_id;
 
-	void (*totemrrp_log_printf) (int subsys, const char *function,
-		const char *file, int line, unsigned int level,
-		const char *format, ...)__attribute__((format(printf, 6, 7)));
+	void (*totemrrp_log_printf) (
+		unsigned int rec_ident,
+		const char *function,
+		const char *file,
+		int line,
+		const char *format, ...)__attribute__((format(printf, 5, 6)));
 
 	hdb_handle_t handle;
 
@@ -233,31 +240,31 @@ static void none_mcast_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len);
 
 static void none_mcast_noflush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 static void none_mcast_flush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 static void none_token_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len,
 	unsigned int token_seqid);
 
 static void none_token_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);	
+	const void *msg,
+	unsigned int msg_len);
 
 static void none_recv_flush (
 	struct totemrrp_instance *instance);
@@ -280,6 +287,9 @@ static void none_token_target_set (
 static void none_ring_reenable (
 	struct totemrrp_instance *instance);
 
+static int none_mcast_recv_empty (
+	struct totemrrp_instance *instance);
+
 /*
  * Passive Replication Forward Declerations
  */
@@ -291,31 +301,31 @@ static void passive_mcast_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len);
 
 static void passive_mcast_noflush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 static void passive_mcast_flush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 static void passive_token_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len,
 	unsigned int token_seqid);
 
 static void passive_token_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);	
+	const void *msg,
+	unsigned int msg_len);
 
 static void passive_recv_flush (
 	struct totemrrp_instance *instance);
@@ -338,6 +348,9 @@ static void passive_token_target_set (
 static void passive_ring_reenable (
 	struct totemrrp_instance *instance);
 
+static int passive_mcast_recv_empty (
+	struct totemrrp_instance *instance);
+
 /*
  * Active Replication Forward Definitions
  */
@@ -349,31 +362,31 @@ static void active_mcast_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len);
 
 static void active_mcast_noflush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 static void active_mcast_flush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);
+	const void *msg,
+	unsigned int msg_len);
 
 static void active_token_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len,
 	unsigned int token_seqid);
 
 static void active_token_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len);	
+	const void *msg,
+	unsigned int msg_len);
 
 static void active_recv_flush (
 	struct totemrrp_instance *instance);
@@ -396,6 +409,9 @@ static void active_token_target_set (
 static void active_ring_reenable (
 	struct totemrrp_instance *instance);
 
+static int active_mcast_recv_empty (
+	struct totemrrp_instance *instance);
+
 static void active_timer_expired_token_start (
 	struct active_instance *active_instance);
 
@@ -421,7 +437,8 @@ struct rrp_algo none_algo = {
 	.iface_check		= none_iface_check,
 	.processor_count_set	= none_processor_count_set,
 	.token_target_set	= none_token_target_set,
-	.ring_reenable		= none_ring_reenable
+	.ring_reenable		= none_ring_reenable,
+	.mcast_recv_empty	= none_mcast_recv_empty
 };
 
 struct rrp_algo passive_algo = {
@@ -437,7 +454,8 @@ struct rrp_algo passive_algo = {
 	.iface_check		= passive_iface_check,
 	.processor_count_set	= passive_processor_count_set,
 	.token_target_set	= passive_token_target_set,
-	.ring_reenable		= passive_ring_reenable
+	.ring_reenable		= passive_ring_reenable,
+	.mcast_recv_empty	= passive_mcast_recv_empty
 };
 
 struct rrp_algo active_algo = {
@@ -453,7 +471,8 @@ struct rrp_algo active_algo = {
 	.iface_check		= active_iface_check,
 	.processor_count_set	= active_processor_count_set,
 	.token_target_set	= active_token_target_set,
-	.ring_reenable		= active_ring_reenable
+	.ring_reenable		= active_ring_reenable,
+	.mcast_recv_empty	= active_mcast_recv_empty
 };
 
 struct rrp_algo *rrp_algos[] = {
@@ -467,19 +486,15 @@ struct rrp_algo *rrp_algos[] = {
 /*
  * All instances in one database
  */
-static struct hdb_handle_database totemrrp_instance_database = {
-	.handle_count	= 0,
-	.handles	= 0,
-	.iterator	= 0,
-	.mutex		= PTHREAD_MUTEX_INITIALIZER
-};
-
+DECLARE_HDB_DATABASE (totemrrp_instance_database,NULL);
 
 #define log_printf(level, format, args...)				\
 do {									\
 	rrp_instance->totemrrp_log_printf (				\
-		rrp_instance->totemrrp_subsys_id,			\
-		__FUNCTION__, __FILE__, __LINE__, level,		\
+		LOGSYS_ENCODE_RECID(level,				\
+				    rrp_instance->totemrrp_subsys_id,	\
+				    LOGSYS_RECID_LOG),			\
+		__FUNCTION__, __FILE__, __LINE__,			\
 		format, ##args);					\
 } while (0);
 
@@ -491,7 +506,7 @@ static void none_mcast_recv (
 	struct totemrrp_instance *rrp_instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len)
 {
 	rrp_instance->totemrrp_deliver_fn (
@@ -502,25 +517,25 @@ static void none_mcast_recv (
 
 static void none_mcast_flush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
-	totemnet_mcast_flush_send (instance->net_handles[0], iovec, iov_len);
+	totemnet_mcast_flush_send (instance->net_handles[0], msg, msg_len);
 }
 
 static void none_mcast_noflush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
-	totemnet_mcast_noflush_send (instance->net_handles[0], iovec, iov_len);
+	totemnet_mcast_noflush_send (instance->net_handles[0], msg, msg_len);
 }
 
 static void none_token_recv (
 	struct totemrrp_instance *rrp_instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len,
 	unsigned int token_seq)
 {
@@ -532,12 +547,12 @@ static void none_token_recv (
 
 static void none_token_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	totemnet_token_send (
 		instance->net_handles[0],
-		iovec, iov_len);
+		msg, msg_len);
 }
 
 static void none_recv_flush (struct totemrrp_instance *instance)
@@ -579,6 +594,16 @@ static void none_ring_reenable (
 	 */
 }
 
+static int none_mcast_recv_empty (
+	struct totemrrp_instance *instance)
+{
+	int res;
+
+	res = totemnet_recv_mcast_empty (instance->net_handles[0]);
+
+	return (res);
+}
+
 /*
  * Passive Replication Implementation
  */
@@ -691,7 +716,7 @@ static void passive_mcast_recv (
 	struct totemrrp_instance *rrp_instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len)
 {
 	struct passive_instance *passive_instance = (struct passive_instance *)rrp_instance->rrp_algo_instance;
@@ -729,10 +754,10 @@ static void passive_mcast_recv (
 
 	for (i = 0; i < rrp_instance->interface_count; i++) {
 		if ((passive_instance->faulty[i] == 0) &&
-			(max - passive_instance->mcast_recv_count[i] > 
+			(max - passive_instance->mcast_recv_count[i] >
 			rrp_instance->totem_config->rrp_problem_count_threshold)) {
 			passive_instance->faulty[i] = 1;
-			sprintf (rrp_instance->status[i], 
+			sprintf (rrp_instance->status[i],
 				"Marking ringid %u interface %s FAULTY - adminisrtative intervention required.",
 				i,
 				totemnet_iface_print (rrp_instance->net_handles[i]));
@@ -746,38 +771,38 @@ static void passive_mcast_recv (
 
 static void passive_mcast_flush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	struct passive_instance *passive_instance = (struct passive_instance *)instance->rrp_algo_instance;
 
 	do {
 		passive_instance->msg_xmit_iface = (passive_instance->msg_xmit_iface + 1) % instance->interface_count;
 	} while (passive_instance->faulty[passive_instance->msg_xmit_iface] == 1);
-	
-	totemnet_mcast_flush_send (instance->net_handles[passive_instance->msg_xmit_iface], iovec, iov_len);
+
+	totemnet_mcast_flush_send (instance->net_handles[passive_instance->msg_xmit_iface], msg, msg_len);
 }
 
 static void passive_mcast_noflush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	struct passive_instance *passive_instance = (struct passive_instance *)instance->rrp_algo_instance;
 
 	do {
 		passive_instance->msg_xmit_iface = (passive_instance->msg_xmit_iface + 1) % instance->interface_count;
 	} while (passive_instance->faulty[passive_instance->msg_xmit_iface] == 1);
-	
-	
-	totemnet_mcast_noflush_send (instance->net_handles[passive_instance->msg_xmit_iface], iovec, iov_len);
+
+
+	totemnet_mcast_noflush_send (instance->net_handles[passive_instance->msg_xmit_iface], msg, msg_len);
 }
 
 static void passive_token_recv (
 	struct totemrrp_instance *rrp_instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len,
 	unsigned int token_seq)
 {
@@ -812,10 +837,10 @@ static void passive_token_recv (
 
 	for (i = 0; i < rrp_instance->interface_count; i++) {
 		if ((passive_instance->faulty[i] == 0) &&
-			(max - passive_instance->token_recv_count[i] > 
+			(max - passive_instance->token_recv_count[i] >
 			rrp_instance->totem_config->rrp_problem_count_threshold)) {
 			passive_instance->faulty[i] = 1;
-			sprintf (rrp_instance->status[i], 
+			sprintf (rrp_instance->status[i],
 				"Marking seqid %d ringid %u interface %s FAULTY - adminisrtative intervention required.",
 				token_seq,
 				i,
@@ -830,18 +855,18 @@ static void passive_token_recv (
 
 static void passive_token_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	struct passive_instance *passive_instance = (struct passive_instance *)instance->rrp_algo_instance;
 
 	do {
 		passive_instance->token_xmit_iface = (passive_instance->token_xmit_iface + 1) % instance->interface_count;
 	} while (passive_instance->faulty[passive_instance->token_xmit_iface] == 1);
-	
+
 	totemnet_token_send (
 		instance->net_handles[passive_instance->token_xmit_iface],
-		iovec, iov_len);
+		msg, msg_len);
 
 }
 
@@ -908,6 +933,26 @@ static void passive_token_target_set (
 	totemnet_token_target_set (instance->net_handles[iface_no], token_target);
 }
 
+static int passive_mcast_recv_empty (
+	struct totemrrp_instance *instance)
+{
+	int res;
+	int msgs_emptied = 0;
+	int i;
+
+	for (i = 0; i < instance->interface_count; i++) {
+		res = totemnet_recv_mcast_empty (instance->net_handles[i]);
+		if (res == -1) {
+			return (-1);
+		}
+		if (res == 1) {
+			msgs_emptied = 1;
+		}
+	}
+
+	return (msgs_emptied);
+}
+
 static void passive_ring_reenable (
 	struct totemrrp_instance *instance)
 {
@@ -978,16 +1023,16 @@ static void timer_function_active_problem_decrementer (void *context)
 	struct totemrrp_instance *rrp_instance = active_instance->rrp_instance;
 	unsigned int problem_found = 0;
 	unsigned int i;
-	
+
 	for (i = 0; i < rrp_instance->interface_count; i++) {
 		if (active_instance->counter_problems[i] > 0) {
 			problem_found = 1;
 			active_instance->counter_problems[i] -= 1;
 			if (active_instance->counter_problems[i] == 0) {
-				sprintf (rrp_instance->status[i], 
+				sprintf (rrp_instance->status[i],
 					"ring %d active with no faults", i);
 			} else {
-				sprintf (rrp_instance->status[i], 
+				sprintf (rrp_instance->status[i],
 					"Decrementing problem counter for iface %s to [%d of %d]",
 					totemnet_iface_print (rrp_instance->net_handles[i]),
 					active_instance->counter_problems[i],
@@ -1019,7 +1064,7 @@ static void timer_function_active_token_expired (void *context)
 			if (active_instance->timer_problem_decrementer == 0) {
 				active_timer_problem_decrementer_start (active_instance);
 			}
-			sprintf (rrp_instance->status[i], 
+			sprintf (rrp_instance->status[i],
 				"Incrementing problem counter for seqid %d iface %s to [%d of %d]",
 				active_instance->last_token_seq,
 				totemnet_iface_print (rrp_instance->net_handles[i]),
@@ -1035,7 +1080,7 @@ static void timer_function_active_token_expired (void *context)
 		if (active_instance->counter_problems[i] >= rrp_instance->totem_config->rrp_problem_count_threshold)
 		{
 			active_instance->faulty[i] = 1;
-			sprintf (rrp_instance->status[i], 
+			sprintf (rrp_instance->status[i],
 				"Marking seqid %d ringid %u interface %s FAULTY - adminisrtative intervention required.",
 				active_instance->last_token_seq,
 				i,
@@ -1100,7 +1145,7 @@ static void active_mcast_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len)
 {
 	instance->totemrrp_deliver_fn (
@@ -1111,30 +1156,30 @@ static void active_mcast_recv (
 
 static void active_mcast_flush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	int i;
 	struct active_instance *rrp_algo_instance = (struct active_instance *)instance->rrp_algo_instance;
 
 	for (i = 0; i < instance->interface_count; i++) {
 		if (rrp_algo_instance->faulty[i] == 0) {
-			totemnet_mcast_flush_send (instance->net_handles[i], iovec, iov_len);
+			totemnet_mcast_flush_send (instance->net_handles[i], msg, msg_len);
 		}
 	}
 }
 
 static void active_mcast_noflush_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	int i;
 	struct active_instance *rrp_algo_instance = (struct active_instance *)instance->rrp_algo_instance;
 
 	for (i = 0; i < instance->interface_count; i++) {
 		if (rrp_algo_instance->faulty[i] == 0) {
-			totemnet_mcast_noflush_send (instance->net_handles[i], iovec, iov_len);
+			totemnet_mcast_noflush_send (instance->net_handles[i], msg, msg_len);
 		}
 	}
 }
@@ -1143,7 +1188,7 @@ static void active_token_recv (
 	struct totemrrp_instance *instance,
 	unsigned int iface_no,
 	void *context,
-	void *msg,
+	const void *msg,
 	unsigned int msg_len,
 	unsigned int token_seq)
 {
@@ -1183,8 +1228,8 @@ static void active_token_recv (
 
 static void active_token_send (
 	struct totemrrp_instance *instance,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	struct active_instance *rrp_algo_instance = (struct active_instance *)instance->rrp_algo_instance;
 	int i;
@@ -1193,7 +1238,7 @@ static void active_token_send (
 		if (rrp_algo_instance->faulty[i] == 0) {
 			totemnet_token_send (
 				instance->net_handles[i],
-				iovec, iov_len);
+				msg, msg_len);
 
 		}
 	}
@@ -1262,6 +1307,26 @@ static void active_token_target_set (
 	totemnet_token_target_set (instance->net_handles[iface_no], token_target);
 }
 
+static int active_mcast_recv_empty (
+	struct totemrrp_instance *instance)
+{
+	int res;
+	int msgs_emptied = 0;
+	int i;
+
+	for (i = 0; i < instance->interface_count; i++) {
+		res = totemnet_recv_mcast_empty (instance->net_handles[i]);
+		if (res == -1) {
+			return (-1);
+		}
+		if (res == 1) {
+			msgs_emptied = 1;
+		}
+	}
+
+	return (msgs_emptied);
+}
+
 static void active_ring_reenable (
 	struct totemrrp_instance *instance)
 {
@@ -1314,8 +1379,8 @@ static int totemrrp_algorithm_set (
 
 void rrp_deliver_fn (
 	void *context,
-	void *msg,
-	int msg_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	unsigned int token_seqid;
 	unsigned int token_is;
@@ -1329,7 +1394,7 @@ void rrp_deliver_fn (
 
 	if (token_is) {
 		/*
-		 * Deliver to the token receiver for this rrp algorithm 
+		 * Deliver to the token receiver for this rrp algorithm
 		 */
 		deliver_fn_context->instance->rrp_algo->token_recv (
 			deliver_fn_context->instance,
@@ -1340,7 +1405,7 @@ void rrp_deliver_fn (
 			token_seqid);
 	} else {
 		/*
-		 * Deliver to the mcast receiver for this rrp algorithm 
+		 * Deliver to the mcast receiver for this rrp algorithm
 		 */
 		deliver_fn_context->instance->rrp_algo->mcast_recv (
 			deliver_fn_context->instance,
@@ -1353,7 +1418,7 @@ void rrp_deliver_fn (
 
 void rrp_iface_change_fn (
 	void *context,
-	struct totem_ip_address *iface_addr)
+	const struct totem_ip_address *iface_addr)
 {
 	struct deliver_fn_context *deliver_fn_context = (struct deliver_fn_context *)context;
 
@@ -1403,16 +1468,16 @@ int totemrrp_initialize (
 
 	void (*deliver_fn) (
 		void *context,
-		void *msg,
-		int msg_len),
+		const void *msg,
+		unsigned int msg_len),
 
 	void (*iface_change_fn) (
 		void *context,
-		struct totem_ip_address *iface_addr,
+		const struct totem_ip_address *iface_addr,
 		unsigned int iface_no),
 
 	void (*token_seqid_get) (
-		void *msg,
+		const void *msg,
 		unsigned int *seqid,
 		unsigned int *token_is),
 
@@ -1584,7 +1649,7 @@ int totemrrp_send_flush (hdb_handle_t handle)
 		res = ENOENT;
 		goto error_exit;
 	}
-	
+
 	instance->rrp_algo->send_flush (instance);
 
 	hdb_handle_put (&totemrrp_instance_database, handle);
@@ -1595,8 +1660,8 @@ error_exit:
 
 int totemrrp_token_send (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
@@ -1608,7 +1673,7 @@ int totemrrp_token_send (
 		goto error_exit;
 	}
 
-	instance->rrp_algo->token_send (instance, iovec, iov_len);
+	instance->rrp_algo->token_send (instance, msg, msg_len);
 
 	hdb_handle_put (&totemrrp_instance_database, handle);
 
@@ -1618,8 +1683,8 @@ error_exit:
 
 int totemrrp_mcast_flush_send (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
@@ -1630,9 +1695,9 @@ int totemrrp_mcast_flush_send (
 		res = ENOENT;
 		goto error_exit;
 	}
-	
+
 // TODO this needs to return the result
-	instance->rrp_algo->mcast_flush_send (instance, iovec, iov_len);
+	instance->rrp_algo->mcast_flush_send (instance, msg, msg_len);
 
 	hdb_handle_put (&totemrrp_instance_database, handle);
 error_exit:
@@ -1641,8 +1706,8 @@ error_exit:
 
 int totemrrp_mcast_noflush_send (
 	hdb_handle_t handle,
-	struct iovec *iovec,
-	unsigned int iov_len)
+	const void *msg,
+	unsigned int msg_len)
 {
 	struct totemrrp_instance *instance;
 	int res = 0;
@@ -1653,7 +1718,7 @@ int totemrrp_mcast_noflush_send (
 		res = ENOENT;
 		goto error_exit;
 	}
-	
+
 	/*
 	 * merge detects go out through mcast_flush_send so it is safe to
 	 * flush these messages if we are only one processor.  This avoids
@@ -1662,7 +1727,7 @@ int totemrrp_mcast_noflush_send (
 	if (instance->processor_count > 1) {
 
 // TODO this needs to return the result
-		instance->rrp_algo->mcast_noflush_send (instance, iovec, iov_len);
+		instance->rrp_algo->mcast_noflush_send (instance, msg, msg_len);
 	}
 
 	hdb_handle_put (&totemrrp_instance_database, handle);
@@ -1681,7 +1746,7 @@ int totemrrp_iface_check (hdb_handle_t handle)
 		res = ENOENT;
 		goto error_exit;
 	}
-	
+
 	instance->rrp_algo->iface_check (instance);
 
 	hdb_handle_put (&totemrrp_instance_database, handle);
@@ -1703,9 +1768,9 @@ int totemrrp_ifaces_get (
 		res = ENOENT;
 		goto error_exit;
 	}
-	
+
 	*status = instance->status;
-	
+
 	if (iface_count) {
 		*iface_count = instance->interface_count;
 	}
@@ -1716,6 +1781,26 @@ error_exit:
 	return (res);
 }
 
+int totemrrp_crypto_set (
+	hdb_handle_t handle,
+	unsigned int type)
+{
+	int res;
+	struct totemrrp_instance *instance;
+
+	res = hdb_handle_get (&totemrrp_instance_database, handle,
+		(void *)&instance);
+	if (res != 0) {
+		return (0);
+	}
+
+	res = totemnet_crypto_set(instance->net_handles[0], type);
+
+	hdb_handle_put (&totemrrp_instance_database, handle);
+	return (res);
+}
+
+
 int totemrrp_ring_reenable (
         hdb_handle_t handle)
 {
@@ -1730,7 +1815,7 @@ printf ("totemrrp ring reenable\n");
 		res = ENOENT;
 		goto error_exit;
 	}
-	
+
 	instance->rrp_algo->ring_reenable (instance);
 
 	for (i = 0; i < instance->interface_count; i++) {
@@ -1742,3 +1827,25 @@ printf ("totemrrp ring reenable\n");
 error_exit:
 	return (res);
 }
+
+extern int totemrrp_mcast_recv_empty (
+        hdb_handle_t handle)
+{
+	struct totemrrp_instance *instance;
+	int res;
+
+	res = hdb_handle_get (&totemrrp_instance_database, handle,
+		(void *)&instance);
+	if (res != 0) {
+		res = ENOENT;
+		goto error_exit;
+	}
+
+	res = instance->rrp_algo->mcast_recv_empty (instance);
+
+	hdb_handle_put (&totemrrp_instance_database, handle);
+
+error_exit:
+	return (res);
+}
+
