@@ -1,14 +1,14 @@
 /*
  * Copyright (c) 2002-2004 MontaVista Software, Inc.
  * Copyright (c) 2004 Open Source Development Lab
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006-2007, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com), Mark Haverkamp (markh@osdl.org)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -41,23 +41,25 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/time.h>
+#include <assert.h>
 
 #include <corosync/corotypes.h>
 #include <corosync/list.h>
 #include <corosync/engine/logsys.h>
+#include <corosync/coroipc_types.h>
 #include "util.h"
 
-LOGSYS_DECLARE_SUBSYS ("MAIN", LOG_INFO);
+LOGSYS_DECLARE_SUBSYS ("MAIN");
 
 /*
  * Compare two names.  returns non-zero on match.
  */
-int name_match(cs_name_t *name1, cs_name_t *name2) 
+int name_match(cs_name_t *name1, cs_name_t *name2)
 {
 	if (name1->length == name2->length) {
 		return ((strncmp ((char *)name1->value, (char *)name2->value,
 			name1->length)) == 0);
-	} 
+	}
 	return 0;
 }
 
@@ -88,8 +90,10 @@ void _corosync_out_of_memory_error (void)
 void _corosync_exit_error (
 	enum e_ais_done err, const char *file, unsigned int line)
 {
-	log_printf (LOG_LEVEL_ERROR, "AIS Executive exiting "
+	log_printf (LOGSYS_LEVEL_ERROR, "Corosync Cluster Engine exiting "
 		"with status %d at %s:%u.\n", err, file, line);
+	logsys_fork_completed ();
+	logsys_flush ();
 	exit (err);
 }
 
@@ -108,32 +112,15 @@ char *getcs_name_t (cs_name_t *name)
 	return ((char *)name->value);
 }
 
-char *strstr_rs (const char *haystack, const char *needle)
+char *strchr_rs (const char *haystack, int byte)
 {
-	char *end_address;
-	char *new_needle;
-
-	new_needle = (char *)strdup (needle);
-	new_needle[strlen (new_needle) - 1] = '\0';
-
-	end_address = strstr (haystack, new_needle);
-	if (end_address) {
-		end_address += strlen (new_needle);
-		end_address = strstr (end_address, needle + strlen (new_needle));
-	}
+	const char *end_address = strchr (haystack, byte);
 	if (end_address) {
 		end_address += 1; /* skip past { or = */
-		do {
-			if (*end_address == '\t' || *end_address == ' ') {
-				end_address++;
-			} else {
-				break;
-			}
-		} while (*end_address != '\0');
+		end_address += strspn (end_address, " \t");
 	}
 
-	free (new_needle);
-	return (end_address);
+	return ((char *) end_address);
 }
 
 void setcs_name_t (cs_name_t *name, char *str) {
@@ -153,4 +140,3 @@ int cs_name_tisEqual (cs_name_t *str1, char *str2) {
 		return 0;
 	}
 }
-
