@@ -7,7 +7,7 @@
  * Author: Steven Dake (sdake@redhat.com), Mark Haverkamp (markh@osdl.org)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -36,7 +36,6 @@
 #define UTIL_H_DEFINED
 
 #include <sys/time.h>
-#include <corosync/mar_gen.h>
 #include <corosync/corotypes.h>
 
 /*
@@ -60,21 +59,21 @@ enum e_ais_done {
 	AIS_DONE_OBJDB = -13,
 	AIS_DONE_INIT_SERVICES = -14,
 	AIS_DONE_OUT_OF_MEMORY = -15,
-	AIS_DONE_FATAL_ERR = -16
+	AIS_DONE_FATAL_ERR = -16,
+	AIS_DONE_DIR_NOT_PRESENT = -17
 };
 
 /*
  * Compare two names.  returns non-zero on match.
  */
 extern int name_match(cs_name_t *name1, cs_name_t *name2);
-extern int mar_name_match(mar_name_t *name1, mar_name_t *name2);
 #define corosync_exit_error(err) _corosync_exit_error ((err), __FILE__, __LINE__)
-extern void _corosync_exit_error (
-	enum e_ais_done err, const char *file, unsigned int line);
-void _corosync_out_of_memory_error (void);
+extern void _corosync_exit_error (enum e_ais_done err, const char *file,
+				  unsigned int line)
+  __attribute__((__noreturn__));
+void _corosync_out_of_memory_error (void) __attribute__((__noreturn__));
 extern char *getcs_name_t (cs_name_t *name);
-extern char *strstr_rs (const char *haystack, const char *needle);
+extern char *strchr_rs (const char *haystack, int byte);
 extern void setcs_name_t (cs_name_t *name, char *str);
-char *get_mar_name_t (mar_name_t *name);
 extern int cs_name_tisEqual (cs_name_t *str1, char *str2);
 #endif /* UTIL_H_DEFINED */
