@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008 Red Hat, Inc.
+ * Copyright (c) 2008, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -37,6 +37,10 @@
 #include <corosync/corotypes.h>
 #include <corosync/hdb.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * @addtogroup confdb_corosync
  *
@@ -62,25 +66,25 @@ typedef void (*confdb_key_change_notify_fn_t) (
 	confdb_change_type_t change_type,
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t object_handle,
-	void *object_name,
-	int  object_name_len,
-	void *key_name,
-	int key_name_len,
-	void *key_value,
-	int key_value_len);
+	const void *object_name,
+	size_t  object_name_len,
+	const void *key_name,
+	size_t key_name_len,
+	const void *key_value,
+	size_t key_value_len);
 
 typedef void (*confdb_object_create_notify_fn_t) (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t object_handle,
-	uint8_t *name_pt,
-	int  name_len);
+	const void *name_pt,
+	size_t name_len);
 
 typedef void (*confdb_object_delete_notify_fn_t) (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
-	uint8_t *name_pt,
-	int  name_len);
+	const void *name_pt,
+	size_t name_len);
 
 typedef struct {
 	confdb_object_create_notify_fn_t confdb_object_create_change_notify_fn;
@@ -109,7 +113,8 @@ cs_error_t confdb_finalize (
  */
 cs_error_t confdb_write (
 	confdb_handle_t handle,
-	char *error_text);
+	char *error_text,
+	size_t errbuf_len);
 
 /*
  * Reload the configuration
@@ -117,7 +122,8 @@ cs_error_t confdb_write (
 cs_error_t confdb_reload (
 	confdb_handle_t handle,
 	int flush,
-	char *error_text);
+	char *error_text,
+	size_t errbuf_len);
 
 /*
  * Get a file descriptor on which to poll.  confdb_handle_t is NOT a
@@ -152,7 +158,7 @@ cs_error_t confdb_object_create (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *object_name,
-	int object_name_len,
+	size_t object_name_len,
 	hdb_handle_t *object_handle);
 
 cs_error_t confdb_object_destroy (
@@ -171,17 +177,17 @@ cs_error_t confdb_key_create (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	const void *value,
-	int value_len);
+	size_t value_len);
 
 cs_error_t confdb_key_delete (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	const void *value,
-	int value_len);
+	size_t value_len);
 
 /*
  * Key queries
@@ -190,32 +196,32 @@ cs_error_t confdb_key_get (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	void *value,
-	int *value_len);
+	size_t *value_len);
 
 cs_error_t confdb_key_replace (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	const void *old_value,
-	int old_value_len,
+	size_t old_value_len,
 	const void *new_value,
-	int new_value_len);
+	size_t new_value_len);
 
 cs_error_t confdb_key_increment (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	unsigned int *value);
 
 cs_error_t confdb_key_decrement (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	unsigned int *value);
 
 /*
@@ -232,7 +238,7 @@ cs_error_t confdb_object_find (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *object_name,
-	int object_name_len,
+	size_t object_name_len,
 	hdb_handle_t *object_handle);
 
 cs_error_t confdb_object_find_destroy(
@@ -248,7 +254,7 @@ cs_error_t confdb_object_iter (
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t *object_handle,
 	void *object_name,
-	int *object_name_len);
+	size_t *object_name_len);
 
 cs_error_t confdb_object_iter_destroy(
 	confdb_handle_t handle,
@@ -265,9 +271,9 @@ cs_error_t confdb_key_iter (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	void *key_name,
-	int *key_name_len,
+	size_t *key_name_len,
 	void *value,
-	int *value_len);
+	size_t *value_len);
 
 /*
  * Get/set context variable
@@ -281,4 +287,8 @@ cs_error_t confdb_context_set (
 	confdb_handle_t handle,
 	const void *context);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* COROSYNC_CONFDB_H_DEFINED */
