@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -37,6 +37,10 @@
 #include <netinet/in.h>
 #include <corosync/corotypes.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * @addtogroup cpg_corosync
  *
@@ -81,30 +85,26 @@ struct cpg_name {
 
 typedef void (*cpg_deliver_fn_t) (
 	cpg_handle_t handle,
-	struct cpg_name *group_name,
+	const struct cpg_name *group_name,
 	uint32_t nodeid,
 	uint32_t pid,
+	/*
+	 * Unlike many "msg" pointers, this one is deliberately *not*
+	 * declared const in order to permit in-place endian conversion.
+	 */
 	void *msg,
-	int msg_len);
+	size_t msg_len);
 
 typedef void (*cpg_confchg_fn_t) (
 	cpg_handle_t handle,
-	struct cpg_name *group_name,
-	struct cpg_address *member_list, int member_list_entries,
-	struct cpg_address *left_list, int left_list_entries,
-	struct cpg_address *joined_list, int joined_list_entries);
-
-typedef void (*cpg_groups_get_fn_t) (
-	cpg_handle_t handle,
-	uint32_t group_num,
-	uint32_t group_total,
-	struct cpg_name *group_name,
-	struct cpg_address *member_list, int member_list_entries);
+	const struct cpg_name *group_name,
+	const struct cpg_address *member_list, size_t member_list_entries,
+	const struct cpg_address *left_list, size_t left_list_entries,
+	const struct cpg_address *joined_list, size_t joined_list_entries);
 
 typedef struct {
 	cpg_deliver_fn_t cpg_deliver_fn;
 	cpg_confchg_fn_t cpg_confchg_fn;
-	cpg_groups_get_fn_t cpg_groups_get_fn;
 } cpg_callbacks_t;
 
 /** @} */
@@ -130,7 +130,7 @@ cs_error_t cpg_fd_get (
 	cpg_handle_t handle,
 	int *fd);
 
-/* 
+/*
  * Get and set contexts for a CPG handle
  */
 cs_error_t cpg_context_get (
@@ -157,14 +157,14 @@ cs_error_t cpg_dispatch (
  */
 cs_error_t cpg_join (
 	cpg_handle_t handle,
-	struct cpg_name *group);
+	const struct cpg_name *group);
 
 /*
  * Leave one or more groups
  */
 cs_error_t cpg_leave (
 	cpg_handle_t handle,
-	struct cpg_name *group);
+	const struct cpg_name *group);
 
 /*
  * Multicast to groups joined with cpg_join.
@@ -174,8 +174,8 @@ cs_error_t cpg_leave (
 cs_error_t cpg_mcast_joined (
 	cpg_handle_t handle,
 	cpg_guarantee_t guarantee,
-	struct iovec *iovec,
-	int iov_len);
+	const struct iovec *iovec,
+	unsigned int iov_len);
 
 /*
  * Get membership information from cpg
@@ -190,12 +190,27 @@ cs_error_t cpg_local_get (
 	cpg_handle_t handle,
 	unsigned int *local_nodeid);
 
-cs_error_t cpg_groups_get (
-	cpg_handle_t handle,
-	unsigned int *num_groups);
-
 cs_error_t cpg_flow_control_state_get (
 	cpg_handle_t handle,
 	cpg_flow_control_state_t *flow_control_enabled);
 
+cs_error_t cpg_zcb_alloc (
+	cpg_handle_t handle,
+	size_t size,
+	void **buffer);
+
+cs_error_t cpg_zcb_free (
+	cpg_handle_t handle,
+	void *buffer);
+
+cs_error_t cpg_zcb_mcast_joined (
+	cpg_handle_t handle,
+	cpg_guarantee_t guarantee,
+	void *msg,
+	size_t msg_len);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* COROSYNC_CPG_H_DEFINED */
