@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008 Red Hat, Inc.
+ * Copyright (c) 2008, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -39,12 +39,28 @@
 #include <sys/uio.h>
 #endif
 #include <corosync/hdb.h>
+#include <corosync/swab.h>
 
+typedef struct {
+	uint32_t nodeid __attribute__((aligned(8)));
+	void *conn __attribute__((aligned(8)));
+} mar_message_source_t __attribute__((aligned(8)));
+
+static inline void swab_mar_message_source_t (mar_message_source_t *to_swab)
+{
+	swab32 (to_swab->nodeid);
+	/*
+	 * if it is from a byteswapped machine, then we can safely
+	 * ignore its conn info data structure since this is only
+	 * local to the machine
+	 */
+	to_swab->conn = NULL;
+}
 typedef void * corosync_timer_handle_t;
 
 struct corosync_tpg_group {
-	void *group;
-	int group_len;
+	const void *group;
+	size_t group_len;
 };
 
 #define TOTEMIP_ADDRLEN (sizeof(struct in6_addr))
@@ -118,6 +134,11 @@ enum cs_flow_control_state {
 
 #endif /* COROSYNC_FLOW_CONTROL_STATE */
 
+enum cs_sync_mode {
+	CS_SYNC_V1	= 0,
+	CS_SYNC_V2	= 1
+};
+
 typedef enum {
 	COROSYNC_FATAL_ERROR_EXIT = -1,
 	COROSYNC_LIBAIS_SOCKET = -6,
@@ -136,13 +157,14 @@ typedef enum {
 
 struct object_valid {
 	char *object_name;
-	int object_len;
+	size_t object_len;
 };
 
 struct object_key_valid {
 	char *key_name;
-	int key_len;
-	int (*validate_callback) (void *key, int key_len, void *value, int value_len);
+	size_t key_len;
+	int (*validate_callback) (const void *key, size_t key_len,
+				  const void *value, size_t value_len);
 };
 /* deprecated */
 
@@ -167,28 +189,28 @@ typedef void (*object_key_change_notify_fn_t)(
 	object_change_type_t change_type,
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t object_handle,
-	void *object_name_pt, int object_name_len,
-	void *key_name_pt, int key_len,
-	void *key_value_pt, int key_value_len,
+	const void *object_name_pt, size_t object_name_len,
+	const void *key_name_pt, size_t key_len,
+	const void *key_value_pt, size_t key_value_len,
 	void *priv_data_pt);
 
 typedef void (*object_create_notify_fn_t) (
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t object_handle,
-	uint8_t *name_pt, int name_len,
+	const uint8_t *name_pt, size_t name_len,
 	void *priv_data_pt);
 
 typedef void (*object_destroy_notify_fn_t) (
 	hdb_handle_t parent_object_handle,
-	uint8_t *name_pt, int name_len,
+	const uint8_t *name_pt, size_t name_len,
 	void *priv_data_pt);
 
 typedef void (*object_notify_callback_fn_t)(
 	hdb_handle_t object_handle,
-	void *key_name, int key_len,
-	void *value, int value_len,
+	const void *key_name, size_t key_len,
+	const void *value, size_t value_len,
 	object_change_type_t type,
-	void * priv_data_pt);
+	const void * priv_data_pt);
 
 typedef void (*object_reload_notify_fn_t) (
 	objdb_reload_notify_type_t,
@@ -208,14 +230,13 @@ struct quorum_callin_functions
 };
 
 typedef void (*sync_callback_fn_t) (
-	unsigned int *view_list,
-	int view_list_entries,
+	const unsigned int *view_list,
+	size_t view_list_entries,
 	int primary_designated,
 	struct memb_ring_id *ring_id);
 
 #endif /* QUORUM_H_DEFINED */
 
-
 struct corosync_api_v1 {
 	/*
 	 * Object and configuration APIs
@@ -223,7 +244,8 @@ struct corosync_api_v1 {
 	int (*object_create) (
 		hdb_handle_t parent_object_handle,
 		hdb_handle_t *object_handle,
-		void *object_name, unsigned int object_name_len);
+		const void *object_name,
+		size_t object_name_len);
 
 	int (*object_priv_set) (
 		hdb_handle_t object_handle,
@@ -231,10 +253,10 @@ struct corosync_api_v1 {
 
 	int (*object_key_create) (
 		hdb_handle_t object_handle,
-		void *key_name,
-		int key_len,
-		void *value,
-		int value_len);
+		const void *key_name,
+		size_t key_len,
+		const void *value,
+		size_t value_len);
 
 	int (*object_destroy) (
 		hdb_handle_t object_handle);
@@ -242,17 +264,17 @@ struct corosync_api_v1 {
 	int (*object_valid_set) (
 		hdb_handle_t object_handle,
 		struct object_valid *object_valid_list,
-		unsigned int object_valid_list_entries);
+		size_t object_valid_list_entries);
 
 	int (*object_key_valid_set) (
 		hdb_handle_t object_handle,
 		struct object_key_valid *object_key_valid_list,
-		unsigned int object_key_valid_list_entries);
+		size_t object_key_valid_list_entries);
 
 	int (*object_find_create) (
 		hdb_handle_t parent_object_handle,
-		void *object_name,
-		int object_name_len,
+		const void *object_name,
+		size_t object_name_len,
 		hdb_handle_t *object_find_handle);
 
 	int (*object_find_next) (
@@ -264,10 +286,10 @@ struct corosync_api_v1 {
 
 	int (*object_key_get) (
 		hdb_handle_t object_handle,
-		void *key_name,
-		int key_len,
+		const void *key_name,
+		size_t key_len,
 		void **value,
-		int *value_len);
+		size_t *value_len);
 
 	int (*object_priv_get) (
 		hdb_handle_t jobject_handle,
@@ -275,15 +297,15 @@ struct corosync_api_v1 {
 
 	int (*object_key_replace) (
 		hdb_handle_t object_handle,
-		void *key_name,
-		int key_len,
-		void *new_value,
-		int new_value_len);
+		const void *key_name,
+		size_t key_len,
+		const void *new_value,
+		size_t new_value_len);
 
 	int (*object_key_delete) (
 		hdb_handle_t object_handle,
-		void *key_name,
-		int key_len);
+		const void *key_name,
+		size_t key_len);
 
 	int (*object_iter_reset) (
 		hdb_handle_t parent_object_handle);
@@ -291,7 +313,7 @@ struct corosync_api_v1 {
 	int (*object_iter) (
 		hdb_handle_t parent_object_handle,
 		void **object_name,
-		int *name_len,
+		size_t *name_len,
 		hdb_handle_t *object_handle);
 
 	int (*object_key_iter_reset) (
@@ -300,9 +322,9 @@ struct corosync_api_v1 {
 	int (*object_key_iter) (
 		hdb_handle_t parent_object_handle,
 		void **key_name,
-		int *key_len,
+		size_t *key_len,
 		void **value,
-		int *value_len);
+		size_t *value_len);
 
 	int (*object_parent_get) (
 		hdb_handle_t object_handle,
@@ -311,7 +333,7 @@ struct corosync_api_v1 {
 	int (*object_name_get) (
 		hdb_handle_t object_handle,
 		char *object_name,
-		int *object_name_len);
+		size_t *object_name_len);
 
 	int (*object_dump) (
 		hdb_handle_t object_handle,
@@ -321,9 +343,9 @@ struct corosync_api_v1 {
 		hdb_handle_t parent_object_handle,
 		hdb_handle_t start_pos,
 		void **key_name,
-		int *key_len,
+		size_t *key_len,
 		void **value,
-		int *value_len);
+		size_t *value_len);
 
 	int (*object_track_start) (
 		hdb_handle_t object_handle,
@@ -341,21 +363,21 @@ struct corosync_api_v1 {
 		object_reload_notify_fn_t object_reload_notify_fn,
 		void * priv_data_pt);
 
-	int (*object_write_config) (char **error_string);
+	int (*object_write_config) (const char **error_string);
 
 	int (*object_reload_config) (int flush,
-				     char **error_string);
+				     const char **error_string);
 
 	int (*object_key_increment) (
 		hdb_handle_t object_handle,
-		void *key_name,
-		int key_len,
+		const void *key_name,
+		size_t key_len,
 		unsigned int *value);
 
 	int (*object_key_decrement) (
 		hdb_handle_t object_handle,
-		void *key_name,
-		int key_len,
+		const void *key_name,
+		size_t key_len,
 		unsigned int *value);
 
 	/*
@@ -372,7 +394,7 @@ struct corosync_api_v1 {
 		void *data,
 		void (*timer_fn) (void *data),
 		corosync_timer_handle_t *handle);
-	
+
 	void (*timer_delete) (
 		corosync_timer_handle_t timer_handle);
 
@@ -386,17 +408,19 @@ struct corosync_api_v1 {
 	 */
 	void (*ipc_source_set) (mar_message_source_t *source, void *conn);
 
-	int (*ipc_source_is_local) (mar_message_source_t *source);
+	int (*ipc_source_is_local) (const mar_message_source_t *source);
 
 	void *(*ipc_private_data_get) (void *conn);
 
-	int (*ipc_response_send) (void *conn, void *msg, int mlen);
+	int (*ipc_response_send) (void *conn, const void *msg, size_t mlen);
 
-	int (*ipc_response_iov_send) (void *conn, struct iovec *iov, int iov_len);
+	int (*ipc_response_iov_send) (void *conn,
+				      const struct iovec *iov, unsigned int iov_len);
 
-	int (*ipc_dispatch_send) (void *conn, void *msg, int mlen);
+	int (*ipc_dispatch_send) (void *conn, const void *msg, size_t mlen);
 
-	int (*ipc_dispatch_iov_send) (void *conn, struct iovec *iov, int iov_len);
+	int (*ipc_dispatch_iov_send) (void *conn,
+				      const struct iovec *iov, unsigned int iov_len);
 
 	void (*ipc_refcnt_inc) (void *conn);
 
@@ -411,7 +435,8 @@ struct corosync_api_v1 {
 
 	int (*totem_ring_reenable) (void);
 
-	int (*totem_mcast) (struct iovec *iovec, int iov_len, unsigned int guarantee);
+	int (*totem_mcast) (const struct iovec *iovec,
+			    unsigned int iov_len, unsigned int guarantee);
 
 	int (*totem_ifaces_get) (
 		unsigned int nodeid,
@@ -419,17 +444,19 @@ struct corosync_api_v1 {
 		char ***status,
 		unsigned int *iface_count);
 
-	char *(*totem_ifaces_print) (unsigned int nodeid);
+	const char *(*totem_ifaces_print) (unsigned int nodeid);
 
-	char *(*totem_ip_print) (struct totem_ip_address *addr);
+	const char *(*totem_ip_print) (const struct totem_ip_address *addr);
 
+	int (*totem_crypto_set) (unsigned int type);
 
 	int (*totem_callback_token_create) (
 		void **handle_out,
 		enum totem_callback_token_type type,
 		int delete,
-		int (*callback_fn) (enum totem_callback_token_type type, void *),
-		void *data);
+		int (*callback_fn) (enum totem_callback_token_type type,
+				    const void *),
+		const void *data);
 
 	/*
 	 * Totem open process groups API for those service engines
@@ -440,40 +467,43 @@ struct corosync_api_v1 {
 
 		void (*deliver_fn) (
 			unsigned int nodeid,
-			struct iovec *iovec,
-			int iov_len,
+			const void *msg,
+			unsigned int msg_len,
 			int endian_conversion_required),
 
 		void (*confchg_fn) (
 			enum totem_configuration_type configuration_type,
-			unsigned int *member_list, int member_list_entries,
-			unsigned int *left_list, int left_list_entries,
-			unsigned int *joined_list, int joined_list_entries,
-			struct memb_ring_id *ring_id));
+			const unsigned int *member_list,
+			size_t member_list_entries,
+			const unsigned int *left_list,
+			size_t left_list_entries,
+			const unsigned int *joined_list,
+			size_t joined_list_entries,
+			const struct memb_ring_id *ring_id));
 
 	int (*tpg_exit) (
        		hdb_handle_t handle);
 
 	int (*tpg_join) (
 		hdb_handle_t handle,
-		struct corosync_tpg_group *groups,
-		int gruop_cnt);
+		const struct corosync_tpg_group *groups,
+		size_t group_cnt);
 
 	int (*tpg_leave) (
 		hdb_handle_t handle,
-		struct corosync_tpg_group *groups,
-		int gruop_cnt);
+		const struct corosync_tpg_group *groups,
+		size_t group_cnt);
 
 	int (*tpg_joined_mcast) (
 		hdb_handle_t handle,
-		struct iovec *iovec,
-		int iov_len,
+		const struct iovec *iovec,
+		unsigned int iov_len,
 		int guarantee);
 
 	int (*tpg_joined_reserve) (
 		hdb_handle_t handle,
-		struct iovec *iovec,
-		int iov_len);
+		const struct iovec *iovec,
+		unsigned int iov_len);
 
 	int (*tpg_joined_release) (
 		int reserved_msgs);
@@ -481,23 +511,30 @@ struct corosync_api_v1 {
 	int (*tpg_groups_mcast) (
 		hdb_handle_t handle,
 		int guarantee,
-		struct corosync_tpg_group *groups,
-		int groups_cnt,
-		struct iovec *iovec,
-		int iov_len);
+		const struct corosync_tpg_group *groups,
+		size_t groups_cnt,
+		const struct iovec *iovec,
+		unsigned int iov_len);
 
 	int (*tpg_groups_reserve) (
 		hdb_handle_t handle,
-		struct corosync_tpg_group *groups,
-		int groups_cnt,
-		struct iovec *iovec,
-		int iov_len);
+		const struct corosync_tpg_group *groups,
+		size_t groups_cnt,
+		const struct iovec *iovec,
+		unsigned int iov_len);
 
 	int (*tpg_groups_release) (
 		int reserved_msgs);
 
+	int (*schedwrk_create) (
+		hdb_handle_t *handle,
+		int (schedwrk_fn) (const void *),
+		const void *context);
+
+	void (*schedwrk_destroy) (hdb_handle_t handle);
+
 	int (*sync_request) (
-		char *service_name);
+		const char *service_name);
 
 	/*
 	 * User plugin-callable functions for quorum
@@ -509,15 +546,14 @@ struct corosync_api_v1 {
 	/*
 	 * This one is for the quorum management plugin's use
 	 */
-	int (*quorum_initialize)(struct quorum_callin_functions *fns,
-				 sync_callback_fn_t *sync_callback_fn);
+	int (*quorum_initialize)(struct quorum_callin_functions *fns);
 
 	/*
 	 * Plugin loading and unloading
 	 */
 	int (*plugin_interface_reference) (
-		hdb_handle_t *handle, 
-		char *iface_name,
+		hdb_handle_t *handle,
+		const char *iface_name,
 		int version,
 		void **interface,
 		void *context);
@@ -529,12 +565,12 @@ struct corosync_api_v1 {
 	*/
 	unsigned int (*service_link_and_init) (
 		struct corosync_api_v1 *corosync_api_v1,
-		char *service_name,
+		const char *service_name,
 		unsigned int service_ver);
 
 	unsigned int (*service_unlink_and_exit) (
 		struct corosync_api_v1 *corosync_api_v1,
-		char *service_name,
+		const char *service_name,
 		unsigned int service_ver);
 
 	/*
@@ -543,6 +579,16 @@ struct corosync_api_v1 {
 	void (*error_memory_failure) (void);
 #define corosync_fatal_error(err) api->fatal_error ((err), __FILE__, __LINE__)
 	void (*fatal_error) (cs_fatal_error_t err, const char *file, unsigned int line);
+
+	void (*shutdown_request) (void);
+
+	void (*state_dump) (void);
+
+	/*
+	 * The use of this interface is highly discouraged.
+	 * Please avoid using any of coropoll apis in your service engines.
+	 */
+	hdb_handle_t (*poll_handle_get) (void);
 };
 
 #define SERVICE_ID_MAKE(a,b) ( ((a)<<16) | (b) )
@@ -550,14 +596,12 @@ struct corosync_api_v1 {
 #define SERVICE_HANDLER_MAXIMUM_COUNT 64
 
 struct corosync_lib_handler {
-	void (*lib_handler_fn) (void *conn, void *msg);
-	int response_size;
-	int response_id;
+	void (*lib_handler_fn) (void *conn, const void *msg);
 	enum cs_lib_flow_control flow_control;
 };
 
 struct corosync_exec_handler {
-	void (*exec_handler_fn) (void *msg, unsigned int nodeid);
+	void (*exec_handler_fn) (const void *msg, unsigned int nodeid);
 	void (*exec_endian_convert_fn) (void *msg);
 };
 
@@ -566,8 +610,11 @@ struct corosync_service_engine_iface_ver0 {
 };
 
 struct corosync_service_engine {
-	char *name;
+	const char *name;
 	unsigned short id;
+	unsigned short priority; /* Lower priority are loaded first, unloaded last.
+				  * 0 is a special case which always loaded _and_ unloaded last
+				  */
 	unsigned int private_data_size;
 	enum cs_lib_flow_control flow_control;
 	enum cs_lib_allow_inquorate allow_inquorate;
@@ -583,15 +630,18 @@ struct corosync_service_engine {
 	int (*config_init_fn) (struct corosync_api_v1 *);
 	void (*confchg_fn) (
 		enum totem_configuration_type configuration_type,
-		unsigned int *member_list, int member_list_entries,
-		unsigned int *left_list, int left_list_entries,
-		unsigned int *joined_list, int joined_list_entries,
-		struct memb_ring_id *ring_id);
-	void (*sync_init) (void);
+		const unsigned int *member_list, size_t member_list_entries,
+		const unsigned int *left_list, size_t left_list_entries,
+		const unsigned int *joined_list, size_t joined_list_entries,
+		const struct memb_ring_id *ring_id);
+	enum cs_sync_mode sync_mode;
+	void (*sync_init) (
+		const unsigned int *member_list,
+		size_t member_list_entries,
+		const struct memb_ring_id *ring_id);
 	int (*sync_process) (void);
 	void (*sync_activate) (void);
 	void (*sync_abort) (void);
 };
 
 #endif /* COROAPI_H_DEFINED */
-	
