@@ -7,7 +7,7 @@
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -36,12 +36,21 @@
 #ifndef HDB_H_DEFINED
 #define HDB_H_DEFINED
 
+#include <errno.h>
+#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <pthread.h>
+#include <stdint.h>
+#include <inttypes.h>
 
-typedef unsigned long long hdb_handle_t;
+typedef uint64_t hdb_handle_t;
+
+/*
+ * Formatting for string printing on 32/64 bit systems
+ */
+#define HDB_D_FORMAT "%"PRIu64
+#define HDB_X_FORMAT "%"PRIx64
 
 enum HDB_HANDLE_STATE {
 	HDB_HANDLE_STATE_EMPTY,
@@ -60,23 +69,77 @@ struct hdb_handle_database {
 	unsigned int handle_count;
 	struct hdb_handle *handles;
 	unsigned int iterator;
-	pthread_mutex_t mutex;
+        void (*destructor) (void *);
+#if defined(HAVE_PTHREAD_SPIN_LOCK)
+	pthread_spinlock_t lock;
+#else
+	pthread_mutex_t lock;
+#endif
+	unsigned int first_run;
 };
 
+#if defined(HAVE_PTHREAD_SPIN_LOCK)
+static inline void hdb_database_lock (pthread_spinlock_t *spinlock)
+{
+	pthread_spin_lock (spinlock);
+}
+
+static inline void hdb_database_unlock (pthread_spinlock_t *spinlock)
+{
+	pthread_spin_unlock (spinlock);
+}
+static inline void hdb_database_lock_init (pthread_spinlock_t *spinlock)
+{
+	pthread_spin_init (spinlock, 0);
+}
+
+static inline void hdb_database_lock_destroy (pthread_spinlock_t *spinlock)
+{
+	pthread_spin_destroy (spinlock);
+}
+
+#else
+static inline void hdb_database_lock (pthread_mutex_t *mutex)
+{
+	pthread_mutex_lock (mutex);
+}
+
+static inline void hdb_database_unlock (pthread_mutex_t *mutex)
+{
+	pthread_mutex_unlock (mutex);
+}
+static inline void hdb_database_lock_init (pthread_mutex_t *mutex)
+{
+	pthread_mutex_init (mutex, NULL);
+}
+
+static inline void hdb_database_lock_destroy (pthread_mutex_t *mutex)
+{
+	pthread_mutex_destroy (mutex);
+}
+#endif
+
+#define DECLARE_HDB_DATABASE(database_name,destructor_function)		\
+static struct hdb_handle_database (database_name) = {			\
+	.handle_count	= 0,						\
+	.handles 	= NULL,						\
+	.iterator	= 0,						\
+	.destructor	= destructor_function,				\
+	.first_run	= 1						\
+};									\
+
 static inline void hdb_create (
 	struct hdb_handle_database *handle_database)
 {
 	memset (handle_database, 0, sizeof (struct hdb_handle_database));
-	pthread_mutex_init (&handle_database->mutex, NULL);
+	hdb_database_lock_init (&handle_database->lock);
 }
 
 static inline void hdb_destroy (
 	struct hdb_handle_database *handle_database)
 {
-	if (handle_database->handles) {
-		free (handle_database->handles);
-	}
-	pthread_mutex_destroy (&handle_database->mutex);
+	free (handle_database->handles);
+	hdb_database_lock_destroy (&handle_database->lock);
 	memset (handle_database, 0, sizeof (struct hdb_handle_database));
 }
 
@@ -93,7 +156,11 @@ static inline int hdb_handle_create (
 	void *instance;
 	int i;
 
-	pthread_mutex_lock (&handle_database->mutex);
+	if (handle_database->first_run == 1) {
+		handle_database->first_run = 0;
+		hdb_database_lock_init (&handle_database->lock);
+	}
+	hdb_database_lock (&handle_database->lock);
 
 	for (handle = 0; handle < handle_database->handle_count; handle++) {
 		if (handle_database->handles[handle].state == HDB_HANDLE_STATE_EMPTY) {
@@ -107,7 +174,8 @@ static inline int hdb_handle_create (
 		new_handles = (struct hdb_handle *)realloc (handle_database->handles,
 			sizeof (struct hdb_handle) * handle_database->handle_count);
 		if (new_handles == NULL) {
-			pthread_mutex_unlock (&handle_database->mutex);
+			hdb_database_unlock (&handle_database->lock);
+			errno = ENOMEM;
 			return (-1);
 		}
 		handle_database->handles = new_handles;
@@ -115,6 +183,7 @@ static inline int hdb_handle_create (
 
 	instance = (void *)malloc (instance_size);
 	if (instance == 0) {
+		errno = ENOMEM;
 		return (-1);
 	}
 
@@ -143,7 +212,7 @@ static inline int hdb_handle_create (
 
 	*handle_id_out = (((unsigned long long)(check)) << 32) | handle;
 
-	pthread_mutex_unlock (&handle_database->mutex);
+	hdb_database_unlock (&handle_database->lock);
 
 	return (0);
 }
@@ -156,23 +225,30 @@ static inline int hdb_handle_get (
 	unsigned int check = ((unsigned int)(((unsigned long long)handle_in) >> 32));
 	unsigned int handle = handle_in & 0xffffffff;
 
-	pthread_mutex_lock (&handle_database->mutex);
-
-	if (check != 0xffffffff &&
-		check != handle_database->handles[handle].check) {
-
-		pthread_mutex_unlock (&handle_database->mutex);
-		return (-1);
+	if (handle_database->first_run == 1) {
+		handle_database->first_run = 0;
+		hdb_database_lock_init (&handle_database->lock);
 	}
+	hdb_database_lock (&handle_database->lock);
 
 	*instance = NULL;
 	if (handle >= handle_database->handle_count) {
-		pthread_mutex_unlock (&handle_database->mutex);
+		hdb_database_unlock (&handle_database->lock);
+		errno = EBADF;
 		return (-1);
 	}
 
 	if (handle_database->handles[handle].state != HDB_HANDLE_STATE_ACTIVE) {
-		pthread_mutex_unlock (&handle_database->mutex);
+		hdb_database_unlock (&handle_database->lock);
+		errno = EBADF;
+		return (-1);
+	}
+
+	if (check != 0xffffffff &&
+		check != handle_database->handles[handle].check) {
+
+		hdb_database_unlock (&handle_database->lock);
+		errno = EBADF;
 		return (-1);
 	}
 
@@ -180,7 +256,7 @@ static inline int hdb_handle_get (
 
 	handle_database->handles[handle].ref_count += 1;
 
-	pthread_mutex_unlock (&handle_database->mutex);
+	hdb_database_unlock (&handle_database->lock);
 	return (0);
 }
 
@@ -191,23 +267,38 @@ static inline int hdb_handle_put (
 	unsigned int check = ((unsigned int)(((unsigned long long)handle_in) >> 32));
 	unsigned int handle = handle_in & 0xffffffff;
 
-	pthread_mutex_lock (&handle_database->mutex);
+	if (handle_database->first_run == 1) {
+		handle_database->first_run = 0;
+		hdb_database_lock_init (&handle_database->lock);
+	}
+	hdb_database_lock (&handle_database->lock);
+
+	if (handle >= handle_database->handle_count) {
+		hdb_database_unlock (&handle_database->lock);
+
+		errno = EBADF;
+		return (-1);
+	}
 
 	if (check != 0xffffffff &&
 		check != handle_database->handles[handle].check) {
 
-		pthread_mutex_unlock (&handle_database->mutex);
+		hdb_database_unlock (&handle_database->lock);
+		errno = EBADF;
 		return (-1);
 	}
-		
+
 	handle_database->handles[handle].ref_count -= 1;
 	assert (handle_database->handles[handle].ref_count >= 0);
 
 	if (handle_database->handles[handle].ref_count == 0) {
+		if (handle_database->destructor) {
+			handle_database->destructor (handle_database->handles[handle].instance);
+		}
 		free (handle_database->handles[handle].instance);
 		memset (&handle_database->handles[handle], 0, sizeof (struct hdb_handle));
 	}
-	pthread_mutex_unlock (&handle_database->mutex);
+	hdb_database_unlock (&handle_database->lock);
 	return (0);
 }
 
@@ -219,20 +310,67 @@ static inline int hdb_handle_destroy (
 	unsigned int handle = handle_in & 0xffffffff;
 	int res;
 
-	pthread_mutex_lock (&handle_database->mutex);
+	if (handle_database->first_run == 1) {
+		handle_database->first_run = 0;
+		hdb_database_lock_init (&handle_database->lock);
+	}
+	hdb_database_lock (&handle_database->lock);
+
+	if (handle >= handle_database->handle_count) {
+		hdb_database_unlock (&handle_database->lock);
+
+		errno = EBADF;
+		return (-1);
+	}
 
 	if (check != 0xffffffff &&
 		check != handle_database->handles[handle].check) {
-		pthread_mutex_unlock (&handle_database->mutex);
+		hdb_database_unlock (&handle_database->lock);
+		errno = EBADF;
 		return (-1);
 	}
 
 	handle_database->handles[handle].state = HDB_HANDLE_STATE_PENDINGREMOVAL;
-	pthread_mutex_unlock (&handle_database->mutex);
-	res = hdb_handle_put (handle_database, handle);
+	hdb_database_unlock (&handle_database->lock);
+	res = hdb_handle_put (handle_database, handle_in);
 	return (res);
 }
 
+static inline int hdb_handle_refcount_get (
+	struct hdb_handle_database *handle_database,
+	hdb_handle_t handle_in)
+{
+	unsigned int check = ((unsigned int)(((unsigned long long)handle_in) >> 32));
+	unsigned int handle = handle_in & 0xffffffff;
+
+	int refcount = 0;
+
+	if (handle_database->first_run == 1) {
+		handle_database->first_run = 0;
+		hdb_database_lock_init (&handle_database->lock);
+	}
+	hdb_database_lock (&handle_database->lock);
+
+	if (handle >= handle_database->handle_count) {
+		hdb_database_unlock (&handle_database->lock);
+		errno = EBADF;
+		return (-1);
+	}
+
+	if (check != 0xffffffff &&
+		check != handle_database->handles[handle].check) {
+		hdb_database_unlock (&handle_database->lock);
+		errno = EBADF;
+		return (-1);
+	}
+
+	refcount = handle_database->handles[handle].ref_count;
+
+	hdb_database_unlock (&handle_database->lock);
+
+	return (refcount);
+}
+
 static inline void hdb_iterator_reset (
 	struct hdb_handle_database *handle_database)
 {
