@@ -4,6 +4,7 @@
  * All rights reserved.
  *
  * Author: Christine Caulfield (ccaulfie@redhat.com)
+ * Author: Jan Friesse (jfriesse@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -36,18 +37,14 @@
 
 #include <netinet/in.h>
 #include <corosync/corotypes.h>
-#include "ipc_gen.h"
-#include "mar_cpg.h"
+#include <corosync/mar_gen.h>
 
 enum req_cpg_types {
 	MESSAGE_REQ_CPG_JOIN = 0,
 	MESSAGE_REQ_CPG_LEAVE = 1,
 	MESSAGE_REQ_CPG_MCAST = 2,
 	MESSAGE_REQ_CPG_MEMBERSHIP = 3,
-	MESSAGE_REQ_CPG_TRACKSTART = 4,
-	MESSAGE_REQ_CPG_TRACKSTOP = 5,
-	MESSAGE_REQ_CPG_LOCAL_GET = 6,
-	MESSAGE_REQ_CPG_GROUPS_GET = 7
+	MESSAGE_REQ_CPG_LOCAL_GET = 4,
 };
 
 enum res_cpg_types {
@@ -57,13 +54,9 @@ enum res_cpg_types {
 	MESSAGE_RES_CPG_MEMBERSHIP = 3,
 	MESSAGE_RES_CPG_CONFCHG_CALLBACK = 4,
 	MESSAGE_RES_CPG_DELIVER_CALLBACK = 5,
-	MESSAGE_RES_CPG_TRACKSTART = 6,
-	MESSAGE_RES_CPG_TRACKSTOP = 7,
-	MESSAGE_RES_CPG_FLOW_CONTROL_STATE_SET = 8,
-	MESSAGE_RES_CPG_LOCAL_GET = 9,
-	MESSAGE_RES_CPG_GROUPS_GET = 10,
-	MESSAGE_RES_CPG_GROUPS_CALLBACK = 11,
-	MESSAGE_RES_CPG_FLOWCONTROL_CALLBACK = 12
+	MESSAGE_RES_CPG_FLOW_CONTROL_STATE_SET = 6,
+	MESSAGE_RES_CPG_LOCAL_GET = 7,
+	MESSAGE_RES_CPG_FLOWCONTROL_CALLBACK = 8
 };
 
 enum lib_cpg_confchg_reason {
@@ -74,59 +67,118 @@ enum lib_cpg_confchg_reason {
 	CONFCHG_CPG_REASON_PROCDOWN = 5
 };
 
+typedef struct {
+	uint32_t length __attribute__((aligned(8)));
+	char value[CPG_MAX_NAME_LENGTH] __attribute__((aligned(8)));
+} mar_cpg_name_t;
+
+static inline void swab_mar_cpg_name_t (mar_cpg_name_t *to_swab)
+{
+	swab_mar_uint32_t (&to_swab->length);
+}
+
+static inline void marshall_from_mar_cpg_name_t (
+	struct cpg_name *dest,
+	const mar_cpg_name_t *src)
+{
+	dest->length = src->length;
+	memcpy (&dest->value, &src->value, CPG_MAX_NAME_LENGTH);
+}
+
+static inline void marshall_to_mar_cpg_name_t (
+	mar_cpg_name_t *dest,
+	const struct cpg_name *src)
+{
+	dest->length = src->length;
+	memcpy (&dest->value, &src->value, CPG_MAX_NAME_LENGTH);
+}
+
+typedef struct {
+        mar_uint32_t nodeid __attribute__((aligned(8)));
+        mar_uint32_t pid __attribute__((aligned(8)));
+        mar_uint32_t reason __attribute__((aligned(8)));
+} mar_cpg_address_t;
+
+static inline void marshall_from_mar_cpg_address_t (
+	struct cpg_address *dest,
+	const mar_cpg_address_t *src)
+{
+	dest->nodeid = src->nodeid;
+	dest->pid = src->pid;
+	dest->reason = src->reason;
+}
+
+static inline void marshall_to_mar_cpg_address_t (
+	mar_cpg_address_t *dest,
+	const struct cpg_address *src)
+{
+	dest->nodeid = src->nodeid;
+	dest->pid = src->pid;
+	dest->reason = src->reason;
+}
+
+static inline int mar_name_compare (
+		const mar_cpg_name_t *g1,
+		const mar_cpg_name_t *g2)
+{
+	return (g1->length == g2->length?
+		memcmp (g1->value, g2->value, g1->length):
+		g1->length - g2->length);
+}
+
 struct req_lib_cpg_join {
-	mar_req_header_t header __attribute__((aligned(8)));
+	coroipc_request_header_t header __attribute__((aligned(8)));
 	mar_cpg_name_t group_name __attribute__((aligned(8)));
 	mar_uint32_t pid __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_join {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 };
 
 struct req_lib_cpg_trackstart {
-	mar_req_header_t header __attribute__((aligned(8)));
+	coroipc_request_header_t header __attribute__((aligned(8)));
 	mar_cpg_name_t group_name __attribute__((aligned(8)));
 	mar_uint32_t pid __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_trackstart {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 };
 
 struct req_lib_cpg_trackstop {
-	mar_req_header_t header __attribute__((aligned(8)));
+	coroipc_request_header_t header __attribute__((aligned(8)));
 	mar_cpg_name_t group_name __attribute__((aligned(8)));
 	mar_uint32_t pid __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_trackstop {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 };
 
 struct req_lib_cpg_local_get {
-	mar_req_header_t header __attribute__((aligned(8)));
+	coroipc_request_header_t header __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_local_get {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 	mar_uint32_t local_nodeid __attribute__((aligned(8)));
 };
 
 struct req_lib_cpg_mcast {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 	mar_uint32_t guarantee __attribute__((aligned(8)));
 	mar_uint32_t msglen __attribute__((aligned(8)));
 	mar_uint8_t message[] __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_mcast {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 };
 
 /* Message from another node */
 struct res_lib_cpg_deliver_callback {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 	mar_cpg_name_t group_name __attribute__((aligned(8)));
 	mar_uint32_t msglen __attribute__((aligned(8)));
 	mar_uint32_t nodeid __attribute__((aligned(8)));
@@ -135,17 +187,17 @@ struct res_lib_cpg_deliver_callback {
 };
 
 struct res_lib_cpg_flowcontrol_callback {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 	mar_uint32_t flow_control_state __attribute__((aligned(8)));
 };
 
 struct req_lib_cpg_membership {
-	mar_req_header_t header __attribute__((aligned(8)));
+	coroipc_request_header_t header __attribute__((aligned(8)));
 //	mar_cpg_name_t group_name __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_confchg_callback {
-	mar_res_header_t header __attribute__((aligned(8)));
+	coroipc_response_header_t header __attribute__((aligned(8)));
 	mar_cpg_name_t group_name __attribute__((aligned(8)));
 	mar_uint32_t member_list_entries __attribute__((aligned(8)));
 	mar_uint32_t joined_list_entries __attribute__((aligned(8)));
@@ -156,31 +208,13 @@ struct res_lib_cpg_confchg_callback {
 };
 
 struct req_lib_cpg_leave {
-	mar_req_header_t header __attribute__((aligned(8)));
+	coroipc_request_header_t header __attribute__((aligned(8)));
 	mar_cpg_name_t group_name __attribute__((aligned(8)));
 	mar_uint32_t pid __attribute__((aligned(8)));
 };
 
 struct res_lib_cpg_leave {
-	mar_res_header_t header __attribute__((aligned(8)));
-};
-
-struct req_lib_cpg_groups_get {
-	mar_req_header_t header __attribute__((aligned(8)));
-};
-
-struct res_lib_cpg_groups_get {
-	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t num_groups __attribute__((aligned(8)));
-};
-
-struct res_lib_cpg_groups_get_callback {
-	mar_res_header_t header __attribute__((aligned(8)));
-	mar_uint32_t group_num  __attribute__((aligned(8)));
-	mar_uint32_t total_groups  __attribute__((aligned(8)));
-	mar_cpg_name_t group_name __attribute__((aligned(8)));
-	mar_uint32_t num_members __attribute__((aligned(8)));
-	mar_cpg_address_t member_list[];
+	coroipc_response_header_t header __attribute__((aligned(8)));
 };
 
 #endif /* IPC_CPG_H_DEFINED */
