@@ -1,13 +1,13 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * All rights reserved.
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -58,14 +58,13 @@ struct totem_interface {
 };
 
 struct totem_logging_configuration {
-        void (*log_printf) (
-                int subsys,
-                const char *function_name,
-                const char *file_name,
-                int file_line,
-                unsigned int level,
-                const char *format,
-                ...) __attribute__((format(printf, 6, 7)));
+	void (*log_printf) (
+		unsigned int rec_ident,
+		const char *function_name,
+		const char *file_name,
+		int file_line,
+		const char *format,
+		...) __attribute__((format(printf, 5, 6)));
 
 	int log_level_security;
 	int log_level_error;
@@ -75,6 +74,9 @@ struct totem_logging_configuration {
 	int log_subsys_id;
 };
 
+enum { TOTEM_PRIVATE_KEY_LEN = 128 };
+enum { TOTEM_RRP_MODE_BYTES = 64 };
+
 struct totem_config {
 	int version;
 
@@ -82,15 +84,16 @@ struct totem_config {
 	 * network
 	 */
 	struct totem_interface *interfaces;
-	int interface_count;
+	unsigned int interface_count;
 	unsigned int node_id;
+	unsigned int clear_node_high_bit;
 
 	/*
 	 * key information
 	 */
-	unsigned char private_key[128];
+	unsigned char private_key[TOTEM_PRIVATE_KEY_LEN];
 
-	int private_key_len;
+	unsigned int private_key_len;
 
 	/*
 	 * Totem configuration parameters
@@ -123,31 +126,47 @@ struct totem_config {
 
 	unsigned int rrp_problem_count_threshold;
 
-	char rrp_mode[64];
+	char rrp_mode[TOTEM_RRP_MODE_BYTES];
 
 	struct totem_logging_configuration totem_logging_configuration;
 
+	void (*log_rec) (
+		int subsysid,
+		const char *function_name,
+		const char *file_name,
+		int file_line,
+		unsigned int rec_ident,
+		...);
+
 	unsigned int secauth;
 
 	unsigned int net_mtu;
 
 	unsigned int threads;
-	
+
 	unsigned int heartbeat_failures_allowed;
-	
+
 	unsigned int max_network_delay;
 
 	unsigned int window_size;
 
 	unsigned int max_messages;
 
-	char *vsf_type;
+	const char *vsf_type;
+
+	unsigned int broadcast_use;
+
+	enum { TOTEM_CRYPTO_SOBER=0, TOTEM_CRYPTO_NSS } crypto_type;
+	enum { TOTEM_CRYPTO_ACCEPT_OLD=0, TOTEM_CRYPTO_ACCEPT_NEW } crypto_accept;
+
+	int crypto_crypt_type;
+	int crypto_sign_type;
 };
 
 #define TOTEM_CONFIGURATION_TYPE
 enum totem_configuration_type {
 	TOTEM_CONFIGURATION_REGULAR,
-	TOTEM_CONFIGURATION_TRANSITIONAL	
+	TOTEM_CONFIGURATION_TRANSITIONAL
 };
 
 #define TOTEM_CALLBACK_TOKEN_TYPE
