@@ -40,7 +40,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
-#include <signal.h>
 #include <pthread.h>
 #include <limits.h>
 #include <sys/types.h>
@@ -49,53 +48,35 @@
 #include <sys/un.h>
 
 #include <corosync/corotypes.h>
-#include <corosync/cfg.h>
-#include <corosync/totem/totemip.h>
-#include <corosync/mar_gen.h>
-#include <corosync/ipc_gen.h>
-#include <corosync/ipc_cfg.h>
+#include <corosync/coroipc_types.h>
 #include <corosync/coroipcc.h>
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
 
-struct cfg_res_overlay {
-	mar_res_header_t header;
-	char data[4096];
-};
+#include <corosync/cfg.h>
+#include <corosync/ipc_cfg.h>
+
+#include "util.h"
 
 /*
  * Data structure for instance data
  */
 struct cfg_instance {
-	void *ipc_ctx;
+	hdb_handle_t handle;
 	corosync_cfg_callbacks_t callbacks;
 	cs_name_t comp_name;
 	int comp_registered;
 	int finalize;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 };
 
-static void cfg_handle_instance_destructor (void *);
-
 /*
  * All instances in one database
  */
-static struct saHandleDatabase cfg_hdb = {
-	.handleCount			= 0,
-	.handles			= 0,
-	.mutex				= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= cfg_handle_instance_destructor
-};
+DECLARE_HDB_DATABASE (cfg_hdb,NULL);
 
 /*
  * Implementation
  */
-void cfg_handle_instance_destructor (void *instance)
-{
-	struct cfg_instance *cfg_instance = instance;
-
-	pthread_mutex_destroy (&cfg_instance->response_mutex);
-	pthread_mutex_destroy (&cfg_instance->dispatch_mutex);
-}
 
 cs_error_t
 corosync_cfg_initialize (
@@ -105,17 +86,23 @@ corosync_cfg_initialize (
 	struct cfg_instance *cfg_instance;
 	cs_error_t error = CS_OK;
 
-	error = saHandleCreate (&cfg_hdb, sizeof (struct cfg_instance), cfg_handle);
+	error = hdb_error_to_cs (hdb_handle_create (&cfg_hdb, sizeof (struct cfg_instance), cfg_handle));
 	if (error != CS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&cfg_hdb, *cfg_handle, (void *)&cfg_instance);
+	error = hdb_error_to_cs (hdb_handle_get (&cfg_hdb, *cfg_handle, (void *)&cfg_instance));
 	if (error != CS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, CFG_SERVICE, &cfg_instance->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		CFG_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		&cfg_instance->handle);
 	if (error != CS_OK) {
 		goto error_put_destroy;
 	}
@@ -124,18 +111,14 @@ corosync_cfg_initialize (
 	memcpy (&cfg_instance->callbacks, cfg_callbacks, sizeof (corosync_cfg_callbacks_t));
 	}
 
-	pthread_mutex_init (&cfg_instance->response_mutex, NULL);
-
-	pthread_mutex_init (&cfg_instance->dispatch_mutex, NULL);
-
-	(void)saHandleInstancePut (&cfg_hdb, *cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, *cfg_handle);
 
 	return (CS_OK);
 
 error_put_destroy:
-	(void)saHandleInstancePut (&cfg_hdb, *cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, *cfg_handle);
 error_destroy:
-	(void)saHandleDestroy (&cfg_hdb, *cfg_handle);
+	(void)hdb_handle_destroy (&cfg_hdb, *cfg_handle);
 error_no_destroy:
 	return (error);
 }
@@ -148,15 +131,15 @@ corosync_cfg_fd_get (
 	struct cfg_instance *cfg_instance;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle, (void *)&cfg_instance);
+	error = hdb_error_to_cs (hdb_handle_get (&cfg_hdb, cfg_handle, (void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	*selection_fd = coroipcc_fd_get (cfg_instance->ipc_ctx);
+	error = coroipcc_fd_get (cfg_instance->handle, selection_fd);
 
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
-	return (CS_OK);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
+	return (error);
 }
 
 cs_error_t
@@ -167,21 +150,13 @@ corosync_cfg_dispatch (
 	int timeout = -1;
 	cs_error_t error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct cfg_instance *cfg_instance;
 	struct res_lib_cfg_testshutdown *res_lib_cfg_testshutdown;
-#ifdef COMPILE_OUT
-	struct res_lib_corosync_healthcheckcallback *res_lib_corosync_healthcheckcallback;
-	struct res_lib_corosync_readinessstatesetcallback *res_lib_corosync_readinessstatesetcallback;
-	struct res_lib_corosync_csisetcallback *res_lib_corosync_csisetcallback;
-	struct res_lib_corosync_csiremovecallback *res_lib_corosync_csiremovecallback;
-	struct res_lib_cfg_statetrackcallback *res_lib_cfg_statetrackcallback;
-#endif
 	corosync_cfg_callbacks_t callbacks;
-	struct cfg_res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
+	error = hdb_error_to_cs (hdb_handle_get (&cfg_hdb, cfg_handle,
+		(void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -194,25 +169,25 @@ corosync_cfg_dispatch (
 	}
 
 	do {
-		dispatch_avail = coroipcc_dispatch_recv (cfg_instance->ipc_ctx,
-			(void *)&dispatch_data, timeout);
 
-		/*
-		 * Handle has been finalized in another thread
-		 */
-		if (cfg_instance->finalize == 1) {
+		error = coroipcc_dispatch_get (
+			cfg_instance->handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
 			error = CS_OK;
-			pthread_mutex_unlock (&cfg_instance->dispatch_mutex);
-			goto error_unlock;
+			goto error_put;
+		}
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
-		if (dispatch_avail == 0 && dispatch_flags == CS_DISPATCH_ALL) {
-			pthread_mutex_unlock (&cfg_instance->dispatch_mutex);
-			break; /* exit do while cont is 1 loop */
-		} else
-		if (dispatch_avail == 0) {
-			pthread_mutex_unlock (&cfg_instance->dispatch_mutex);
-			continue; /* next poll */
+		if (dispatch_data == NULL) {
+			if (dispatch_flags == CPG_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
+			} else {
+				continue; /* next poll */
+			}
 		}
 
 		/*
@@ -221,23 +196,24 @@ corosync_cfg_dispatch (
 		 * operate at the same time that cfgFinalize has been called in another thread.
 		 */
 		memcpy (&callbacks, &cfg_instance->callbacks, sizeof (corosync_cfg_callbacks_t));
-		pthread_mutex_unlock (&cfg_instance->dispatch_mutex);
 
 		/*
 		 * Dispatch incoming response
 		 */
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 		case MESSAGE_RES_CFG_TESTSHUTDOWN:
 			if (callbacks.corosync_cfg_shutdown_callback) {
-				res_lib_cfg_testshutdown = (struct res_lib_cfg_testshutdown *)&dispatch_data;
+				res_lib_cfg_testshutdown = (struct res_lib_cfg_testshutdown *)dispatch_data;
 				callbacks.corosync_cfg_shutdown_callback(cfg_handle, res_lib_cfg_testshutdown->flags);
 			}
 			break;
 		default:
+			coroipcc_dispatch_put (cfg_instance->handle);
 			error = CS_ERR_LIBRARY;
 			goto error_nounlock;
 			break;
 		}
+		coroipcc_dispatch_put (cfg_instance->handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -253,8 +229,8 @@ corosync_cfg_dispatch (
 		}
 	} while (cont);
 
-error_unlock:
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+error_put:
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 error_nounlock:
 	return (error);
 }
@@ -266,40 +242,26 @@ corosync_cfg_finalize (
 	struct cfg_instance *cfg_instance;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle, (void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle, (void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&cfg_instance->dispatch_mutex);
-
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (cfg_instance->finalize) {
-		pthread_mutex_unlock (&cfg_instance->response_mutex);
-		pthread_mutex_unlock (&cfg_instance->dispatch_mutex);
-		(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+		(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 		return (CS_ERR_BAD_HANDLE);
 	}
 
 	cfg_instance->finalize = 1;
 
-	coroipcc_service_disconnect (cfg_instance->ipc_ctx);
+	coroipcc_service_disconnect (cfg_instance->handle);
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
+	(void)hdb_handle_destroy (&cfg_hdb, cfg_handle);
 
-	pthread_mutex_unlock (&cfg_instance->dispatch_mutex);
-
-	pthread_mutex_destroy (&cfg_instance->response_mutex);
-
-	pthread_mutex_destroy (&cfg_instance->dispatch_mutex);
-
-	(void)saHandleDestroy (&cfg_hdb, cfg_handle);
-
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
 	return (error);
 }
@@ -318,7 +280,7 @@ corosync_cfg_ring_status_get (
 	cs_error_t error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle, (void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle, (void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -326,19 +288,15 @@ corosync_cfg_ring_status_get (
 	req_lib_cfg_ringstatusget.header.size = sizeof (struct req_lib_cfg_ringstatusget);
 	req_lib_cfg_ringstatusget.header.id = MESSAGE_REQ_CFG_RINGSTATUSGET;
 
-	iov.iov_base = 	&req_lib_cfg_ringstatusget,
+	iov.iov_base = (void *)&req_lib_cfg_ringstatusget,
 	iov.iov_len = sizeof (struct req_lib_cfg_ringstatusget),
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive(cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive(cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_ringstatusget,
 		sizeof (struct res_lib_cfg_ringstatusget));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
 	*interface_count = res_lib_cfg_ringstatusget.interface_count;
 	*interface_names = malloc (sizeof (char *) * *interface_count);
 	if (*interface_names == NULL) {
@@ -369,12 +327,8 @@ corosync_cfg_ring_status_get (
 
 error_free_contents:
 	for (i = 0; i < res_lib_cfg_ringstatusget.interface_count; i++) {
-		if ((*(interface_names))[i]) {
-			free ((*(interface_names))[i]);
-		}
-		if ((*(status))[i]) {
-			free ((*(status))[i]);
-		}
+		free (*interface_names + i);
+		free (*status + i);
 	}
 
 	free (*status);
@@ -383,7 +337,7 @@ error_free_interface_names:
 	free (*interface_names);
 
 no_error:
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
 	return (error);
 }
@@ -398,7 +352,7 @@ corosync_cfg_ring_reenable (
 	cs_error_t error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle, (void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle, (void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -406,19 +360,16 @@ corosync_cfg_ring_reenable (
 	req_lib_cfg_ringreenable.header.size = sizeof (struct req_lib_cfg_ringreenable);
 	req_lib_cfg_ringreenable.header.id = MESSAGE_REQ_CFG_RINGREENABLE;
 
-	iov.iov_base = &req_lib_cfg_ringreenable,
+	iov.iov_base = (void *)&req_lib_cfg_ringreenable,
 	iov.iov_len = sizeof (struct req_lib_cfg_ringreenable);
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_ringreenable,
 		sizeof (struct res_lib_cfg_ringreenable));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
 	return (error);
 }
@@ -426,7 +377,7 @@ corosync_cfg_ring_reenable (
 cs_error_t
 corosync_cfg_service_load (
 	corosync_cfg_handle_t cfg_handle,
-	char *service_name,
+	const char *service_name,
 	unsigned int service_ver)
 {
 	struct cfg_instance *cfg_instance;
@@ -435,7 +386,7 @@ corosync_cfg_service_load (
 	cs_error_t error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle, (void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle, (void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -448,19 +399,16 @@ corosync_cfg_service_load (
 		sizeof (req_lib_cfg_serviceload.service_name) - 1);
 	req_lib_cfg_serviceload.service_ver = service_ver;
 
-	iov.iov_base = &req_lib_cfg_serviceload;
+	iov.iov_base = (void *)&req_lib_cfg_serviceload;
 	iov.iov_len = sizeof (req_lib_cfg_serviceload);
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_serviceload,
 		sizeof (struct res_lib_cfg_serviceload));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
 	return (error);
 }
@@ -468,7 +416,7 @@ corosync_cfg_service_load (
 cs_error_t
 corosync_cfg_service_unload (
 	corosync_cfg_handle_t cfg_handle,
-	char *service_name,
+	const char *service_name,
 	unsigned int service_ver)
 {
 	struct cfg_instance *cfg_instance;
@@ -477,7 +425,7 @@ corosync_cfg_service_unload (
 	cs_error_t error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle, (void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle, (void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -490,19 +438,16 @@ corosync_cfg_service_unload (
 		sizeof (req_lib_cfg_serviceunload.service_name) - 1);
 	req_lib_cfg_serviceunload.service_ver = service_ver;
 
-	iov.iov_base = &req_lib_cfg_serviceunload;
+	iov.iov_base = (void *)&req_lib_cfg_serviceunload;
 	iov.iov_len = sizeof (req_lib_cfg_serviceunload);
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_serviceunload,
 		sizeof (struct res_lib_cfg_serviceunload));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
 	return (error);
 }
@@ -523,26 +468,22 @@ corosync_cfg_state_track (
 	req_lib_cfg_statetrack.track_flags = track_flags;
 	req_lib_cfg_statetrack.notification_buffer_address = (corosync_cfg_state_notification_t *)notification_buffer;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle,
+		(void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	iov.iov_base = &req_lib_cfg_statetrack,
+	iov.iov_base = (void *)&req_lib_cfg_statetrack,
 	iov.iov_len = sizeof (struct req_lib_cfg_statetrack),
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_statetrack,
 		sizeof (struct res_lib_cfg_statetrack));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
         return (error == CS_OK ? res_lib_cfg_statetrack.header.error : error);
 }
@@ -557,8 +498,8 @@ corosync_cfg_state_track_stop (
 	cs_error_t error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
+	error = hdb_error_to_cs (hdb_handle_get (&cfg_hdb, cfg_handle,
+		(void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -566,117 +507,25 @@ corosync_cfg_state_track_stop (
 	req_lib_cfg_statetrackstop.header.size = sizeof (struct req_lib_cfg_statetrackstop);
 	req_lib_cfg_statetrackstop.header.id = MESSAGE_REQ_CFG_STATETRACKSTOP;
 
-	iov.iov_base = &req_lib_cfg_statetrackstop,
+	iov.iov_base = (void *)&req_lib_cfg_statetrackstop,
 	iov.iov_len = sizeof (struct req_lib_cfg_statetrackstop),
-	pthread_mutex_lock (&cfg_instance->response_mutex);
 
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_statetrackstop,
 		sizeof (struct res_lib_cfg_statetrackstop));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
         return (error == CS_OK ? res_lib_cfg_statetrackstop.header.error : error);
 }
 
-cs_error_t
-corosync_cfg_admin_state_get (
-	corosync_cfg_handle_t cfg_handle,
-	corosync_cfg_administrative_target_t administrative_target,
-	corosync_cfg_administrative_state_t *administrative_state)
-{
-	struct cfg_instance *cfg_instance;
-	struct req_lib_cfg_administrativestateget req_lib_cfg_administrativestateget;
-	struct res_lib_cfg_administrativestateget res_lib_cfg_administrativestateget;
-	cs_error_t error;
-	struct iovec iov;
-
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
-	if (error != CS_OK) {
-		return (error);
-	}
-
-	req_lib_cfg_administrativestateget.header.id = MESSAGE_REQ_CFG_ADMINISTRATIVESTATEGET;
-	req_lib_cfg_administrativestateget.header.size = sizeof (struct req_lib_cfg_administrativestateget);
-	req_lib_cfg_administrativestateget.administrative_target = administrative_target;
-
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	iov.iov_base = &req_lib_cfg_administrativestateget,
-	iov.iov_len = sizeof (struct req_lib_cfg_administrativestateget),
-
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
-		&iov,
-		1,
-		&res_lib_cfg_administrativestateget,
-		sizeof (struct res_lib_cfg_administrativestateget));
-
-	error = res_lib_cfg_administrativestateget.header.error;
-
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
-
-        return (error == CS_OK ? res_lib_cfg_administrativestateget.header.error : error);
-}
-
-cs_error_t
-corosync_cfg_admin_state_set (
-	corosync_cfg_handle_t cfg_handle,
-	corosync_cfg_administrative_target_t administrative_target,
-	corosync_cfg_administrative_state_t administrative_state)
-{
-	struct cfg_instance *cfg_instance;
-	struct req_lib_cfg_administrativestateset req_lib_cfg_administrativestateset;
-	struct res_lib_cfg_administrativestateset res_lib_cfg_administrativestateset;
-	cs_error_t error;
-	struct iovec iov;
-
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
-	if (error != CS_OK) {
-		return (error);
-	}
-
-	req_lib_cfg_administrativestateset.header.id = MESSAGE_REQ_CFG_ADMINISTRATIVESTATEGET;
-	req_lib_cfg_administrativestateset.header.size = sizeof (struct req_lib_cfg_administrativestateset);
-	req_lib_cfg_administrativestateset.administrative_target = administrative_target;
-	req_lib_cfg_administrativestateset.administrative_state = administrative_state;
-
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	iov.iov_base = &req_lib_cfg_administrativestateset,
-	iov.iov_len = sizeof (struct req_lib_cfg_administrativestateset),
-
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
-		&iov,
-		1,
-		&res_lib_cfg_administrativestateset,
-		sizeof (struct res_lib_cfg_administrativestateset));
-
-	error = res_lib_cfg_administrativestateset.header.error;
-
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
-
-        return (error == CS_OK ? res_lib_cfg_administrativestateset.header.error : error);
-}
-
 cs_error_t
 corosync_cfg_kill_node (
 	corosync_cfg_handle_t cfg_handle,
 	unsigned int nodeid,
-	char *reason)
+	const char *reason)
 {
 	struct cfg_instance *cfg_instance;
 	struct req_lib_cfg_killnode req_lib_cfg_killnode;
@@ -687,8 +536,8 @@ corosync_cfg_kill_node (
 	if (strlen(reason) >= CS_MAX_NAME_LENGTH)
 		return CS_ERR_NAME_TOO_LONG;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
+	error = hdb_error_to_cs (hdb_handle_get (&cfg_hdb, cfg_handle,
+		(void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -699,12 +548,10 @@ corosync_cfg_kill_node (
 	strcpy((char *)req_lib_cfg_killnode.reason.value, reason);
 	req_lib_cfg_killnode.reason.length = strlen(reason)+1;
 
-	iov.iov_base = &req_lib_cfg_killnode;
+	iov.iov_base = (void *)&req_lib_cfg_killnode;
 	iov.iov_len = sizeof (struct req_lib_cfg_killnode);
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_killnode,
@@ -712,9 +559,7 @@ corosync_cfg_kill_node (
 
 	error = res_lib_cfg_killnode.header.error;
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
         return (error == CS_OK ? res_lib_cfg_killnode.header.error : error);
 }
@@ -730,8 +575,8 @@ corosync_cfg_try_shutdown (
 	cs_error_t error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle,
+		(void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -740,20 +585,16 @@ corosync_cfg_try_shutdown (
 	req_lib_cfg_tryshutdown.header.size = sizeof (struct req_lib_cfg_tryshutdown);
 	req_lib_cfg_tryshutdown.flags = flags;
 
-	iov.iov_base = &req_lib_cfg_tryshutdown;
+	iov.iov_base = (void *)&req_lib_cfg_tryshutdown;
 	iov.iov_len = sizeof (req_lib_cfg_tryshutdown);
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_tryshutdown,
 		sizeof (struct res_lib_cfg_tryshutdown));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
-	(void)saHandleInstancePut (&cfg_hdb, cfg_handle);
+	(void)hdb_handle_put (&cfg_hdb, cfg_handle);
 
         return (error == CS_OK ? res_lib_cfg_tryshutdown.header.error : error);
 }
@@ -769,8 +610,8 @@ corosync_cfg_replyto_shutdown (
 	struct iovec iov;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle,
+		(void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -779,26 +620,22 @@ corosync_cfg_replyto_shutdown (
 	req_lib_cfg_replytoshutdown.header.size = sizeof (struct req_lib_cfg_replytoshutdown);
 	req_lib_cfg_replytoshutdown.response = response;
 
-	iov.iov_base = &req_lib_cfg_replytoshutdown;
+	iov.iov_base = (void *)&req_lib_cfg_replytoshutdown;
 	iov.iov_len = sizeof (struct req_lib_cfg_replytoshutdown);
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
 		&iov,
 		1,
 		&res_lib_cfg_replytoshutdown,
 		sizeof (struct res_lib_cfg_replytoshutdown));
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
 	return (error);
 }
 
 cs_error_t corosync_cfg_get_node_addrs (
 	corosync_cfg_handle_t cfg_handle,
 	int nodeid,
-	int max_addrs,
+	size_t max_addrs,
 	int *num_addrs,
 	corosync_cfg_node_address_t *addrs)
 {
@@ -806,13 +643,13 @@ cs_error_t corosync_cfg_get_node_addrs (
 	struct req_lib_cfg_get_node_addrs req_lib_cfg_get_node_addrs;
 	struct res_lib_cfg_get_node_addrs *res_lib_cfg_get_node_addrs;
 	struct cfg_instance *cfg_instance;
-	int addrlen;
+	int addrlen = 0;
 	int i;
 	struct iovec iov;
 	void *return_address;
 
-	error = saHandleInstanceGet (&cfg_hdb, cfg_handle,
-		(void *)&cfg_instance);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, cfg_handle,
+		(void *)&cfg_instance));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -824,18 +661,15 @@ cs_error_t corosync_cfg_get_node_addrs (
 	iov.iov_base = (char *)&req_lib_cfg_get_node_addrs;
 	iov.iov_len = sizeof (req_lib_cfg_get_node_addrs);
 
-	pthread_mutex_lock (&cfg_instance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive_in_buf (cfg_instance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive_in_buf_get (
+		cfg_instance->handle,
 		&iov,
 		1,
 		&return_address);
 	res_lib_cfg_get_node_addrs = return_address;
 
-	pthread_mutex_unlock (&cfg_instance->response_mutex);
-
 	if (error != CS_OK) {
-		goto error_exit;
+		goto error_put;
 	}
 
 	if (res_lib_cfg_get_node_addrs->family == AF_INET)
@@ -844,10 +678,11 @@ cs_error_t corosync_cfg_get_node_addrs (
 		addrlen = sizeof(struct sockaddr_in6);
 
 	for (i=0; i<max_addrs && i<res_lib_cfg_get_node_addrs->num_addrs; i++) {
-		addrs[i].address_length = addrlen;
 		struct sockaddr_in *in;
 		struct sockaddr_in6 *in6;
 
+		addrs[i].address_length = addrlen;
+
 		if (res_lib_cfg_get_node_addrs->family == AF_INET) {
 			in = (struct sockaddr_in *)addrs[i].address;
 			in->sin_family = AF_INET;
@@ -862,7 +697,9 @@ cs_error_t corosync_cfg_get_node_addrs (
 	*num_addrs = res_lib_cfg_get_node_addrs->num_addrs;
 	errno = error = res_lib_cfg_get_node_addrs->header.error;
 
-error_exit:
+error_put:
+	error = coroipcc_msg_send_reply_receive_in_buf_put (cfg_instance->handle);
+	hdb_handle_put (&cfg_hdb, cfg_handle);
 
 	return (error);
 }
@@ -877,28 +714,24 @@ cs_error_t corosync_cfg_local_get (
 	struct req_lib_cfg_local_get req_lib_cfg_local_get;
 	struct res_lib_cfg_local_get res_lib_cfg_local_get;
 
-	error = saHandleInstanceGet (&cfg_hdb, handle, (void *)&cfg_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, handle, (void *)&cfg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	req_lib_cfg_local_get.header.size = sizeof (mar_req_header_t);
+	req_lib_cfg_local_get.header.size = sizeof (coroipc_request_header_t);
 	req_lib_cfg_local_get.header.id = MESSAGE_REQ_CFG_LOCAL_GET;
 
-	iov.iov_base = &req_lib_cfg_local_get;
+	iov.iov_base = (void *)&req_lib_cfg_local_get;
 	iov.iov_len = sizeof (struct req_lib_cfg_local_get);
 
-	pthread_mutex_lock (&cfg_inst->response_mutex);
-
 	error = coroipcc_msg_send_reply_receive (
-		cfg_inst->ipc_ctx,
+		cfg_inst->handle,
 		&iov,
 		1,
 		&res_lib_cfg_local_get,
 		sizeof (struct res_lib_cfg_local_get));
 
-	pthread_mutex_unlock (&cfg_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -908,7 +741,44 @@ cs_error_t corosync_cfg_local_get (
 	*local_nodeid = res_lib_cfg_local_get.local_nodeid;
 
 error_exit:
-	(void)saHandleInstancePut (&cfg_hdb, handle);
+	(void)hdb_handle_put (&cfg_hdb, handle);
 
 	return (error);
 }
+
+cs_error_t
+corosync_cfg_crypto_set (
+	corosync_cfg_handle_t handle,
+	unsigned int type)
+{
+	struct cfg_instance *cfg_instance;
+	struct req_lib_cfg_crypto_set req_lib_cfg_crypto_set;
+	struct res_lib_cfg_crypto_set res_lib_cfg_crypto_set;
+	struct iovec iov;
+	cs_error_t error;
+
+
+	error = hdb_error_to_cs(hdb_handle_get (&cfg_hdb, handle, (void *)&cfg_instance));
+	if (error != CS_OK) {
+		return (error);
+	}
+
+	req_lib_cfg_crypto_set.header.id = MESSAGE_REQ_CFG_CRYPTO_SET;
+	req_lib_cfg_crypto_set.header.size = sizeof (struct req_lib_cfg_crypto_set);
+	req_lib_cfg_crypto_set.type = type;
+
+	iov.iov_base = (void *)&req_lib_cfg_crypto_set;
+	iov.iov_len = sizeof (struct req_lib_cfg_crypto_set);
+
+	error = coroipcc_msg_send_reply_receive (cfg_instance->handle,
+		&iov,
+		1,
+		&res_lib_cfg_crypto_set,
+		sizeof (struct res_lib_cfg_crypto_set));
+
+	if (error == CS_OK)
+		error = res_lib_cfg_crypto_set.header.error;
+
+	(void)hdb_handle_put (&cfg_hdb, handle);
+	return (error);
+}
