@@ -45,14 +45,22 @@
 #include <errno.h>
 
 #include <corosync/corotypes.h>
-#include <corosync/confdb.h>
-#include <corosync/ipc_confdb.h>
-#include <corosync/mar_gen.h>
+#include <corosync/coroipc_types.h>
 #include <corosync/coroipcc.h>
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
 #include <corosync/list.h>
 
+#include <corosync/confdb.h>
+#include <corosync/ipc_confdb.h>
+
+#include "util.h"
+
 #include "sa-confdb.h"
 
+#undef MIN
+#define MIN(x,y) ((x) < (y) ? (x) : (y))
+
 /* Hold the information for iterators so that
    callers can do recursive tree traversals.
    each object_handle can have its own iterator */
@@ -64,28 +72,18 @@ struct iter_context {
 };
 
 struct confdb_inst {
-	void *ipc_ctx;
+	hdb_handle_t handle;
 	int finalize;
 	int standalone;
 	confdb_callbacks_t callbacks;
 	const void *context;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 
 	struct list_head object_find_head;
 	struct list_head object_iter_head;
 	struct list_head key_iter_head;
 };
 
-static void confdb_instance_destructor (void *instance);
-
-static struct saHandleDatabase confdb_handle_t_db = {
-	.handleCount		        = 0,
-	.handles			= 0,
-	.mutex				= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= confdb_instance_destructor
-};
-
+DECLARE_HDB_DATABASE(confdb_handle_t_db,NULL);
 
 static cs_error_t do_find_destroy(struct confdb_inst *confdb_inst, hdb_handle_t find_handle);
 
@@ -105,17 +103,6 @@ static void free_context_list(struct confdb_inst *confdb_inst, struct list_head
 	}
 }
 
-/*
- * Clean up function for a confdb instance (confdb_initialize) handle
- */
-static void confdb_instance_destructor (void *instance)
-{
-	struct confdb_inst *confdb_inst = instance;
-
-	pthread_mutex_destroy (&confdb_inst->response_mutex);
-	pthread_mutex_destroy (&confdb_inst->dispatch_mutex);
-}
-
 static struct iter_context *find_iter_context(struct list_head *list, hdb_handle_t object_handle)
 {
 	struct iter_context *context;
@@ -145,12 +132,12 @@ cs_error_t confdb_initialize (
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 
-	error = saHandleCreate (&confdb_handle_t_db, sizeof (struct confdb_inst), handle);
+	error = hdb_error_to_cs(hdb_handle_create (&confdb_handle_t_db, sizeof (struct confdb_inst), handle));
 	if (error != CS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, *handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, *handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		goto error_destroy;
 	}
@@ -160,28 +147,31 @@ cs_error_t confdb_initialize (
 		confdb_inst->standalone = 1;
 	}
 	else {
-		error = coroipcc_service_connect (IPC_SOCKET_NAME, CONFDB_SERVICE, &confdb_inst->ipc_ctx);
+		error = coroipcc_service_connect (
+			COROSYNC_SOCKET_NAME,
+			CONFDB_SERVICE,
+			IPC_REQUEST_SIZE,
+			IPC_RESPONSE_SIZE,
+			IPC_DISPATCH_SIZE,
+			&confdb_inst->handle);
 	}
 	if (error != CS_OK)
 		goto error_put_destroy;
 
 	memcpy (&confdb_inst->callbacks, callbacks, sizeof (confdb_callbacks_t));
 
-	pthread_mutex_init (&confdb_inst->response_mutex, NULL);
-	pthread_mutex_init (&confdb_inst->dispatch_mutex, NULL);
-
 	list_init (&confdb_inst->object_find_head);
 	list_init (&confdb_inst->object_iter_head);
 	list_init (&confdb_inst->key_iter_head);
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, *handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, *handle);
 
 	return (CS_OK);
 
 error_put_destroy:
-	(void)saHandleInstancePut (&confdb_handle_t_db, *handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, *handle);
 error_destroy:
-	(void)saHandleDestroy (&confdb_handle_t_db, *handle);
+	(void)hdb_handle_destroy (&confdb_handle_t_db, *handle);
 error_no_destroy:
 	return (error);
 }
@@ -192,38 +182,33 @@ cs_error_t confdb_finalize (
 	struct confdb_inst *confdb_inst;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (confdb_inst->finalize) {
-		pthread_mutex_unlock (&confdb_inst->response_mutex);
-		(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+		(void)hdb_handle_put (&confdb_handle_t_db, handle);
 		return (CS_ERR_BAD_HANDLE);
 	}
 
 	confdb_inst->finalize = 1;
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
-
 	/* Free saved context handles */
 	free_context_list(confdb_inst, &confdb_inst->object_find_head);
 	free_context_list(confdb_inst, &confdb_inst->object_iter_head);
 	free_context_list(confdb_inst, &confdb_inst->key_iter_head);
 
 	if (!confdb_inst->standalone) {
-		coroipcc_service_disconnect (confdb_inst->ipc_ctx);
+		coroipcc_service_disconnect (confdb_inst->handle);
 	}
 
-	(void)saHandleDestroy (&confdb_handle_t_db, handle);
+	(void)hdb_handle_destroy (&confdb_handle_t_db, handle);
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -235,16 +220,16 @@ cs_error_t confdb_fd_get (
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	*fd = coroipcc_fd_get (confdb_inst->ipc_ctx);
+	error = coroipcc_fd_get (confdb_inst->handle, fd);
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
-	return (CS_OK);
+	return (error);
 }
 
 cs_error_t confdb_context_get (
@@ -254,14 +239,14 @@ cs_error_t confdb_context_get (
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	*context = confdb_inst->context;
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -273,23 +258,18 @@ cs_error_t confdb_context_set (
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	confdb_inst->context = context;
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (CS_OK);
 }
 
-struct confdb_res_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char data[512000];
-};
-
 cs_error_t confdb_dispatch (
 	confdb_handle_t handle,
 	cs_dispatch_flags_t dispatch_types)
@@ -297,15 +277,14 @@ cs_error_t confdb_dispatch (
 	int timeout = -1;
 	cs_error_t error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct confdb_inst *confdb_inst;
 	confdb_callbacks_t callbacks;
 	struct res_lib_confdb_key_change_callback *res_key_changed_pt;
 	struct res_lib_confdb_object_create_callback *res_object_created_pt;
 	struct res_lib_confdb_object_destroy_callback *res_object_destroyed_pt;
-	struct confdb_res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -324,27 +303,25 @@ cs_error_t confdb_dispatch (
 	}
 
 	do {
-		pthread_mutex_lock (&confdb_inst->dispatch_mutex);
-
-		dispatch_avail = coroipcc_dispatch_recv (confdb_inst->ipc_ctx,
-			(void *)&dispatch_data, timeout);
-
-
-		/*
-		 * Handle has been finalized in another thread
-		 */
-		if (confdb_inst->finalize == 1) {
+		error = coroipcc_dispatch_get (
+			confdb_inst->handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
 			error = CS_OK;
-			pthread_mutex_unlock (&confdb_inst->dispatch_mutex);
 			goto error_put;
 		}
-
-		if (dispatch_avail == 0 && dispatch_types == CONFDB_DISPATCH_ALL) {
-			break; /* exit do while cont is 1 loop */
-		} else
-		if (dispatch_avail == 0) {
-			continue; /* next poll */
+		if (error != CS_OK) {
+			goto error_put;
 		}
+		if (dispatch_data == NULL) {
+			if (dispatch_types == CONFDB_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
+			} else {
+				continue; /* next poll */
+			}
+		}
+
 
 		/*
 		 * Make copy of callbacks, message data, unlock instance, and call callback
@@ -353,14 +330,13 @@ cs_error_t confdb_dispatch (
 		*/
 		memcpy (&callbacks, &confdb_inst->callbacks, sizeof (confdb_callbacks_t));
 
-		pthread_mutex_unlock (&confdb_inst->dispatch_mutex);
 
 		/*
 		 * Dispatch incoming message
 		 */
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 			case MESSAGE_RES_CONFDB_KEY_CHANGE_CALLBACK:
-				res_key_changed_pt = (struct res_lib_confdb_key_change_callback *)&dispatch_data;
+				res_key_changed_pt = (struct res_lib_confdb_key_change_callback *)dispatch_data;
 
 				callbacks.confdb_key_change_notify_fn(handle,
 					res_key_changed_pt->change_type,
@@ -375,7 +351,7 @@ cs_error_t confdb_dispatch (
 				break;
 
 			case MESSAGE_RES_CONFDB_OBJECT_CREATE_CALLBACK:
-				res_object_created_pt = (struct res_lib_confdb_object_create_callback *)&dispatch_data;
+				res_object_created_pt = (struct res_lib_confdb_object_create_callback *)dispatch_data;
 
 				callbacks.confdb_object_create_change_notify_fn(handle,
 					res_object_created_pt->object_handle,
@@ -385,7 +361,7 @@ cs_error_t confdb_dispatch (
 				break;
 
 			case MESSAGE_RES_CONFDB_OBJECT_DESTROY_CALLBACK:
-				res_object_destroyed_pt = (struct res_lib_confdb_object_destroy_callback *)&dispatch_data;
+				res_object_destroyed_pt = (struct res_lib_confdb_object_destroy_callback *)dispatch_data;
 
 				callbacks.confdb_object_delete_change_notify_fn(handle,
 					res_object_destroyed_pt->parent_object_handle,
@@ -394,10 +370,12 @@ cs_error_t confdb_dispatch (
 				break;
 
 			default:
+				coroipcc_dispatch_put (confdb_inst->handle);
 				error = CS_ERR_LIBRARY;
 				goto error_noput;
 				break;
 		}
+		coroipcc_dispatch_put (confdb_inst->handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -414,7 +392,7 @@ cs_error_t confdb_dispatch (
 	} while (cont);
 
 error_put:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 error_noput:
 	return (error);
 }
@@ -423,7 +401,7 @@ cs_error_t confdb_object_create (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *object_name,
-	int object_name_len,
+	size_t object_name_len,
 	hdb_handle_t *object_handle)
 {
 	cs_error_t error;
@@ -432,7 +410,7 @@ cs_error_t confdb_object_create (
 	struct req_lib_confdb_object_create req_lib_confdb_object_create;
 	struct res_lib_confdb_object_create res_lib_confdb_object_create;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -456,16 +434,13 @@ cs_error_t confdb_object_create (
 	iov.iov_base = (char *)&req_lib_confdb_object_create;
 	iov.iov_len = sizeof (struct req_lib_confdb_object_create);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_object_create,
 		sizeof (struct res_lib_confdb_object_create));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -474,7 +449,7 @@ cs_error_t confdb_object_create (
 	*object_handle = res_lib_confdb_object_create.object_handle;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -487,9 +462,9 @@ cs_error_t confdb_object_destroy (
 	struct confdb_inst *confdb_inst;
 	struct iovec iov;
 	struct req_lib_confdb_object_destroy req_lib_confdb_object_destroy;
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -509,16 +484,13 @@ cs_error_t confdb_object_destroy (
 	iov.iov_base = (char *)&req_lib_confdb_object_destroy;
 	iov.iov_len = sizeof (struct req_lib_confdb_object_destroy);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res,
-		sizeof (mar_res_header_t));
+		sizeof (coroipc_response_header_t));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -526,7 +498,7 @@ cs_error_t confdb_object_destroy (
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -542,7 +514,7 @@ cs_error_t confdb_object_parent_get (
 	struct req_lib_confdb_object_parent_get req_lib_confdb_object_parent_get;
 	struct res_lib_confdb_object_parent_get res_lib_confdb_object_parent_get;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -562,16 +534,13 @@ cs_error_t confdb_object_parent_get (
 	iov.iov_base = (char *)&req_lib_confdb_object_parent_get;
 	iov.iov_len = sizeof (struct req_lib_confdb_object_parent_get);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_object_parent_get,
 		sizeof (struct res_lib_confdb_object_parent_get));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -580,7 +549,7 @@ cs_error_t confdb_object_parent_get (
 	*parent_object_handle = res_lib_confdb_object_parent_get.parent_object_handle;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -592,7 +561,7 @@ static cs_error_t do_find_destroy(
 	cs_error_t error;
 	struct iovec iov;
 	struct req_lib_confdb_object_find_destroy req_lib_confdb_object_find_destroy;
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
 	if (!find_handle)
 		return CS_OK;
@@ -612,16 +581,13 @@ static cs_error_t do_find_destroy(
 	iov.iov_base = (char *)&req_lib_confdb_object_find_destroy;
 	iov.iov_len = sizeof (struct req_lib_confdb_object_find_destroy);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res,
-		sizeof (mar_res_header_t));
+		sizeof (coroipc_response_header_t));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -641,7 +607,7 @@ cs_error_t confdb_object_find_destroy(
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -653,7 +619,7 @@ cs_error_t confdb_object_find_destroy(
 		free(context);
 	}
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 	return error;
 }
 
@@ -665,7 +631,7 @@ cs_error_t confdb_object_iter_destroy(
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -677,7 +643,7 @@ cs_error_t confdb_object_iter_destroy(
 		free(context);
 	}
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 	return error;
 }
 
@@ -686,17 +652,17 @@ cs_error_t confdb_key_create (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	const void *value,
-	int value_len)
+	size_t value_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 	struct iovec iov;
 	struct req_lib_confdb_key_create req_lib_confdb_key_create;
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -722,16 +688,13 @@ cs_error_t confdb_key_create (
 	iov.iov_base = (char *)&req_lib_confdb_key_create;
 	iov.iov_len = sizeof (struct req_lib_confdb_key_create);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res,
 		sizeof (res));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -739,7 +702,7 @@ cs_error_t confdb_key_create (
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -748,17 +711,17 @@ cs_error_t confdb_key_delete (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	const void *value,
-	int value_len)
+	size_t value_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 	struct iovec iov;
 	struct req_lib_confdb_key_delete req_lib_confdb_key_delete;
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -784,16 +747,13 @@ cs_error_t confdb_key_delete (
 	iov.iov_base = (char *)&req_lib_confdb_key_delete;
 	iov.iov_len = sizeof (struct req_lib_confdb_key_delete);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res,
 		sizeof (res));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -801,7 +761,7 @@ cs_error_t confdb_key_delete (
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -810,9 +770,9 @@ cs_error_t confdb_key_get (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	void *value,
-	int *value_len)
+	size_t *value_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
@@ -820,7 +780,7 @@ cs_error_t confdb_key_get (
 	struct req_lib_confdb_key_get req_lib_confdb_key_get;
 	struct res_lib_confdb_key_get res_lib_confdb_key_get;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -844,16 +804,13 @@ cs_error_t confdb_key_get (
 	iov.iov_base = (char *)&req_lib_confdb_key_get;
 	iov.iov_len = sizeof (struct req_lib_confdb_key_get);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_key_get,
 		sizeof (struct res_lib_confdb_key_get));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -865,7 +822,7 @@ cs_error_t confdb_key_get (
 	}
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -874,7 +831,7 @@ cs_error_t confdb_key_increment (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	unsigned int *value)
 {
 	cs_error_t error;
@@ -883,7 +840,7 @@ cs_error_t confdb_key_increment (
 	struct req_lib_confdb_key_get req_lib_confdb_key_get;
 	struct res_lib_confdb_key_incdec res_lib_confdb_key_incdec;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -907,16 +864,13 @@ cs_error_t confdb_key_increment (
 	iov.iov_base = (char *)&req_lib_confdb_key_get;
 	iov.iov_len = sizeof (struct req_lib_confdb_key_get);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_key_incdec,
 		sizeof (struct res_lib_confdb_key_incdec));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -927,7 +881,7 @@ cs_error_t confdb_key_increment (
 	}
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -936,7 +890,7 @@ cs_error_t confdb_key_decrement (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	unsigned int *value)
 {
 	cs_error_t error;
@@ -945,7 +899,7 @@ cs_error_t confdb_key_decrement (
 	struct req_lib_confdb_key_get req_lib_confdb_key_get;
 	struct res_lib_confdb_key_incdec res_lib_confdb_key_incdec;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -969,16 +923,13 @@ cs_error_t confdb_key_decrement (
 	iov.iov_base = (char *)&req_lib_confdb_key_get;
 	iov.iov_len = sizeof (struct req_lib_confdb_key_get);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_key_incdec,
 		sizeof (struct res_lib_confdb_key_incdec));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -989,7 +940,7 @@ cs_error_t confdb_key_decrement (
 	}
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -998,19 +949,19 @@ cs_error_t confdb_key_replace (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *key_name,
-	int key_name_len,
+	size_t key_name_len,
 	const void *old_value,
-	int old_value_len,
+	size_t old_value_len,
 	const void *new_value,
-	int new_value_len)
+	size_t new_value_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 	struct iovec iov;
 	struct req_lib_confdb_key_replace req_lib_confdb_key_replace;
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1038,16 +989,13 @@ cs_error_t confdb_key_replace (
 	iov.iov_base = (char *)&req_lib_confdb_key_replace;
 	iov.iov_len = sizeof (struct req_lib_confdb_key_replace);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res,
 		sizeof (res));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -1055,7 +1003,7 @@ cs_error_t confdb_key_replace (
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -1068,7 +1016,7 @@ cs_error_t confdb_object_iter_start (
 	cs_error_t error = CS_OK;
 	struct iter_context *context;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1091,7 +1039,7 @@ cs_error_t confdb_object_iter_start (
 		context->find_handle = 0;
 	}
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 ret:
 	return error;
@@ -1105,7 +1053,7 @@ cs_error_t confdb_key_iter_start (
 	cs_error_t error = CS_OK;
 	struct iter_context *context;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1124,7 +1072,7 @@ cs_error_t confdb_key_iter_start (
 	context->find_handle = 0;
 	context->next_entry = 0;
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 ret:
 	return error;
@@ -1138,7 +1086,7 @@ cs_error_t confdb_object_find_start (
 	cs_error_t error = CS_OK;
 	struct iter_context *context;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1160,7 +1108,7 @@ cs_error_t confdb_object_find_start (
 		context->find_handle = 0;
 	}
 
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 ret:
 	return error;
@@ -1170,7 +1118,7 @@ cs_error_t confdb_object_find (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	const void *object_name,
-	int object_name_len,
+	size_t object_name_len,
 	hdb_handle_t *object_handle)
 {
 	cs_error_t error;
@@ -1180,7 +1128,7 @@ cs_error_t confdb_object_find (
 	struct req_lib_confdb_object_find req_lib_confdb_object_find;
 	struct res_lib_confdb_object_find res_lib_confdb_object_find;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1213,16 +1161,13 @@ cs_error_t confdb_object_find (
 	iov.iov_base = (char *)&req_lib_confdb_object_find;
 	iov.iov_len = sizeof (struct req_lib_confdb_object_find);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_object_find,
 		sizeof (struct res_lib_confdb_object_find));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -1232,7 +1177,7 @@ cs_error_t confdb_object_find (
 	context->find_handle = res_lib_confdb_object_find.find_handle;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -1243,7 +1188,7 @@ cs_error_t confdb_object_iter (
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t *object_handle,
 	void *object_name,
-	int *object_name_len)
+	size_t *object_name_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
@@ -1252,7 +1197,7 @@ cs_error_t confdb_object_iter (
 	struct req_lib_confdb_object_iter req_lib_confdb_object_iter;
 	struct res_lib_confdb_object_iter res_lib_confdb_object_iter;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1285,16 +1230,13 @@ cs_error_t confdb_object_iter (
 	iov.iov_base = (char *)&req_lib_confdb_object_iter;
 	iov.iov_len = sizeof (struct req_lib_confdb_object_iter);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_object_iter,
 		sizeof (struct res_lib_confdb_object_iter));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -1309,7 +1251,7 @@ cs_error_t confdb_object_iter (
 sa_exit:
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -1318,9 +1260,9 @@ cs_error_t confdb_key_iter (
 	confdb_handle_t handle,
 	hdb_handle_t parent_object_handle,
 	void *key_name,
-	int *key_name_len,
+	size_t *key_name_len,
 	void *value,
-	int *value_len)
+	size_t *value_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
@@ -1329,7 +1271,7 @@ cs_error_t confdb_key_iter (
 	struct req_lib_confdb_key_iter req_lib_confdb_key_iter;
 	struct res_lib_confdb_key_iter res_lib_confdb_key_iter;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1360,16 +1302,13 @@ cs_error_t confdb_key_iter (
 	iov.iov_base = (char *)&req_lib_confdb_key_iter;
 	iov.iov_len = sizeof (struct req_lib_confdb_key_iter);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_key_iter,
 		sizeof (struct res_lib_confdb_key_iter));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -1386,60 +1325,63 @@ sa_exit:
 	context->next_entry++;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
 
 cs_error_t confdb_write (
 	confdb_handle_t handle,
-	char *error_text)
+	char *error_text,
+	size_t errbuf_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 	struct iovec iov;
-	mar_req_header_t req;
+	coroipc_request_header_t req;
 	struct res_lib_confdb_write res_lib_confdb_write;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
+		/* FIXME: set error_text */
 		return (error);
 	}
 
 	if (confdb_inst->standalone) {
 		error = CS_OK;
 
-		if (confdb_sa_write(error_text))
+		if (confdb_sa_write(error_text, errbuf_len))
 			error = CS_ERR_ACCESS;
 		goto error_exit;
 	}
 
-	req.size = sizeof (mar_req_header_t);
+	req.size = sizeof (coroipc_request_header_t);
 	req.id = MESSAGE_REQ_CONFDB_WRITE;
 
 	iov.iov_base = (char *)&req;
-	iov.iov_len = sizeof (mar_req_header_t);
-
-	pthread_mutex_lock (&confdb_inst->response_mutex);
+	iov.iov_len = sizeof (coroipc_request_header_t);
 
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_write,
 		sizeof (struct res_lib_confdb_write));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
+		/* FIXME: set error_text */
 		goto error_exit;
 	}
 
 	error = res_lib_confdb_write.header.error;
-	if (res_lib_confdb_write.error.length)
-		memcpy(error_text, res_lib_confdb_write.error.value, res_lib_confdb_write.error.length);
+	if (res_lib_confdb_write.error.length) {
+		memcpy(error_text, res_lib_confdb_write.error.value,
+		       MIN(res_lib_confdb_write.error.length,errbuf_len));
+		error_text[errbuf_len-1] = '\0';
+	}
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -1447,7 +1389,8 @@ error_exit:
 cs_error_t confdb_reload (
 	confdb_handle_t handle,
 	int flush,
-	char *error_text)
+	char *error_text,
+	size_t errbuf_len)
 {
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
@@ -1455,15 +1398,16 @@ cs_error_t confdb_reload (
 	struct res_lib_confdb_reload res_lib_confdb_reload;
 	struct req_lib_confdb_reload req_lib_confdb_reload;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
+		/* FIXME: set error_text */
 		return (error);
 	}
 
 	if (confdb_inst->standalone) {
 		error = CS_OK;
 
-		if (confdb_sa_reload(flush, error_text))
+		if (confdb_sa_reload(flush, error_text, errbuf_len))
 			error = CS_ERR_ACCESS;
 		goto error_exit;
 	}
@@ -1475,27 +1419,27 @@ cs_error_t confdb_reload (
 	iov.iov_base = (char *)&req_lib_confdb_reload;
 	iov.iov_len = sizeof (req_lib_confdb_reload);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res_lib_confdb_reload,
 		sizeof (struct res_lib_confdb_reload));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
-
 	if (error != CS_OK) {
+		/* FIXME: set error_text */
 		goto error_exit;
 	}
 
 	error = res_lib_confdb_reload.header.error;
-	if(res_lib_confdb_reload.error.length)
-		memcpy(error_text, res_lib_confdb_reload.error.value, res_lib_confdb_reload.error.length);
+	if(res_lib_confdb_reload.error.length) {
+		memcpy(error_text, res_lib_confdb_reload.error.value,
+		       MIN(res_lib_confdb_reload.error.length,errbuf_len));
+		error_text[errbuf_len-1] = '\0';
+	}
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -1509,9 +1453,9 @@ cs_error_t confdb_track_changes (
 	struct confdb_inst *confdb_inst;
 	struct iovec iov;
 	struct req_lib_confdb_object_track_start req;
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1529,16 +1473,13 @@ cs_error_t confdb_track_changes (
 	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof (struct req_lib_confdb_object_track_start);
 
-	pthread_mutex_lock (&confdb_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res,
-		sizeof (mar_res_header_t));
+		sizeof (coroipc_response_header_t));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -1546,7 +1487,7 @@ cs_error_t confdb_track_changes (
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
@@ -1556,10 +1497,10 @@ cs_error_t confdb_stop_track_changes (confdb_handle_t handle)
 	cs_error_t error;
 	struct confdb_inst *confdb_inst;
 	struct iovec iov;
-	mar_req_header_t req;
-	mar_res_header_t res;
+	coroipc_request_header_t req;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&confdb_handle_t_db, handle, (void *)&confdb_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&confdb_handle_t_db, handle, (void *)&confdb_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -1569,22 +1510,19 @@ cs_error_t confdb_stop_track_changes (confdb_handle_t handle)
 		goto error_exit;
 	}
 
-	req.size = sizeof (mar_req_header_t);
+	req.size = sizeof (coroipc_request_header_t);
 	req.id = MESSAGE_REQ_CONFDB_TRACK_STOP;
 
 	iov.iov_base = (char *)&req;
-	iov.iov_len = sizeof (mar_req_header_t);
-
-	pthread_mutex_lock (&confdb_inst->response_mutex);
+	iov.iov_len = sizeof (coroipc_request_header_t);
 
         error = coroipcc_msg_send_reply_receive (
-		confdb_inst->ipc_ctx,
+		confdb_inst->handle,
 		&iov,
 		1,
                 &res,
-		sizeof (mar_res_header_t));
+		sizeof (coroipc_response_header_t));
 
-	pthread_mutex_unlock (&confdb_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -1592,8 +1530,7 @@ cs_error_t confdb_stop_track_changes (confdb_handle_t handle)
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&confdb_handle_t_db, handle);
+	(void)hdb_handle_put (&confdb_handle_t_db, handle);
 
 	return (error);
 }
-
