@@ -1,7 +1,7 @@
 /*
  * vi: set autoindent tabstop=4 shiftwidth=4 :
  *
- * Copyright (c) 2006-2008 Red Hat, Inc.
+ * Copyright (c) 2006-2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -42,46 +42,29 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <pthread.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
 
 #include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/coroipcc.h>
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
+
 #include <corosync/cpg.h>
 #include <corosync/ipc_cpg.h>
-#include <corosync/mar_cpg.h>
-#include <corosync/coroipcc.h>
+
+#include "util.h"
 
 struct cpg_inst {
-	void *ipc_ctx;
+	hdb_handle_t handle;
 	int finalize;
 	cpg_callbacks_t callbacks;
 	void *context;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 };
 
-static void cpg_instance_destructor (void *instance);
-
-static struct saHandleDatabase cpg_handle_t_db = {
-	.handleCount		        = 0,
-	.handles			= 0,
-	.mutex				= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= cpg_instance_destructor
-};
-
-/*
- * Clean up function for a cpg instance (cpg_nitialize) handle
- */
-static void cpg_instance_destructor (void *instance)
-{
-	struct cpg_inst *cpg_inst = instance;
-
-	pthread_mutex_destroy (&cpg_inst->response_mutex);
-	pthread_mutex_destroy (&cpg_inst->dispatch_mutex);
-}
-
+DECLARE_HDB_DATABASE(cpg_handle_t_db,NULL);
 
 /**
  * @defgroup cpg_coroipcc The closed process group API
@@ -97,35 +80,37 @@ cs_error_t cpg_initialize (
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
-	error = saHandleCreate (&cpg_handle_t_db, sizeof (struct cpg_inst), handle);
+	error = hdb_error_to_cs (hdb_handle_create (&cpg_handle_t_db, sizeof (struct cpg_inst), handle));
 	if (error != CS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, *handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, *handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, CPG_SERVICE, &cpg_inst->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		CPG_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		&cpg_inst->handle);
 	if (error != CS_OK) {
 		goto error_put_destroy;
 	}
 
 	memcpy (&cpg_inst->callbacks, callbacks, sizeof (cpg_callbacks_t));
 
-	pthread_mutex_init (&cpg_inst->response_mutex, NULL);
-
-	pthread_mutex_init (&cpg_inst->dispatch_mutex, NULL);
-
-	saHandleInstancePut (&cpg_handle_t_db, *handle);
+	hdb_handle_put (&cpg_handle_t_db, *handle);
 
 	return (CS_OK);
 
 error_put_destroy:
-	saHandleInstancePut (&cpg_handle_t_db, *handle);
+	hdb_handle_put (&cpg_handle_t_db, *handle);
 error_destroy:
-	saHandleDestroy (&cpg_handle_t_db, *handle);
+	hdb_handle_destroy (&cpg_handle_t_db, *handle);
 error_no_destroy:
 	return (error);
 }
@@ -136,31 +121,26 @@ cs_error_t cpg_finalize (
 	struct cpg_inst *cpg_inst;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&cpg_inst->response_mutex);
-
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (cpg_inst->finalize) {
-		pthread_mutex_unlock (&cpg_inst->response_mutex);
-		saHandleInstancePut (&cpg_handle_t_db, handle);
+		hdb_handle_put (&cpg_handle_t_db, handle);
 		return (CPG_ERR_BAD_HANDLE);
 	}
 
 	cpg_inst->finalize = 1;
 
-	coroipcc_service_disconnect (cpg_inst->ipc_ctx);
+	coroipcc_service_disconnect (cpg_inst->handle);
 
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
+	hdb_handle_destroy (&cpg_handle_t_db, handle);
 
-	saHandleDestroy (&cpg_handle_t_db, handle);
-
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (CPG_OK);
 }
@@ -172,16 +152,16 @@ cs_error_t cpg_fd_get (
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	*fd = coroipcc_fd_get (cpg_inst->ipc_ctx);
+	error = coroipcc_fd_get (cpg_inst->handle, fd);
 
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
-	return (CS_OK);
+	return (error);
 }
 
 cs_error_t cpg_context_get (
@@ -191,14 +171,14 @@ cs_error_t cpg_context_get (
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	*context = cpg_inst->context;
 
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -210,23 +190,18 @@ cs_error_t cpg_context_set (
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	cpg_inst->context = context;
 
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (CS_OK);
 }
 
-struct res_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char data[512000];
-};
-
 cs_error_t cpg_dispatch (
 	cpg_handle_t handle,
 	cs_dispatch_flags_t dispatch_types)
@@ -234,12 +209,11 @@ cs_error_t cpg_dispatch (
 	int timeout = -1;
 	cs_error_t error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct cpg_inst *cpg_inst;
 	struct res_lib_cpg_confchg_callback *res_cpg_confchg_callback;
 	struct res_lib_cpg_deliver_callback *res_cpg_deliver_callback;
 	cpg_callbacks_t callbacks;
-	struct res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 	int ignore_dispatch = 0;
 	struct cpg_address member_list[CPG_MEMBERS_MAX];
 	struct cpg_address left_list[CPG_MEMBERS_MAX];
@@ -249,7 +223,7 @@ cs_error_t cpg_dispatch (
 	mar_cpg_address_t *joined_list_start;
 	unsigned int i;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -263,32 +237,24 @@ cs_error_t cpg_dispatch (
 	}
 
 	do {
-		pthread_mutex_lock (&cpg_inst->dispatch_mutex);
-
-		dispatch_avail = coroipcc_dispatch_recv (cpg_inst->ipc_ctx,
-			(void *)&dispatch_data, timeout);
-
-		pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
-
+		error = coroipcc_dispatch_get (
+			cpg_inst->handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
+			error = CS_OK;
+			goto error_put;
+		}
 		if (error != CS_OK) {
 			goto error_put;
 		}
 
-		if (dispatch_avail == 0 && dispatch_types == CPG_DISPATCH_ALL) {
-			pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
-			break; /* exit do while cont is 1 loop */
-		} else
-		if (dispatch_avail == 0) {
-			pthread_mutex_unlock (&cpg_inst->dispatch_mutex);
-			continue; /* next poll */
-		}
-		if (dispatch_avail == -1) {
-			if (cpg_inst->finalize == 1) {
-				error = CS_OK;
+		if (dispatch_data == NULL) {
+			if (dispatch_types == CPG_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
 			} else {
-				error = CS_ERR_LIBRARY;
+				continue; /* next poll */
 			}
-			goto error_put;
 		}
 
 		/*
@@ -300,9 +266,9 @@ cs_error_t cpg_dispatch (
 		/*
 		 * Dispatch incoming message
 		 */
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 		case MESSAGE_RES_CPG_DELIVER_CALLBACK:
-			res_cpg_deliver_callback = (struct res_lib_cpg_deliver_callback *)&dispatch_data;
+			res_cpg_deliver_callback = (struct res_lib_cpg_deliver_callback *)dispatch_data;
 
 			marshall_from_mar_cpg_name_t (
 				&group_name,
@@ -317,7 +283,7 @@ cs_error_t cpg_dispatch (
 			break;
 
 		case MESSAGE_RES_CPG_CONFCHG_CALLBACK:
-			res_cpg_confchg_callback = (struct res_lib_cpg_confchg_callback *)&dispatch_data;
+			res_cpg_confchg_callback = (struct res_lib_cpg_confchg_callback *)dispatch_data;
 
 			for (i = 0; i < res_cpg_confchg_callback->member_list_entries; i++) {
 				marshall_from_mar_cpg_address_t (&member_list[i],
@@ -351,10 +317,12 @@ cs_error_t cpg_dispatch (
 			break;
 
 		default:
+			coroipcc_dispatch_put (cpg_inst->handle);
 			error = CS_ERR_LIBRARY;
 			goto error_put;
 			break;
 		}
+		coroipcc_dispatch_put (cpg_inst->handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -378,46 +346,25 @@ cs_error_t cpg_dispatch (
 	} while (cont);
 
 error_put:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 	return (error);
 }
 
 cs_error_t cpg_join (
     cpg_handle_t handle,
-    struct cpg_name *group)
+    const struct cpg_name *group)
 {
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 	struct iovec iov[2];
 	struct req_lib_cpg_join req_lib_cpg_join;
 	struct res_lib_cpg_join res_lib_cpg_join;
-	struct req_lib_cpg_trackstart req_lib_cpg_trackstart;
-	struct res_lib_cpg_trackstart res_lib_cpg_trackstart;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&cpg_inst->response_mutex);
-
-	/* Automatically add a tracker */
-	req_lib_cpg_trackstart.header.size = sizeof (struct req_lib_cpg_trackstart);
-	req_lib_cpg_trackstart.header.id = MESSAGE_REQ_CPG_TRACKSTART;
-	marshall_to_mar_cpg_name_t (&req_lib_cpg_trackstart.group_name,
-		group);
-
-	iov[0].iov_base = &req_lib_cpg_trackstart;
-	iov[0].iov_len = sizeof (struct req_lib_cpg_trackstart);
-
-	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov, 1,
-		&res_lib_cpg_trackstart, sizeof (struct res_lib_cpg_trackstart));
-
-	if (error != CS_OK) {
-		pthread_mutex_unlock (&cpg_inst->response_mutex);
-		goto error_exit;
-	}
-
 	/* Now join */
 	req_lib_cpg_join.header.size = sizeof (struct req_lib_cpg_join);
 	req_lib_cpg_join.header.id = MESSAGE_REQ_CPG_JOIN;
@@ -425,29 +372,29 @@ cs_error_t cpg_join (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_join.group_name,
 		group);
 
-	iov[0].iov_base = &req_lib_cpg_join;
+	iov[0].iov_base = (void *)&req_lib_cpg_join;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_join);
 
-	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov, 1,
-		&res_lib_cpg_join, sizeof (struct res_lib_cpg_join));
+	do {
+		error = coroipcc_msg_send_reply_receive (cpg_inst->handle, iov, 1,
+			&res_lib_cpg_join, sizeof (struct res_lib_cpg_join));
 
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
-
-	if (error != CS_OK) {
-		goto error_exit;
-	}
+		if (error != CS_OK) {
+			goto error_exit;
+		}
+	} while (res_lib_cpg_join.header.error == CPG_ERR_BUSY);
 
 	error = res_lib_cpg_join.header.error;
 
 error_exit:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
 }
 
 cs_error_t cpg_leave (
     cpg_handle_t handle,
-    struct cpg_name *group)
+    const struct cpg_name *group)
 {
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
@@ -455,7 +402,7 @@ cs_error_t cpg_leave (
 	struct req_lib_cpg_leave req_lib_cpg_leave;
 	struct res_lib_cpg_leave res_lib_cpg_leave;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -466,76 +413,22 @@ cs_error_t cpg_leave (
 	marshall_to_mar_cpg_name_t (&req_lib_cpg_leave.group_name,
 		group);
 
-	iov[0].iov_base = &req_lib_cpg_leave;
+	iov[0].iov_base = (void *)&req_lib_cpg_leave;
 	iov[0].iov_len = sizeof (struct req_lib_cpg_leave);
 
-	pthread_mutex_lock (&cpg_inst->response_mutex);
+	do {
+		error = coroipcc_msg_send_reply_receive (cpg_inst->handle, iov, 1,
+			&res_lib_cpg_leave, sizeof (struct res_lib_cpg_leave));
 
-	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov, 1,
-		&res_lib_cpg_leave, sizeof (struct res_lib_cpg_leave));
-
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
-	if (error != CS_OK) {
-		goto error_exit;
-	}
+		if (error != CS_OK) {
+			goto error_exit;
+		}
+	} while (res_lib_cpg_leave.header.error == CPG_ERR_BUSY);
 
 	error = res_lib_cpg_leave.header.error;
 
 error_exit:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
-
-	return (error);
-}
-
-cs_error_t cpg_mcast_joined (
-	cpg_handle_t handle,
-	cpg_guarantee_t guarantee,
-	struct iovec *iovec,
-	int iov_len)
-{
-	int i;
-	cs_error_t error;
-	struct cpg_inst *cpg_inst;
-	struct iovec iov[64];
-	struct req_lib_cpg_mcast req_lib_cpg_mcast;
-	struct res_lib_cpg_mcast res_lib_cpg_mcast;
-	int msg_len = 0;
-
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
-	if (error != CS_OK) {
-		return (error);
-	}
-
-	for (i = 0; i < iov_len; i++ ) {
-		msg_len += iovec[i].iov_len;
-	}
-
-	req_lib_cpg_mcast.header.size = sizeof (struct req_lib_cpg_mcast) +
-		msg_len;
-
-	req_lib_cpg_mcast.header.id = MESSAGE_REQ_CPG_MCAST;
-	req_lib_cpg_mcast.guarantee = guarantee;
-	req_lib_cpg_mcast.msglen = msg_len;
-
-	iov[0].iov_base = &req_lib_cpg_mcast;
-	iov[0].iov_len = sizeof (struct req_lib_cpg_mcast);
-	memcpy (&iov[1], iovec, iov_len * sizeof (struct iovec));
-
-	pthread_mutex_lock (&cpg_inst->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, iov,
-		iov_len + 1, &res_lib_cpg_mcast, sizeof (res_lib_cpg_mcast));
-
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
-
-	if (error != CS_OK) {
-		goto error_exit;
-	}
-
-	error = res_lib_cpg_mcast.header.error;
-
-error_exit:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
 }
@@ -553,27 +446,25 @@ cs_error_t cpg_membership_get (
 	struct res_lib_cpg_confchg_callback res_lib_cpg_membership_get;
 	unsigned int i;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	req_lib_cpg_membership_get.header.size = sizeof (mar_req_header_t);
+	req_lib_cpg_membership_get.header.size = sizeof (coroipc_request_header_t);
 	req_lib_cpg_membership_get.header.id = MESSAGE_REQ_CPG_MEMBERSHIP;
 
-	iov.iov_base = &req_lib_cpg_membership_get;
-	iov.iov_len = sizeof (mar_req_header_t);
+	iov.iov_base = (void *)&req_lib_cpg_membership_get;
+	iov.iov_len = sizeof (coroipc_request_header_t);
 
-	pthread_mutex_lock (&cpg_inst->response_mutex);
+	do {
+		error = coroipcc_msg_send_reply_receive (cpg_inst->handle, &iov, 1,
+			&res_lib_cpg_membership_get, sizeof (coroipc_response_header_t));
 
-	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, &iov, 1,
-		&res_lib_cpg_membership_get, sizeof (mar_res_header_t));
-
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
-
-	if (error != CS_OK) {
-		goto error_exit;
-	}
+ 		if (error != CS_OK) {
+ 			goto error_exit;
+		}
+	} while (res_lib_cpg_membership_get.header.error == CPG_ERR_BUSY);
 
 	error = res_lib_cpg_membership_get.header.error;
 
@@ -589,7 +480,7 @@ cs_error_t cpg_membership_get (
 	}
 
 error_exit:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
 }
@@ -604,24 +495,20 @@ cs_error_t cpg_local_get (
 	struct req_lib_cpg_local_get req_lib_cpg_local_get;
 	struct res_lib_cpg_local_get res_lib_cpg_local_get;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	req_lib_cpg_local_get.header.size = sizeof (mar_req_header_t);
+	req_lib_cpg_local_get.header.size = sizeof (coroipc_request_header_t);
 	req_lib_cpg_local_get.header.id = MESSAGE_REQ_CPG_LOCAL_GET;
 
-	iov.iov_base = &req_lib_cpg_local_get;
+	iov.iov_base = (void *)&req_lib_cpg_local_get;
 	iov.iov_len = sizeof (struct req_lib_cpg_local_get);
 
-	pthread_mutex_lock (&cpg_inst->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (cpg_inst->ipc_ctx, &iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->handle, &iov, 1,
 		&res_lib_cpg_local_get, sizeof (res_lib_cpg_local_get));
 
-	pthread_mutex_unlock (&cpg_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -631,7 +518,7 @@ cs_error_t cpg_local_get (
 	*local_nodeid = res_lib_cpg_local_get.local_nodeid;
 
 error_exit:
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
 }
@@ -643,14 +530,148 @@ cs_error_t cpg_flow_control_state_get (
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
-	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
-	
-	*flow_control_state = coroipcc_dispatch_flow_control_get (cpg_inst->ipc_ctx);
 
-	saHandleInstancePut (&cpg_handle_t_db, handle);
+	error = coroipcc_dispatch_flow_control_get (cpg_inst->handle, (unsigned int *)flow_control_state);
+
+	hdb_handle_put (&cpg_handle_t_db, handle);
+
+	return (error);
+}
+
+cs_error_t cpg_zcb_alloc (
+	cpg_handle_t handle,
+	size_t size,
+	void **buffer)
+{
+	cs_error_t error;
+	struct cpg_inst *cpg_inst;
+
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
+	if (error != CS_OK) {
+		return (error);
+	}
+
+	error = coroipcc_zcb_alloc (cpg_inst->handle,
+		buffer,
+		size,
+		sizeof (struct req_lib_cpg_mcast));
+
+	hdb_handle_put (&cpg_handle_t_db, handle);
+	*buffer = ((char *)*buffer) + sizeof (struct req_lib_cpg_mcast);
+
+	return (error);
+}
+
+cs_error_t cpg_zcb_free (
+	cpg_handle_t handle,
+	void *buffer)
+{
+	cs_error_t error;
+	struct cpg_inst *cpg_inst;
+
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
+	if (error != CS_OK) {
+		return (error);
+	}
+
+	coroipcc_zcb_free (cpg_inst->handle, ((char *)buffer) - sizeof (struct req_lib_cpg_mcast));
+
+	hdb_handle_put (&cpg_handle_t_db, handle);
+
+	return (error);
+}
+
+cs_error_t cpg_zcb_mcast_joined (
+	cpg_handle_t handle,
+	cpg_guarantee_t guarantee,
+	void *msg,
+	size_t msg_len)
+{
+	cs_error_t error;
+	struct cpg_inst *cpg_inst;
+	struct req_lib_cpg_mcast *req_lib_cpg_mcast;
+	struct res_lib_cpg_mcast res_lib_cpg_mcast;
+
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
+	if (error != CS_OK) {
+		return (error);
+	}
+
+	req_lib_cpg_mcast = (struct req_lib_cpg_mcast *)(((char *)msg) - sizeof (struct req_lib_cpg_mcast));
+	req_lib_cpg_mcast->header.size = sizeof (struct req_lib_cpg_mcast) +
+		msg_len;
+
+	req_lib_cpg_mcast->header.id = MESSAGE_REQ_CPG_MCAST;
+	req_lib_cpg_mcast->guarantee = guarantee;
+	req_lib_cpg_mcast->msglen = msg_len;
+
+	error = coroipcc_zcb_msg_send_reply_receive (
+		cpg_inst->handle,
+		req_lib_cpg_mcast,
+		&res_lib_cpg_mcast,
+		sizeof (res_lib_cpg_mcast));
+
+	if (error != CS_OK) {
+		goto error_exit;
+	}
+
+	error = res_lib_cpg_mcast.header.error;
+
+error_exit:
+	hdb_handle_put (&cpg_handle_t_db, handle);
+
+	return (error);
+}
+
+cs_error_t cpg_mcast_joined (
+	cpg_handle_t handle,
+	cpg_guarantee_t guarantee,
+	const struct iovec *iovec,
+	unsigned int iov_len)
+{
+	int i;
+	cs_error_t error;
+	struct cpg_inst *cpg_inst;
+	struct iovec iov[64];
+	struct req_lib_cpg_mcast req_lib_cpg_mcast;
+	struct res_lib_cpg_mcast res_lib_cpg_mcast;
+	size_t msg_len = 0;
+
+	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
+	if (error != CS_OK) {
+		return (error);
+	}
+
+	for (i = 0; i < iov_len; i++ ) {
+		msg_len += iovec[i].iov_len;
+	}
+
+	req_lib_cpg_mcast.header.size = sizeof (struct req_lib_cpg_mcast) +
+		msg_len;
+
+	req_lib_cpg_mcast.header.id = MESSAGE_REQ_CPG_MCAST;
+	req_lib_cpg_mcast.guarantee = guarantee;
+	req_lib_cpg_mcast.msglen = msg_len;
+
+	iov[0].iov_base = (void *)&req_lib_cpg_mcast;
+	iov[0].iov_len = sizeof (struct req_lib_cpg_mcast);
+	memcpy (&iov[1], iovec, iov_len * sizeof (struct iovec));
+
+	error = coroipcc_msg_send_reply_receive (cpg_inst->handle, iov,
+		iov_len + 1, &res_lib_cpg_mcast, sizeof (res_lib_cpg_mcast));
+
+	if (error != CS_OK) {
+		goto error_exit;
+	}
+
+	error = res_lib_cpg_mcast.header.error;
+
+error_exit:
+	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
 }
