@@ -2,14 +2,14 @@
  * vi: set autoindent tabstop=4 shiftwidth=4 :
 
  * Copyright (c) 2004-2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006-2007, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -43,49 +43,37 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <pthread.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
 
-#include <corosync/totem/totem.h>
+
 #include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/coroipcc.h>
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
+
 #include <corosync/evs.h>
 #include <corosync/ipc_evs.h>
-#include <corosync/coroipcc.h>
+
+#include "util.h"
+
+#undef MIN
+#define MIN(x,y) ((x) < (y) ? (x) : (y))
 
 struct evs_inst {
-	void *ipc_ctx;
+	hdb_handle_t handle;
 	int finalize;
 	evs_callbacks_t callbacks;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
+	void *context;
 };
 
-struct res_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char data[512000];
-};
-
-static void evs_instance_destructor (void *instance);
-
-static struct saHandleDatabase evs_handle_t_db = {
-	.handleCount				= 0,
-	.handles					= 0,
-	.mutex						= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= evs_instance_destructor
-};
+DECLARE_HDB_DATABASE (evs_handle_t_db,NULL);
 
 /*
  * Clean up function for an evt instance (saEvtInitialize) handle
  */
-static void evs_instance_destructor (void *instance)
-{
-	struct evs_inst *evs_inst = instance;
-
-	pthread_mutex_destroy (&evs_inst->response_mutex);
-	pthread_mutex_destroy (&evs_inst->dispatch_mutex);
-}
 
 
 /**
@@ -107,35 +95,37 @@ evs_error_t evs_initialize (
 	cs_error_t error;
 	struct evs_inst *evs_inst;
 
-	error = saHandleCreate (&evs_handle_t_db, sizeof (struct evs_inst), handle);
+	error = hdb_error_to_cs(hdb_handle_create (&evs_handle_t_db, sizeof (struct evs_inst), handle));
 	if (error != CS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&evs_handle_t_db, *handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, *handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, EVS_SERVICE, &evs_inst->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		EVS_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		&evs_inst->handle);
 	if (error != EVS_OK) {
 		goto error_put_destroy;
 	}
 
 	memcpy (&evs_inst->callbacks, callbacks, sizeof (evs_callbacks_t));
 
-	pthread_mutex_init (&evs_inst->response_mutex, NULL);
-
-	pthread_mutex_init (&evs_inst->dispatch_mutex, NULL);
-
-	saHandleInstancePut (&evs_handle_t_db, *handle);
+	hdb_handle_put (&evs_handle_t_db, *handle);
 
 	return (CS_OK);
 
 error_put_destroy:
-	saHandleInstancePut (&evs_handle_t_db, *handle);
+	hdb_handle_put (&evs_handle_t_db, *handle);
 error_destroy:
-	saHandleDestroy (&evs_handle_t_db, *handle);
+	hdb_handle_destroy (&evs_handle_t_db, *handle);
 error_no_destroy:
 	return (error);
 }
@@ -146,30 +136,26 @@ evs_error_t evs_finalize (
 	struct evs_inst *evs_inst;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
-	pthread_mutex_lock (&evs_inst->response_mutex);
 
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (evs_inst->finalize) {
-		pthread_mutex_unlock (&evs_inst->response_mutex);
-		saHandleInstancePut (&evs_handle_t_db, handle);
+		hdb_handle_put (&evs_handle_t_db, handle);
 		return (EVS_ERR_BAD_HANDLE);
 	}
 
 	evs_inst->finalize = 1;
 
-	coroipcc_service_disconnect (evs_inst->ipc_ctx);
+	coroipcc_service_disconnect (evs_inst->handle);
 
-	pthread_mutex_unlock (&evs_inst->response_mutex);
+	hdb_handle_destroy (&evs_handle_t_db, handle);
 
-	saHandleDestroy (&evs_handle_t_db, handle);
-
-	saHandleInstancePut (&evs_handle_t_db, handle);
+	hdb_handle_put (&evs_handle_t_db, handle);
 
 	return (EVS_OK);
 }
@@ -181,14 +167,52 @@ evs_error_t evs_fd_get (
 	cs_error_t error;
 	struct evs_inst *evs_inst;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	*fd = coroipcc_fd_get (evs_inst->ipc_ctx);
+	coroipcc_fd_get (evs_inst->handle, fd);
 
-	saHandleInstancePut (&evs_handle_t_db, handle);
+	hdb_handle_put (&evs_handle_t_db, handle);
+
+	return (CS_OK);
+}
+
+evs_error_t evs_context_get (
+	evs_handle_t handle,
+	void **context)
+{
+	cs_error_t error;
+	struct evs_inst *evs_inst;
+
+	error = hdb_error_to_cs (hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
+	if (error != CS_OK) {
+		return (error);
+	}
+
+	*context = evs_inst->context;
+
+	hdb_handle_put (&evs_handle_t_db, handle);
+
+	return (CS_OK);
+}
+
+cs_error_t evs_context_set (
+	evs_handle_t handle,
+	void *context)
+{
+	cs_error_t error;
+	struct evs_inst *evs_inst;
+
+	error = hdb_error_to_cs (hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
+	if (error != CS_OK) {
+		return (error);
+	}
+
+	evs_inst->context = context;
+
+	hdb_handle_put (&evs_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -200,15 +224,14 @@ evs_error_t evs_dispatch (
 	int timeout = -1;
 	cs_error_t error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct evs_inst *evs_inst;
 	struct res_evs_confchg_callback *res_evs_confchg_callback;
 	struct res_evs_deliver_callback *res_evs_deliver_callback;
 	evs_callbacks_t callbacks;
-	struct res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 	int ignore_dispatch = 0;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -222,31 +245,24 @@ evs_error_t evs_dispatch (
 	}
 
 	do {
-		dispatch_avail = coroipcc_dispatch_recv (evs_inst->ipc_ctx, (void *)&dispatch_data, timeout);
-		if (dispatch_avail == -1) {
-			error = CS_ERR_LIBRARY;
-			goto error_nounlock;
+		error = coroipcc_dispatch_get (
+			evs_inst->handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
+			error = CS_OK;
+			goto error_put;
 		}
-			
-
-		pthread_mutex_lock (&evs_inst->dispatch_mutex);
-
-		/*
-		 * Handle has been finalized in another thread
-		 */
-		if (evs_inst->finalize == 1) {
-			error = EVS_OK;
-			pthread_mutex_unlock (&evs_inst->dispatch_mutex);
-			goto error_unlock;
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
-		if (dispatch_avail == 0 && dispatch_types == EVS_DISPATCH_ALL) {
-			pthread_mutex_unlock (&evs_inst->dispatch_mutex);
-			break; /* exit do while cont is 1 loop */
-		} else 
-		if (dispatch_avail == 0) {
-			pthread_mutex_unlock (&evs_inst->dispatch_mutex);
-			continue; /* next dispatch event */
+		if (dispatch_data == NULL) {
+			if (dispatch_types == CPG_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
+			} else {
+				continue; /* next poll */
+			}
 		}
 
 		/*
@@ -256,35 +272,39 @@ evs_error_t evs_dispatch (
 		*/
 		memcpy (&callbacks, &evs_inst->callbacks, sizeof (evs_callbacks_t));
 
-		pthread_mutex_unlock (&evs_inst->dispatch_mutex);
 		/*
 		 * Dispatch incoming message
 		 */
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 		case MESSAGE_RES_EVS_DELIVER_CALLBACK:
-			res_evs_deliver_callback = (struct res_evs_deliver_callback *)&dispatch_data;
+			res_evs_deliver_callback = (struct res_evs_deliver_callback *)dispatch_data;
 			callbacks.evs_deliver_fn (
+				handle,
 				res_evs_deliver_callback->local_nodeid,
 				&res_evs_deliver_callback->msg,
 				res_evs_deliver_callback->msglen);
 			break;
 
 		case MESSAGE_RES_EVS_CONFCHG_CALLBACK:
-			res_evs_confchg_callback = (struct res_evs_confchg_callback *)&dispatch_data;
+			res_evs_confchg_callback = (struct res_evs_confchg_callback *)dispatch_data;
 			callbacks.evs_confchg_fn (
+				handle,
 				res_evs_confchg_callback->member_list,
 				res_evs_confchg_callback->member_list_entries,
 				res_evs_confchg_callback->left_list,
 				res_evs_confchg_callback->left_list_entries,
 				res_evs_confchg_callback->joined_list,
-				res_evs_confchg_callback->joined_list_entries);
+				res_evs_confchg_callback->joined_list_entries,
+				NULL);
 			break;
 
 		default:
+			coroipcc_dispatch_put (evs_inst->handle);
 			error = CS_ERR_LIBRARY;
-			goto error_nounlock;
+			goto error_put;
 			break;
 		}
+		coroipcc_dispatch_put (evs_inst->handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -307,16 +327,15 @@ evs_error_t evs_dispatch (
 		}
 	} while (cont);
 
-error_unlock:
-	saHandleInstancePut (&evs_handle_t_db, handle);
-error_nounlock:
+error_put:
+	hdb_handle_put (&evs_handle_t_db, handle);
 	return (error);
 }
 
 evs_error_t evs_join (
     evs_handle_t handle,
-    struct evs_group *groups,
-	int group_entries)
+    const struct evs_group *groups,
+    size_t group_entries)
 {
 	evs_error_t error;
 	struct evs_inst *evs_inst;
@@ -324,28 +343,24 @@ evs_error_t evs_join (
 	struct req_lib_evs_join req_lib_evs_join;
 	struct res_lib_evs_join res_lib_evs_join;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != EVS_OK) {
 		return (error);
 	}
 
-	req_lib_evs_join.header.size = sizeof (struct req_lib_evs_join) + 
+	req_lib_evs_join.header.size = sizeof (struct req_lib_evs_join) +
 		(group_entries * sizeof (struct evs_group));
 	req_lib_evs_join.header.id = MESSAGE_REQ_EVS_JOIN;
 	req_lib_evs_join.group_entries = group_entries;
 
-	iov[0].iov_base = &req_lib_evs_join;
+	iov[0].iov_base = (void *)&req_lib_evs_join;
 	iov[0].iov_len = sizeof (struct req_lib_evs_join);
-	iov[1].iov_base = groups;
+	iov[1].iov_base = (void*) groups; /* cast away const */
 	iov[1].iov_len = (group_entries * sizeof (struct evs_group));
-	
-	pthread_mutex_lock (&evs_inst->response_mutex);
 
-	error = coroipcc_msg_send_reply_receive (evs_inst->ipc_ctx, iov, 2,
+	error = coroipcc_msg_send_reply_receive (evs_inst->handle, iov, 2,
 		&res_lib_evs_join, sizeof (struct res_lib_evs_join));
 
-	pthread_mutex_unlock (&evs_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -353,15 +368,15 @@ evs_error_t evs_join (
 	error = res_lib_evs_join.header.error;
 
 error_exit:
-	saHandleInstancePut (&evs_handle_t_db, handle);
+	hdb_handle_put (&evs_handle_t_db, handle);
 
 	return (error);
 }
 
 evs_error_t evs_leave (
     evs_handle_t handle,
-    struct evs_group *groups,
-	int group_entries)
+    const struct evs_group *groups,
+    size_t group_entries)
 {
 	evs_error_t error;
 	struct evs_inst *evs_inst;
@@ -369,28 +384,24 @@ evs_error_t evs_leave (
 	struct req_lib_evs_leave req_lib_evs_leave;
 	struct res_lib_evs_leave res_lib_evs_leave;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	req_lib_evs_leave.header.size = sizeof (struct req_lib_evs_leave) + 
+	req_lib_evs_leave.header.size = sizeof (struct req_lib_evs_leave) +
 		(group_entries * sizeof (struct evs_group));
 	req_lib_evs_leave.header.id = MESSAGE_REQ_EVS_LEAVE;
 	req_lib_evs_leave.group_entries = group_entries;
 
-	iov[0].iov_base = &req_lib_evs_leave;
+	iov[0].iov_base = (void *)&req_lib_evs_leave;
 	iov[0].iov_len = sizeof (struct req_lib_evs_leave);
-	iov[1].iov_base = groups;
+	iov[1].iov_base = (void *) groups; /* cast away const */
 	iov[1].iov_len = (group_entries * sizeof (struct evs_group));
-	
-	pthread_mutex_lock (&evs_inst->response_mutex);
 
-	error = coroipcc_msg_send_reply_receive (evs_inst->ipc_ctx, iov, 2,
+	error = coroipcc_msg_send_reply_receive (evs_inst->handle, iov, 2,
 		&res_lib_evs_leave, sizeof (struct res_lib_evs_leave));
 
-	pthread_mutex_unlock (&evs_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -398,7 +409,7 @@ evs_error_t evs_leave (
 	error = res_lib_evs_leave.header.error;
 
 error_exit:
-	saHandleInstancePut (&evs_handle_t_db, handle);
+	hdb_handle_put (&evs_handle_t_db, handle);
 
 	return (error);
 }
@@ -406,8 +417,8 @@ error_exit:
 evs_error_t evs_mcast_joined (
 	evs_handle_t handle,
 	evs_guarantee_t guarantee,
-	struct iovec *iovec,
-	int iov_len)
+	const struct iovec *iovec,
+	unsigned int iov_len)
 {
 	int i;
 	evs_error_t error;
@@ -415,9 +426,9 @@ evs_error_t evs_mcast_joined (
 	struct iovec iov[64];
 	struct req_lib_evs_mcast_joined req_lib_evs_mcast_joined;
 	struct res_lib_evs_mcast_joined res_lib_evs_mcast_joined;
-	int msg_len = 0;
+	size_t msg_len = 0;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -433,19 +444,15 @@ evs_error_t evs_mcast_joined (
 	req_lib_evs_mcast_joined.guarantee = guarantee;
 	req_lib_evs_mcast_joined.msg_len = msg_len;
 
-	iov[0].iov_base = &req_lib_evs_mcast_joined;
+	iov[0].iov_base = (void *)&req_lib_evs_mcast_joined;
 	iov[0].iov_len = sizeof (struct req_lib_evs_mcast_joined);
 	memcpy (&iov[1], iovec, iov_len * sizeof (struct iovec));
-	
-	pthread_mutex_lock (&evs_inst->response_mutex);
 
-	error = coroipcc_msg_send_reply_receive (evs_inst->ipc_ctx, iov,
+	error = coroipcc_msg_send_reply_receive (evs_inst->handle, iov,
 		iov_len + 1,
 		&res_lib_evs_mcast_joined,
 		sizeof (struct res_lib_evs_mcast_joined));
 
-	pthread_mutex_unlock (&evs_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -453,7 +460,7 @@ evs_error_t evs_mcast_joined (
 	error = res_lib_evs_mcast_joined.header.error;
 
 error_exit:
-	saHandleInstancePut (&evs_handle_t_db, handle);
+	hdb_handle_put (&evs_handle_t_db, handle);
 
 	return (error);
 }
@@ -461,47 +468,44 @@ error_exit:
 evs_error_t evs_mcast_groups (
 	evs_handle_t handle,
 	evs_guarantee_t guarantee,
-	struct evs_group *groups,
-	int group_entries,
-	struct iovec *iovec,
-	int iov_len)
+	const struct evs_group *groups,
+	size_t group_entries,
+	const struct iovec *iovec,
+	unsigned int iov_len)
 {
 	int i;
 	evs_error_t error;
 	struct evs_inst *evs_inst;
-	struct iovec iov[64];
+	struct iovec iov[64]; /* FIXME: what if iov_len > 62 ?  use malloc */
 	struct req_lib_evs_mcast_groups req_lib_evs_mcast_groups;
 	struct res_lib_evs_mcast_groups res_lib_evs_mcast_groups;
-	int msg_len = 0;
+	size_t msg_len = 0;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 	for (i = 0; i < iov_len; i++) {
 		msg_len += iovec[i].iov_len;
 	}
-	req_lib_evs_mcast_groups.header.size = sizeof (struct req_lib_evs_mcast_groups) + 
+	req_lib_evs_mcast_groups.header.size = sizeof (struct req_lib_evs_mcast_groups) +
 		(group_entries * sizeof (struct evs_group)) + msg_len;
 	req_lib_evs_mcast_groups.header.id = MESSAGE_REQ_EVS_MCAST_GROUPS;
 	req_lib_evs_mcast_groups.guarantee = guarantee;
 	req_lib_evs_mcast_groups.msg_len = msg_len;
 	req_lib_evs_mcast_groups.group_entries = group_entries;
 
-	iov[0].iov_base = &req_lib_evs_mcast_groups;
+	iov[0].iov_base = (void *)&req_lib_evs_mcast_groups;
 	iov[0].iov_len = sizeof (struct req_lib_evs_mcast_groups);
-	iov[1].iov_base = groups;
+	iov[1].iov_base = (void *) groups; /* cast away const */
 	iov[1].iov_len = (group_entries * sizeof (struct evs_group));
 	memcpy (&iov[2], iovec, iov_len * sizeof (struct iovec));
-	
-	pthread_mutex_lock (&evs_inst->response_mutex);
 
-	error = coroipcc_msg_send_reply_receive (evs_inst->ipc_ctx, iov,
+	error = coroipcc_msg_send_reply_receive (evs_inst->handle, iov,
 		iov_len + 2,
 		&res_lib_evs_mcast_groups,
 		sizeof (struct res_lib_evs_mcast_groups));
 
-	pthread_mutex_unlock (&evs_inst->response_mutex);
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -509,7 +513,7 @@ evs_error_t evs_mcast_groups (
 	error = res_lib_evs_mcast_groups.header.error;
 
 error_exit:
-	saHandleInstancePut (&evs_handle_t_db, handle);
+	hdb_handle_put (&evs_handle_t_db, handle);
 
 	return (error);
 }
@@ -518,7 +522,7 @@ evs_error_t evs_membership_get (
 	evs_handle_t handle,
 	unsigned int *local_nodeid,
 	unsigned int *member_list,
-	unsigned int *member_list_entries)
+	size_t *member_list_entries)
 {
 	evs_error_t error;
 	struct evs_inst *evs_inst;
@@ -526,7 +530,7 @@ evs_error_t evs_membership_get (
 	struct req_lib_evs_membership_get req_lib_evs_membership_get;
 	struct res_lib_evs_membership_get res_lib_evs_membership_get;
 
-	error = saHandleInstanceGet (&evs_handle_t_db, handle, (void *)&evs_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&evs_handle_t_db, handle, (void *)&evs_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -534,19 +538,15 @@ evs_error_t evs_membership_get (
 	req_lib_evs_membership_get.header.size = sizeof (struct req_lib_evs_membership_get);
 	req_lib_evs_membership_get.header.id = MESSAGE_REQ_EVS_MEMBERSHIP_GET;
 
-	iov.iov_base = &req_lib_evs_membership_get;
+	iov.iov_base = (void *)&req_lib_evs_membership_get;
 	iov.iov_len = sizeof (struct req_lib_evs_membership_get);
 
-	pthread_mutex_lock (&evs_inst->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (evs_inst->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (evs_inst->handle,
 		&iov,
 		1,
 		&res_lib_evs_membership_get,
 		sizeof (struct res_lib_evs_membership_get));
 
-	pthread_mutex_unlock (&evs_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -559,15 +559,15 @@ evs_error_t evs_membership_get (
 	if (local_nodeid) {
 		*local_nodeid = res_lib_evs_membership_get.local_nodeid;
  	}
-	*member_list_entries = *member_list_entries < res_lib_evs_membership_get.member_list_entries ?
-		*member_list_entries : res_lib_evs_membership_get.member_list_entries;
+	*member_list_entries = MIN (*member_list_entries,
+				    res_lib_evs_membership_get.member_list_entries);
 	if (member_list) {
-		memcpy (member_list, &res_lib_evs_membership_get.member_list, 
+		memcpy (member_list, &res_lib_evs_membership_get.member_list,
 			*member_list_entries * sizeof (struct in_addr));
 	}
 
 error_exit:
-	saHandleInstancePut (&evs_handle_t_db, handle);
+	hdb_handle_put (&evs_handle_t_db, handle);
 
 	return (error);
 }
