@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008 Red Hat, Inc.
+ * Copyright (c) 2008, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -40,45 +40,29 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <pthread.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
 
 #include <corosync/corotypes.h>
-#include <corosync/mar_gen.h>
-#include <corosync/ipc_gen.h>
+#include <corosync/coroipc_types.h>
 #include <corosync/coroipcc.h>
-#include "corosync/quorum.h"
-#include "corosync/ipc_quorum.h"
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
+
+#include <corosync/quorum.h>
+#include <corosync/ipc_quorum.h>
+
+#include "util.h"
 
 struct quorum_inst {
-	void *ipc_ctx;
+	hdb_handle_t handle;
 	int finalize;
 	const void *context;
 	quorum_callbacks_t callbacks;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 };
 
-static void quorum_instance_destructor (void *instance);
-
-static struct saHandleDatabase quorum_handle_t_db = {
-	.handleCount		        = 0,
-	.handles			= 0,
-	.mutex				= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= quorum_instance_destructor
-};
-
-/*
- * Clean up function for a quorum instance (quorum_initialize) handle
- */
-static void quorum_instance_destructor (void *instance)
-{
-	struct quorum_inst *quorum_inst = instance;
-
-	pthread_mutex_destroy (&quorum_inst->response_mutex);
-}
+DECLARE_HDB_DATABASE(quorum_handle_t_db,NULL);
 
 cs_error_t quorum_initialize (
 	quorum_handle_t *handle,
@@ -87,36 +71,40 @@ cs_error_t quorum_initialize (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 
-	error = saHandleCreate (&quorum_handle_t_db, sizeof (struct quorum_inst), handle);
+	error = hdb_error_to_cs(hdb_handle_create (&quorum_handle_t_db, sizeof (struct quorum_inst), handle));
 	if (error != CS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, *handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, *handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, QUORUM_SERVICE, &quorum_inst->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		QUORUM_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		&quorum_inst->handle);
 	if (error != CS_OK) {
 		goto error_put_destroy;
 	}
 
-	pthread_mutex_init (&quorum_inst->response_mutex, NULL);
-	pthread_mutex_init (&quorum_inst->dispatch_mutex, NULL);
 	if (callbacks)
 		memcpy(&quorum_inst->callbacks, callbacks, sizeof (callbacks));
 	else
 		memset(&quorum_inst->callbacks, 0, sizeof (callbacks));
 
-	(void)saHandleInstancePut (&quorum_handle_t_db, *handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, *handle);
 
 	return (CS_OK);
 
 error_put_destroy:
-	(void)saHandleInstancePut (&quorum_handle_t_db, *handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, *handle);
 error_destroy:
-	(void)saHandleDestroy (&quorum_handle_t_db, *handle);
+	(void)hdb_handle_destroy (&quorum_handle_t_db, *handle);
 error_no_destroy:
 	return (error);
 }
@@ -127,31 +115,26 @@ cs_error_t quorum_finalize (
 	struct quorum_inst *quorum_inst;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&quorum_inst->response_mutex);
-
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (quorum_inst->finalize) {
-		pthread_mutex_unlock (&quorum_inst->response_mutex);
-		(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+		(void)hdb_handle_put (&quorum_handle_t_db, handle);
 		return (CS_ERR_BAD_HANDLE);
 	}
 
 	quorum_inst->finalize = 1;
 
-	coroipcc_service_disconnect (quorum_inst->ipc_ctx);
+	coroipcc_service_disconnect (quorum_inst->handle);
 
-	pthread_mutex_unlock (&quorum_inst->response_mutex);
+	(void)hdb_handle_destroy (&quorum_handle_t_db, handle);
 
-	(void)saHandleDestroy (&quorum_handle_t_db, handle);
-
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -163,16 +146,14 @@ cs_error_t quorum_getquorate (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 	struct iovec iov;
-	mar_req_header_t req;
+	coroipc_request_header_t req;
 	struct res_lib_quorum_getquorate res_lib_quorum_getquorate;
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&quorum_inst->response_mutex);
-
 	req.size = sizeof (req);
 	req.id = MESSAGE_REQ_QUORUM_GETQUORATE;
 
@@ -180,14 +161,12 @@ cs_error_t quorum_getquorate (
 	iov.iov_len = sizeof (req);
 
        error = coroipcc_msg_send_reply_receive (
-		quorum_inst->ipc_ctx,
+		quorum_inst->handle,
 		&iov,
 		1,
 		&res_lib_quorum_getquorate,
 		sizeof (struct res_lib_quorum_getquorate));
 
-	pthread_mutex_unlock (&quorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -197,7 +176,7 @@ cs_error_t quorum_getquorate (
 	*quorate = res_lib_quorum_getquorate.quorate;
 
 error_exit:
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -209,16 +188,16 @@ cs_error_t quorum_fd_get (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	*fd = coroipcc_fd_get (quorum_inst->ipc_ctx);
+	error = coroipcc_fd_get (quorum_inst->handle, fd);
 
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
-	return (CS_OK);
+	return (error);
 }
 
 
@@ -229,14 +208,14 @@ cs_error_t quorum_context_get (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	*context = quorum_inst->context;
 
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -248,14 +227,14 @@ cs_error_t quorum_context_set (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	quorum_inst->context = context;
 
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -269,15 +248,13 @@ cs_error_t quorum_trackstart (
 	struct quorum_inst *quorum_inst;
 	struct iovec iov;
 	struct req_lib_quorum_trackstart req_lib_quorum_trackstart;
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&quorum_inst->response_mutex);
-
 	req_lib_quorum_trackstart.header.size = sizeof (struct req_lib_quorum_trackstart);
 	req_lib_quorum_trackstart.header.id = MESSAGE_REQ_QUORUM_TRACKSTART;
 	req_lib_quorum_trackstart.track_flags = flags;
@@ -286,14 +263,12 @@ cs_error_t quorum_trackstart (
 	iov.iov_len = sizeof (struct req_lib_quorum_trackstart);
 
        error = coroipcc_msg_send_reply_receive (
-		quorum_inst->ipc_ctx,
+		quorum_inst->handle,
                 &iov,
                 1,
                 &res,
                 sizeof (res));
 
-	pthread_mutex_unlock (&quorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -301,7 +276,7 @@ cs_error_t quorum_trackstart (
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -312,16 +287,14 @@ cs_error_t quorum_trackstop (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 	struct iovec iov;
-	mar_req_header_t req;
-	mar_res_header_t res;
+	coroipc_request_header_t req;
+	coroipc_response_header_t res;
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle, (void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&quorum_inst->response_mutex);
-
 	req.size = sizeof (req);
 	req.id = MESSAGE_REQ_QUORUM_TRACKSTOP;
 
@@ -329,14 +302,12 @@ cs_error_t quorum_trackstop (
 	iov.iov_len = sizeof (req);
 
        error = coroipcc_msg_send_reply_receive (
-		quorum_inst->ipc_ctx,
+		quorum_inst->handle,
                 &iov,
                 1,
                 &res,
                 sizeof (res));
 
-	pthread_mutex_unlock (&quorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -344,16 +315,11 @@ cs_error_t quorum_trackstop (
 	error = res.error;
 
 error_exit:
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
 	return (error);
 }
 
-struct quorum_res_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char data[512000];
-};
-
 cs_error_t quorum_dispatch (
 	quorum_handle_t handle,
 	cs_dispatch_flags_t dispatch_types)
@@ -361,10 +327,9 @@ cs_error_t quorum_dispatch (
 	int timeout = -1;
 	cs_error_t error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct quorum_inst *quorum_inst;
 	quorum_callbacks_t callbacks;
-	struct quorum_res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 	struct res_lib_quorum_notification *res_lib_quorum_notification;
 
 	if (dispatch_types != CS_DISPATCH_ONE &&
@@ -374,8 +339,8 @@ cs_error_t quorum_dispatch (
 		return (CS_ERR_INVALID_PARAM);
 	}
 
-	error = saHandleInstanceGet (&quorum_handle_t_db, handle,
-		(void *)&quorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle,
+		(void *)&quorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -389,26 +354,16 @@ cs_error_t quorum_dispatch (
 	}
 
 	do {
-		pthread_mutex_lock (&quorum_inst->dispatch_mutex);
-
-		dispatch_avail = coroipcc_dispatch_recv (quorum_inst->ipc_ctx,
-			(void *)&dispatch_data, timeout);
-
-		/*
-		 * Handle has been finalized in another thread
-		 */
-		if (quorum_inst->finalize == 1) {
+		error = coroipcc_dispatch_get (
+			quorum_inst->handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
 			error = CS_OK;
-			goto error_unlock;
+			goto error_put;
 		}
-
-		if (dispatch_avail == 0 && dispatch_types == CS_DISPATCH_ALL) {
-			pthread_mutex_unlock (&quorum_inst->dispatch_mutex);
-			break; /* exit do while cont is 1 loop */
-		} else
-		if (dispatch_avail == 0) {
-			pthread_mutex_unlock (&quorum_inst->dispatch_mutex);
-			continue; /* next poll */
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
 		/*
@@ -417,18 +372,16 @@ cs_error_t quorum_dispatch (
 		 * operate at the same time that quorum_finalize has been called in another thread.
 		 */
 		memcpy (&callbacks, &quorum_inst->callbacks, sizeof (quorum_callbacks_t));
-		pthread_mutex_unlock (&quorum_inst->dispatch_mutex);
-
 		/*
 		 * Dispatch incoming message
 		 */
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 
 		case MESSAGE_RES_QUORUM_NOTIFICATION:
 			if (callbacks.quorum_notify_fn == NULL) {
 				continue;
 			}
-			res_lib_quorum_notification = (struct res_lib_quorum_notification *)&dispatch_data;
+			res_lib_quorum_notification = (struct res_lib_quorum_notification *)dispatch_data;
 
 			callbacks.quorum_notify_fn ( handle,
 				res_lib_quorum_notification->quorate,
@@ -438,10 +391,12 @@ cs_error_t quorum_dispatch (
 			break;
 
 		default:
+			coroipcc_dispatch_put (quorum_inst->handle);
 			error = CS_ERR_LIBRARY;
 			goto error_put;
 			break;
 		}
+		coroipcc_dispatch_put (quorum_inst->handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -459,10 +414,7 @@ cs_error_t quorum_dispatch (
 
 	goto error_put;
 
-error_unlock:
-	pthread_mutex_unlock (&quorum_inst->dispatch_mutex);
-
 error_put:
-	(void)saHandleInstancePut (&quorum_handle_t_db, handle);
+	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 	return (error);
 }
