@@ -41,44 +41,30 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <pthread.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
 
-#include <corosync/mar_gen.h>
-#include <corosync/ipc_gen.h>
+
+#include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
 #include <corosync/coroipcc.h>
-#include "corosync/votequorum.h"
-#include "corosync/ipc_votequorum.h"
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
+
+#include <corosync/votequorum.h>
+#include <corosync/ipc_votequorum.h>
+
+#include "util.h"
 
 struct votequorum_inst {
-	void *ipc_ctx;
+	hdb_handle_t handle;
 	int finalize;
 	void *context;
 	votequorum_callbacks_t callbacks;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 };
 
-static void votequorum_instance_destructor (void *instance);
-
-static struct saHandleDatabase votequorum_handle_t_db = {
-	.handleCount		        = 0,
-	.handles			= 0,
-	.mutex				= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= votequorum_instance_destructor
-};
-
-/*
- * Clean up function for a quorum instance (votequorum_initialize) handle
- */
-static void votequorum_instance_destructor (void *instance)
-{
-	struct votequorum_inst *votequorum_inst = instance;
-
-	pthread_mutex_destroy (&votequorum_inst->response_mutex);
-}
+DECLARE_HDB_DATABASE(votequorum_handle_t_db,NULL);
 
 cs_error_t votequorum_initialize (
 	votequorum_handle_t *handle,
@@ -87,36 +73,40 @@ cs_error_t votequorum_initialize (
 	cs_error_t error;
 	struct votequorum_inst *votequorum_inst;
 
-	error = saHandleCreate (&votequorum_handle_t_db, sizeof (struct votequorum_inst), handle);
+	error = hdb_error_to_cs(hdb_handle_create (&votequorum_handle_t_db, sizeof (struct votequorum_inst), handle));
 	if (error != CS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, *handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, *handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, VOTEQUORUM_SERVICE, &votequorum_inst->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		VOTEQUORUM_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		 &votequorum_inst->handle);
 	if (error != CS_OK) {
 		goto error_put_destroy;
 	}
 
-	pthread_mutex_init (&votequorum_inst->response_mutex, NULL);
-	pthread_mutex_init (&votequorum_inst->dispatch_mutex, NULL);
 	if (callbacks)
 		memcpy(&votequorum_inst->callbacks, callbacks, sizeof (*callbacks));
 	else
 		memset(&votequorum_inst->callbacks, 0, sizeof (*callbacks));
 
-	saHandleInstancePut (&votequorum_handle_t_db, *handle);
+	hdb_handle_put (&votequorum_handle_t_db, *handle);
 
 	return (CS_OK);
 
 error_put_destroy:
-	saHandleInstancePut (&votequorum_handle_t_db, *handle);
+	hdb_handle_put (&votequorum_handle_t_db, *handle);
 error_destroy:
-	saHandleDestroy (&votequorum_handle_t_db, *handle);
+	hdb_handle_destroy (&votequorum_handle_t_db, *handle);
 error_no_destroy:
 	return (error);
 }
@@ -127,31 +117,26 @@ cs_error_t votequorum_finalize (
 	struct votequorum_inst *votequorum_inst;
 	cs_error_t error;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (votequorum_inst->finalize) {
-		pthread_mutex_unlock (&votequorum_inst->response_mutex);
-		saHandleInstancePut (&votequorum_handle_t_db, handle);
+		hdb_handle_put (&votequorum_handle_t_db, handle);
 		return (CS_ERR_BAD_HANDLE);
 	}
 
 	votequorum_inst->finalize = 1;
 
-	coroipcc_service_disconnect (votequorum_inst->ipc_ctx);
+	coroipcc_service_disconnect (votequorum_inst->handle);
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
+	hdb_handle_destroy (&votequorum_handle_t_db, handle);
 
-	saHandleDestroy (&votequorum_handle_t_db, handle);
-
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -168,7 +153,7 @@ cs_error_t votequorum_getinfo (
 	struct req_lib_votequorum_getinfo req_lib_votequorum_getinfo;
 	struct res_lib_votequorum_getinfo res_lib_votequorum_getinfo;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -180,17 +165,13 @@ cs_error_t votequorum_getinfo (
 	iov.iov_base = (char *)&req_lib_votequorum_getinfo;
 	iov.iov_len = sizeof (struct req_lib_votequorum_getinfo);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_getinfo,
 		sizeof (struct res_lib_votequorum_getinfo));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -206,7 +187,7 @@ cs_error_t votequorum_getinfo (
 	info->flags = res_lib_votequorum_getinfo.flags;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -221,7 +202,7 @@ cs_error_t votequorum_setexpected (
 	struct req_lib_votequorum_setexpected req_lib_votequorum_setexpected;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -234,17 +215,13 @@ cs_error_t votequorum_setexpected (
 	iov.iov_base = (char *)&req_lib_votequorum_setexpected;
 	iov.iov_len = sizeof (struct req_lib_votequorum_setexpected);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -252,7 +229,7 @@ cs_error_t votequorum_setexpected (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -268,7 +245,7 @@ cs_error_t votequorum_setvotes (
 	struct req_lib_votequorum_setvotes req_lib_votequorum_setvotes;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -281,17 +258,13 @@ cs_error_t votequorum_setvotes (
 	iov.iov_base = (char *)&req_lib_votequorum_setvotes;
 	iov.iov_len = sizeof (struct req_lib_votequorum_setvotes);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -299,14 +272,14 @@ cs_error_t votequorum_setvotes (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
 
 cs_error_t votequorum_qdisk_register (
 	votequorum_handle_t handle,
-	char *name,
+	const char *name,
 	unsigned int votes)
 {
 	cs_error_t error;
@@ -318,7 +291,7 @@ cs_error_t votequorum_qdisk_register (
 	if (strlen(name) > VOTEQUORUM_MAX_QDISK_NAME_LEN)
 		return CS_ERR_INVALID_PARAM;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -332,17 +305,13 @@ cs_error_t votequorum_qdisk_register (
 	iov.iov_base = (char *)&req_lib_votequorum_qdisk_register;
 	iov.iov_len = sizeof (struct req_lib_votequorum_qdisk_register);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -350,7 +319,7 @@ cs_error_t votequorum_qdisk_register (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -365,7 +334,7 @@ cs_error_t votequorum_qdisk_poll (
 	struct req_lib_votequorum_qdisk_poll req_lib_votequorum_qdisk_poll;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -378,17 +347,13 @@ cs_error_t votequorum_qdisk_poll (
 	iov.iov_base = (char *)&req_lib_votequorum_qdisk_poll;
 	iov.iov_len = sizeof (struct req_lib_votequorum_qdisk_poll);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -396,7 +361,7 @@ cs_error_t votequorum_qdisk_poll (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -410,13 +375,11 @@ cs_error_t votequorum_qdisk_unregister (
 	struct req_lib_votequorum_general req_lib_votequorum_general;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
 	req_lib_votequorum_general.header.size = sizeof (struct req_lib_votequorum_general);
 	req_lib_votequorum_general.header.id = MESSAGE_REQ_VOTEQUORUM_QDISK_UNREGISTER;
 
@@ -424,14 +387,12 @@ cs_error_t votequorum_qdisk_unregister (
 	iov.iov_len = sizeof (struct req_lib_votequorum_general);
 
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -439,7 +400,7 @@ cs_error_t votequorum_qdisk_unregister (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -456,7 +417,7 @@ cs_error_t votequorum_qdisk_getinfo (
 	struct req_lib_votequorum_general req_lib_votequorum_general;
 	struct res_lib_votequorum_qdisk_getinfo res_lib_votequorum_qdisk_getinfo;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -468,17 +429,13 @@ cs_error_t votequorum_qdisk_getinfo (
 	iov.iov_base = (char *)&req_lib_votequorum_general;
 	iov.iov_len = sizeof (struct req_lib_votequorum_general);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_qdisk_getinfo,
 		sizeof (struct res_lib_votequorum_qdisk_getinfo));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -491,7 +448,7 @@ cs_error_t votequorum_qdisk_getinfo (
 
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -505,7 +462,7 @@ cs_error_t votequorum_setstate (
 	struct req_lib_votequorum_general req_lib_votequorum_general;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -516,17 +473,13 @@ cs_error_t votequorum_setstate (
 	iov.iov_base = (char *)&req_lib_votequorum_general;
 	iov.iov_len = sizeof (struct req_lib_votequorum_general);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -534,7 +487,7 @@ cs_error_t votequorum_setstate (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -548,7 +501,7 @@ cs_error_t votequorum_leaving (
 	struct req_lib_votequorum_general req_lib_votequorum_general;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -560,17 +513,13 @@ cs_error_t votequorum_leaving (
 	iov.iov_base = (char *)&req_lib_votequorum_general;
 	iov.iov_len = sizeof (struct req_lib_votequorum_general);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -578,7 +527,7 @@ cs_error_t votequorum_leaving (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -594,7 +543,7 @@ cs_error_t votequorum_trackstart (
 	struct req_lib_votequorum_trackstart req_lib_votequorum_trackstart;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -607,17 +556,13 @@ cs_error_t votequorum_trackstart (
 	iov.iov_base = (char *)&req_lib_votequorum_trackstart;
 	iov.iov_len = sizeof (struct req_lib_votequorum_trackstart);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -625,7 +570,7 @@ cs_error_t votequorum_trackstart (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -639,7 +584,7 @@ cs_error_t votequorum_trackstop (
 	struct req_lib_votequorum_general req_lib_votequorum_general;
 	struct res_lib_votequorum_status res_lib_votequorum_status;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -650,17 +595,13 @@ cs_error_t votequorum_trackstop (
 	iov.iov_base = (char *)&req_lib_votequorum_general;
 	iov.iov_len = sizeof (struct req_lib_votequorum_general);
 
-	pthread_mutex_lock (&votequorum_inst->response_mutex);
-
         error = coroipcc_msg_send_reply_receive (
-		votequorum_inst->ipc_ctx,
+		votequorum_inst->handle,
 		&iov,
 		1,
                 &res_lib_votequorum_status,
 		sizeof (struct res_lib_votequorum_status));
 
-	pthread_mutex_unlock (&votequorum_inst->response_mutex);
-
 	if (error != CS_OK) {
 		goto error_exit;
 	}
@@ -668,7 +609,7 @@ cs_error_t votequorum_trackstop (
 	error = res_lib_votequorum_status.header.error;
 
 error_exit:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (error);
 }
@@ -681,14 +622,14 @@ cs_error_t votequorum_context_get (
 	cs_error_t error;
 	struct votequorum_inst *votequorum_inst;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	*context = votequorum_inst->context;
 
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -700,14 +641,14 @@ cs_error_t votequorum_context_set (
 	cs_error_t error;
 	struct votequorum_inst *votequorum_inst;
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
 	votequorum_inst->context = context;
 
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 
 	return (CS_OK);
 }
@@ -720,24 +661,18 @@ cs_error_t votequorum_fd_get (
 	cs_error_t error;
         struct votequorum_inst *votequorum_inst;
 
-        error = saHandleInstanceGet (&votequorum_handle_t_db, handle, (void *)&votequorum_inst);
+        error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle, (void *)&votequorum_inst));
         if (error != CS_OK) {
                 return (error);
         }
 
-	*fd = coroipcc_fd_get (votequorum_inst->ipc_ctx);
+	error = coroipcc_fd_get (votequorum_inst->handle, fd);
 
-	(void)saHandleInstancePut (&votequorum_handle_t_db, handle);
+	(void)hdb_handle_put (&votequorum_handle_t_db, handle);
 
-	return (CS_OK);
+	return (error);
 }
 
-
-struct res_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char data[512000];
-};
-
 cs_error_t votequorum_dispatch (
 	votequorum_handle_t handle,
 	cs_dispatch_flags_t dispatch_types)
@@ -745,10 +680,9 @@ cs_error_t votequorum_dispatch (
 	int timeout = -1;
 	cs_error_t error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct votequorum_inst *votequorum_inst;
 	votequorum_callbacks_t callbacks;
-	struct res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 	struct res_lib_votequorum_notification *res_lib_votequorum_notification;
 	struct res_lib_votequorum_expectedvotes_notification *res_lib_votequorum_expectedvotes_notification;
 
@@ -759,8 +693,8 @@ cs_error_t votequorum_dispatch (
 		return (CS_ERR_INVALID_PARAM);
 	}
 
-	error = saHandleInstanceGet (&votequorum_handle_t_db, handle,
-		(void *)&votequorum_inst);
+	error = hdb_error_to_cs(hdb_handle_get (&votequorum_handle_t_db, handle,
+		(void *)&votequorum_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
@@ -774,28 +708,24 @@ cs_error_t votequorum_dispatch (
 	}
 
 	do {
-		pthread_mutex_lock (&votequorum_inst->dispatch_mutex);
-
-		dispatch_avail = coroipcc_dispatch_recv (
-			votequorum_inst->ipc_ctx,
-			(void *)&dispatch_data, timeout);
-
-
-		/*
-		 * Handle has been finalized in another thread
-		 */
-		if (votequorum_inst->finalize == 1) {
+		error = coroipcc_dispatch_get (
+			votequorum_inst->handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
 			error = CS_OK;
-			goto error_unlock;
+			goto error_put;
+		}
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
-		if (dispatch_avail == 0 && dispatch_types == CS_DISPATCH_ALL) {
-			pthread_mutex_unlock (&votequorum_inst->dispatch_mutex);
-			break; /* exit do while cont is 1 loop */
-		} else
-		if (dispatch_avail == 0) {
-			pthread_mutex_unlock (&votequorum_inst->dispatch_mutex);
-			continue; /* next poll */
+		if (dispatch_data == NULL) {
+			if (dispatch_types == CPG_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
+			} else {
+				continue; /* next poll */
+			}
 		}
 
 		/*
@@ -804,18 +734,17 @@ cs_error_t votequorum_dispatch (
 		 * operate at the same time that votequorum_finalize has been called in another thread.
 		 */
 		memcpy (&callbacks, &votequorum_inst->callbacks, sizeof (votequorum_callbacks_t));
-		pthread_mutex_unlock (&votequorum_inst->dispatch_mutex);
 
 		/*
 		 * Dispatch incoming message
 		 */
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 
 		case MESSAGE_RES_VOTEQUORUM_NOTIFICATION:
 			if (callbacks.votequorum_notify_fn == NULL) {
 				continue;
 			}
-			res_lib_votequorum_notification = (struct res_lib_votequorum_notification *)&dispatch_data;
+			res_lib_votequorum_notification = (struct res_lib_votequorum_notification *)dispatch_data;
 
 			callbacks.votequorum_notify_fn ( handle,
 							 res_lib_votequorum_notification->context,
@@ -829,7 +758,7 @@ cs_error_t votequorum_dispatch (
 			if (callbacks.votequorum_expectedvotes_notify_fn == NULL) {
 				continue;
 			}
-			res_lib_votequorum_expectedvotes_notification = (struct res_lib_votequorum_expectedvotes_notification *)&dispatch_data;
+			res_lib_votequorum_expectedvotes_notification = (struct res_lib_votequorum_expectedvotes_notification *)dispatch_data;
 
 			callbacks.votequorum_expectedvotes_notify_fn ( handle,
 								       res_lib_votequorum_expectedvotes_notification->context,
@@ -837,10 +766,12 @@ cs_error_t votequorum_dispatch (
 			break;
 
 		default:
+			coroipcc_dispatch_put (votequorum_inst->handle);
 			error = CS_ERR_LIBRARY;
 			goto error_put;
 			break;
 		}
+		coroipcc_dispatch_put (votequorum_inst->handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -858,10 +789,7 @@ cs_error_t votequorum_dispatch (
 
 	goto error_put;
 
-error_unlock:
-	pthread_mutex_unlock (&votequorum_inst->dispatch_mutex);
-
 error_put:
-	saHandleInstancePut (&votequorum_handle_t_db, handle);
+	hdb_handle_put (&votequorum_handle_t_db, handle);
 	return (error);
 }
