@@ -42,17 +42,26 @@
 #include <unistd.h>
 
 #include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/corodefs.h>
 #include <corosync/cfg.h>
 #include <corosync/list.h>
-#include <corosync/queue.h>
 #include <corosync/mar_gen.h>
-#include <corosync/ipc_gen.h>
 #include <corosync/ipc_confdb.h>
 #include <corosync/lcr/lcr_comp.h>
 #include <corosync/engine/logsys.h>
 #include <corosync/engine/coroapi.h>
 
-LOGSYS_DECLARE_SUBSYS ("CONFDB", LOG_DEBUG);
+LOGSYS_DECLARE_SUBSYS ("CONFDB");
+
+static hdb_handle_t *
+m2h (mar_uint64_t *m)
+{
+	/* FIXME enable the following when/if we use gnulib:
+	   (it's a compile-time assertion; i.e., zero run-time cost)
+	   verify (sizeof (*m) == sizeof (hdb_handle_t)); */
+	return (void *) m;
+}
 
 static struct corosync_api_v1 *api;
 
@@ -62,47 +71,64 @@ static int confdb_exec_init_fn (
 static int confdb_lib_init_fn (void *conn);
 static int confdb_lib_exit_fn (void *conn);
 
-static void message_handler_req_lib_confdb_object_create (void *conn, void *message);
-static void message_handler_req_lib_confdb_object_destroy (void *conn, void *message);
-static void message_handler_req_lib_confdb_object_find_destroy (void *conn, void *message);
+static void message_handler_req_lib_confdb_object_create (void *conn,
+							  const void *message);
+static void message_handler_req_lib_confdb_object_destroy (void *conn,
+							   const void *message);
+static void message_handler_req_lib_confdb_object_find_destroy (void *conn,
+								const void *message);
 
-static void message_handler_req_lib_confdb_key_create (void *conn, void *message);
-static void message_handler_req_lib_confdb_key_get (void *conn, void *message);
-static void message_handler_req_lib_confdb_key_replace (void *conn, void *message);
-static void message_handler_req_lib_confdb_key_delete (void *conn, void *message);
-static void message_handler_req_lib_confdb_key_iter (void *conn, void *message);
+static void message_handler_req_lib_confdb_key_create (void *conn,
+						       const void *message);
+static void message_handler_req_lib_confdb_key_get (void *conn,
+						    const void *message);
+static void message_handler_req_lib_confdb_key_replace (void *conn,
+							const void *message);
+static void message_handler_req_lib_confdb_key_delete (void *conn,
+						       const void *message);
+static void message_handler_req_lib_confdb_key_iter (void *conn,
+						     const void *message);
 
-static void message_handler_req_lib_confdb_key_increment (void *conn, void *message);
-static void message_handler_req_lib_confdb_key_decrement (void *conn, void *message);
+static void message_handler_req_lib_confdb_key_increment (void *conn,
+							  const void *message);
+static void message_handler_req_lib_confdb_key_decrement (void *conn,
+							  const void *message);
 
-static void message_handler_req_lib_confdb_object_iter (void *conn, void *message);
-static void message_handler_req_lib_confdb_object_find (void *conn, void *message);
+static void message_handler_req_lib_confdb_object_iter (void *conn,
+							const void *message);
+static void message_handler_req_lib_confdb_object_find (void *conn,
+							const void *message);
 
-static void message_handler_req_lib_confdb_object_parent_get (void *conn, void *message);
-static void message_handler_req_lib_confdb_write (void *conn, void *message);
-static void message_handler_req_lib_confdb_reload (void *conn, void *message);
+static void message_handler_req_lib_confdb_object_parent_get (void *conn,
+							      const void *message);
+static void message_handler_req_lib_confdb_write (void *conn,
+						  const void *message);
+static void message_handler_req_lib_confdb_reload (void *conn,
+						   const void *message);
 
-static void message_handler_req_lib_confdb_track_start (void *conn, void *message);
-static void message_handler_req_lib_confdb_track_stop (void *conn, void *message);
+static void message_handler_req_lib_confdb_track_start (void *conn,
+							const void *message);
+static void message_handler_req_lib_confdb_track_stop (void *conn,
+						       const void *message);
 
 static void confdb_notify_lib_of_key_change(
 	object_change_type_t change_type,
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t object_handle,
-	void *object_name_pt, int object_name_len,
-	void *key_name_pt, int key_name_len,
-	void *key_value_pt, int key_value_len,
+	const void *object_name_pt, size_t object_name_len,
+	const void *key_name_pt, size_t key_name_len,
+	const void *key_value_pt, size_t key_value_len,
 	void *priv_data_pt);
 
 static void confdb_notify_lib_of_new_object(
 	hdb_handle_t parent_object_handle,
 	hdb_handle_t object_handle,
-	uint8_t *name_pt, int name_len,
+	const uint8_t *name_pt, size_t name_len,
 	void *priv_data_pt);
 
 static void confdb_notify_lib_of_destroyed_object(
 	hdb_handle_t parent_object_handle,
-	uint8_t *name_pt, int name_len,
+	const uint8_t *name_pt, size_t name_len,
 	void *priv_data_pt);
 
 /*
@@ -112,104 +138,70 @@ static struct corosync_lib_handler confdb_lib_engine[] =
 {
 	{ /* 0 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_object_create,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_OBJECT_CREATE,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 1 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_object_destroy,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_OBJECT_DESTROY,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 2 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_object_find,
-		.response_size				= sizeof (struct res_lib_confdb_object_find),
-		.response_id				= MESSAGE_RES_CONFDB_OBJECT_FIND,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 3 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_key_create,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_KEY_CREATE,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 4 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_key_get,
-		.response_size				= sizeof (struct res_lib_confdb_key_get),
-		.response_id				= MESSAGE_RES_CONFDB_KEY_GET,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 5 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_key_replace,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_KEY_REPLACE,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 6 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_key_delete,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_KEY_DELETE,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 7 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_object_iter,
-		.response_size				= sizeof (struct res_lib_confdb_object_iter),
-		.response_id				= MESSAGE_RES_CONFDB_OBJECT_ITER,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 8 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_object_parent_get,
-		.response_size				= sizeof (struct res_lib_confdb_object_parent_get),
-		.response_id				= MESSAGE_RES_CONFDB_OBJECT_PARENT_GET,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 9 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_key_iter,
-		.response_size				= sizeof (struct res_lib_confdb_key_iter),
-		.response_id				= MESSAGE_RES_CONFDB_KEY_ITER,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 10 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_track_start,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_TRACK_START,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 11 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_track_stop,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_TRACK_STOP,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 12 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_write,
-		.response_size				= sizeof (struct res_lib_confdb_write),
-		.response_id				= MESSAGE_RES_CONFDB_WRITE,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 13 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_reload,
-		.response_size				= sizeof (struct res_lib_confdb_reload),
-		.response_id				= MESSAGE_RES_CONFDB_RELOAD,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 14 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_object_find_destroy,
-		.response_size				= sizeof (mar_res_header_t),
-		.response_id				= MESSAGE_RES_CONFDB_OBJECT_FIND_DESTROY,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 15 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_key_increment,
-		.response_size				= sizeof (struct res_lib_confdb_key_incdec),
-		.response_id				= MESSAGE_RES_CONFDB_KEY_INCREMENT,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 16 */
 		.lib_handler_fn				= message_handler_req_lib_confdb_key_decrement,
-		.response_size				= sizeof (struct res_lib_confdb_key_incdec),
-		.response_id				= MESSAGE_RES_CONFDB_KEY_DECREMENT,
 		.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 };
@@ -218,6 +210,7 @@ static struct corosync_lib_handler confdb_lib_engine[] =
 struct corosync_service_engine confdb_service_engine = {
 	.name				        = "corosync cluster config database access v1.01",
 	.id					= CONFDB_SERVICE,
+	.priority				= 1,
 	.private_data_size			= 0,
 	.flow_control				= CS_LIB_FLOW_CONTROL_NOT_REQUIRED,
 	.allow_inquorate			= CS_LIB_ALLOW_INQUORATE,
@@ -262,7 +255,13 @@ static struct corosync_service_engine *confdb_get_service_engine_ver0 (void)
 	return (&confdb_service_engine);
 }
 
-__attribute__ ((constructor)) static void confdb_comp_register (void) {
+#ifdef COROSYNC_SOLARIS
+void corosync_lcr_component_register (void);
+
+void corosync_lcr_component_register (void) {
+#else
+__attribute__ ((constructor)) static void corosync_lcr_component_register (void) {
+#endif
         lcr_interfaces_set (&corosync_confdb_ver0[0], &confdb_service_engine_iface);
 
 	lcr_component_register (&confdb_comp_ver0);
@@ -271,19 +270,22 @@ __attribute__ ((constructor)) static void confdb_comp_register (void) {
 static int confdb_exec_init_fn (
 	struct corosync_api_v1 *corosync_api)
 {
+#ifdef COROSYNC_SOLARIS
+	logsys_subsys_init();
+#endif
 	api = corosync_api;
 	return 0;
 }
 
 static int confdb_lib_init_fn (void *conn)
 {
-	log_printf(LOG_LEVEL_DEBUG, "lib_init_fn: conn=%p\n", conn);
+	log_printf(LOGSYS_LEVEL_DEBUG, "lib_init_fn: conn=%p\n", conn);
 	return (0);
 }
 
 static int confdb_lib_exit_fn (void *conn)
 {
-	log_printf(LOG_LEVEL_DEBUG, "exit_fn for conn=%p\n", conn);
+	log_printf(LOGSYS_LEVEL_DEBUG, "exit_fn for conn=%p\n", conn);
 	/* cleanup the object trackers for this client. */
 	api->object_track_stop(confdb_notify_lib_of_key_change,
 		confdb_notify_lib_of_new_object,
@@ -293,9 +295,11 @@ static int confdb_lib_exit_fn (void *conn)
 	return (0);
 }
 
-static void message_handler_req_lib_confdb_object_create (void *conn, void *message)
+static void message_handler_req_lib_confdb_object_create (void *conn,
+							  const void *message)
 {
-	struct req_lib_confdb_object_create *req_lib_confdb_object_create = (struct req_lib_confdb_object_create *)message;
+	const struct req_lib_confdb_object_create *req_lib_confdb_object_create
+	  = message;
 	struct res_lib_confdb_object_create res_lib_confdb_object_create;
 	hdb_handle_t object_handle;
 	int ret = CS_OK;
@@ -313,10 +317,12 @@ static void message_handler_req_lib_confdb_object_create (void *conn, void *mess
 	api->ipc_response_send(conn, &res_lib_confdb_object_create, sizeof(res_lib_confdb_object_create));
 }
 
-static void message_handler_req_lib_confdb_object_destroy (void *conn, void *message)
+static void message_handler_req_lib_confdb_object_destroy (void *conn,
+							   const void *message)
 {
-	struct req_lib_confdb_object_destroy *req_lib_confdb_object_destroy = (struct req_lib_confdb_object_destroy *)message;
-	mar_res_header_t res;
+	const struct req_lib_confdb_object_destroy *req_lib_confdb_object_destroy
+	  = message;
+	coroipc_response_header_t res;
 	int ret = CS_OK;
 
 	if (api->object_destroy(req_lib_confdb_object_destroy->object_handle))
@@ -328,10 +334,12 @@ static void message_handler_req_lib_confdb_object_destroy (void *conn, void *mes
 	api->ipc_response_send(conn, &res, sizeof(res));
 }
 
-static void message_handler_req_lib_confdb_object_find_destroy (void *conn, void *message)
+static void message_handler_req_lib_confdb_object_find_destroy (void *conn,
+								const void *message)
 {
-	struct req_lib_confdb_object_find_destroy *req_lib_confdb_object_find_destroy = (struct req_lib_confdb_object_find_destroy *)message;
-	mar_res_header_t res;
+	const struct req_lib_confdb_object_find_destroy
+	  *req_lib_confdb_object_find_destroy = message;
+	coroipc_response_header_t res;
 	int ret = CS_OK;
 
 	if (api->object_find_destroy(req_lib_confdb_object_find_destroy->find_handle))
@@ -344,10 +352,12 @@ static void message_handler_req_lib_confdb_object_find_destroy (void *conn, void
 }
 
 
-static void message_handler_req_lib_confdb_key_create (void *conn, void *message)
+static void message_handler_req_lib_confdb_key_create (void *conn,
+						       const void *message)
 {
-	struct req_lib_confdb_key_create *req_lib_confdb_key_create = (struct req_lib_confdb_key_create *)message;
-	mar_res_header_t res;
+	const struct req_lib_confdb_key_create *req_lib_confdb_key_create
+	  = message;
+	coroipc_response_header_t res;
 	int ret = CS_OK;
 
 	if (api->object_key_create(req_lib_confdb_key_create->object_handle,
@@ -363,11 +373,12 @@ static void message_handler_req_lib_confdb_key_create (void *conn, void *message
 	api->ipc_response_send(conn, &res, sizeof(res));
 }
 
-static void message_handler_req_lib_confdb_key_get (void *conn, void *message)
+static void message_handler_req_lib_confdb_key_get (void *conn,
+						    const void *message)
 {
-	struct req_lib_confdb_key_get *req_lib_confdb_key_get = (struct req_lib_confdb_key_get *)message;
+	const struct req_lib_confdb_key_get *req_lib_confdb_key_get = message;
 	struct res_lib_confdb_key_get res_lib_confdb_key_get;
-	int value_len;
+	size_t value_len;
 	void *value;
 	int ret = CS_OK;
 
@@ -388,9 +399,10 @@ static void message_handler_req_lib_confdb_key_get (void *conn, void *message)
 	api->ipc_response_send(conn, &res_lib_confdb_key_get, sizeof(res_lib_confdb_key_get));
 }
 
-static void message_handler_req_lib_confdb_key_increment (void *conn, void *message)
+static void message_handler_req_lib_confdb_key_increment (void *conn,
+							  const void *message)
 {
-	struct req_lib_confdb_key_get *req_lib_confdb_key_get = (struct req_lib_confdb_key_get *)message;
+	const struct req_lib_confdb_key_get *req_lib_confdb_key_get = message;
 	struct res_lib_confdb_key_incdec res_lib_confdb_key_incdec;
 	int ret = CS_OK;
 
@@ -406,9 +418,10 @@ static void message_handler_req_lib_confdb_key_increment (void *conn, void *mess
 	api->ipc_response_send(conn, &res_lib_confdb_key_incdec, sizeof(res_lib_confdb_key_incdec));
 }
 
-static void message_handler_req_lib_confdb_key_decrement (void *conn, void *message)
+static void message_handler_req_lib_confdb_key_decrement (void *conn,
+							  const void *message)
 {
-	struct req_lib_confdb_key_get *req_lib_confdb_key_get = (struct req_lib_confdb_key_get *)message;
+	const struct req_lib_confdb_key_get *req_lib_confdb_key_get = message;
 	struct res_lib_confdb_key_incdec res_lib_confdb_key_incdec;
 	int ret = CS_OK;
 
@@ -424,10 +437,12 @@ static void message_handler_req_lib_confdb_key_decrement (void *conn, void *mess
 	api->ipc_response_send(conn, &res_lib_confdb_key_incdec, sizeof(res_lib_confdb_key_incdec));
 }
 
-static void message_handler_req_lib_confdb_key_replace (void *conn, void *message)
+static void message_handler_req_lib_confdb_key_replace (void *conn,
+							const void *message)
 {
-	struct req_lib_confdb_key_replace *req_lib_confdb_key_replace = (struct req_lib_confdb_key_replace *)message;
-	mar_res_header_t res;
+	const struct req_lib_confdb_key_replace *req_lib_confdb_key_replace
+	  = message;
+	coroipc_response_header_t res;
 	int ret = CS_OK;
 
 	if (api->object_key_replace(req_lib_confdb_key_replace->object_handle,
@@ -443,10 +458,12 @@ static void message_handler_req_lib_confdb_key_replace (void *conn, void *messag
 	api->ipc_response_send(conn, &res, sizeof(res));
 }
 
-static void message_handler_req_lib_confdb_key_delete (void *conn, void *message)
+static void message_handler_req_lib_confdb_key_delete (void *conn,
+						       const void *message)
 {
-	struct req_lib_confdb_key_delete *req_lib_confdb_key_delete = (struct req_lib_confdb_key_delete *)message;
-	mar_res_header_t res;
+	const struct req_lib_confdb_key_delete *req_lib_confdb_key_delete
+	  = message;
+	coroipc_response_header_t res;
 	int ret = CS_OK;
 
 	if (api->object_key_delete(req_lib_confdb_key_delete->object_handle,
@@ -460,9 +477,11 @@ static void message_handler_req_lib_confdb_key_delete (void *conn, void *message
 	api->ipc_response_send(conn, &res, sizeof(res));
 }
 
-static void message_handler_req_lib_confdb_object_parent_get (void *conn, void *message)
+static void message_handler_req_lib_confdb_object_parent_get (void *conn,
+							      const void *message)
 {
-	struct req_lib_confdb_object_parent_get *req_lib_confdb_object_parent_get = (struct req_lib_confdb_object_parent_get *)message;
+	const struct req_lib_confdb_object_parent_get
+	  *req_lib_confdb_object_parent_get = message;
 	struct res_lib_confdb_object_parent_get res_lib_confdb_object_parent_get;
 	hdb_handle_t object_handle;
 	int ret = CS_OK;
@@ -479,14 +498,15 @@ static void message_handler_req_lib_confdb_object_parent_get (void *conn, void *
 }
 
 
-static void message_handler_req_lib_confdb_key_iter (void *conn, void *message)
+static void message_handler_req_lib_confdb_key_iter (void *conn,
+						     const void *message)
 {
-	struct req_lib_confdb_key_iter *req_lib_confdb_key_iter = (struct req_lib_confdb_key_iter *)message;
+	const struct req_lib_confdb_key_iter *req_lib_confdb_key_iter = message;
 	struct res_lib_confdb_key_iter res_lib_confdb_key_iter;
 	void *key_name;
-	int key_name_len;
+	size_t key_name_len;
 	void *value;
-	int value_len;
+	size_t value_len;
 	int ret = CS_OK;
 
 	if (api->object_key_iter_from(req_lib_confdb_key_iter->parent_object_handle,
@@ -509,23 +529,25 @@ static void message_handler_req_lib_confdb_key_iter (void *conn, void *message)
 	api->ipc_response_send(conn, &res_lib_confdb_key_iter, sizeof(res_lib_confdb_key_iter));
 }
 
-static void message_handler_req_lib_confdb_object_iter (void *conn, void *message)
+static void message_handler_req_lib_confdb_object_iter (void *conn,
+							const void *message)
 {
-	struct req_lib_confdb_object_iter *req_lib_confdb_object_iter = (struct req_lib_confdb_object_iter *)message;
+	const struct req_lib_confdb_object_iter *req_lib_confdb_object_iter
+	  = message;
 	struct res_lib_confdb_object_iter res_lib_confdb_object_iter;
-	int object_name_len;
+	size_t object_name_len;
 	int ret = CS_OK;
 
 	if (!req_lib_confdb_object_iter->find_handle) {
 		api->object_find_create(req_lib_confdb_object_iter->parent_object_handle,
 					NULL, 0,
-					&res_lib_confdb_object_iter.find_handle);
+					m2h(&res_lib_confdb_object_iter.find_handle));
 	}
 	else
 		res_lib_confdb_object_iter.find_handle = req_lib_confdb_object_iter->find_handle;
 
 	if (api->object_find_next(res_lib_confdb_object_iter.find_handle,
-				  &res_lib_confdb_object_iter.object_handle)) {
+				  m2h(&res_lib_confdb_object_iter.object_handle))) {
 		ret = CS_ERR_ACCESS;
 		api->object_find_destroy(res_lib_confdb_object_iter.find_handle);
 	}
@@ -543,9 +565,11 @@ static void message_handler_req_lib_confdb_object_iter (void *conn, void *messag
 	api->ipc_response_send(conn, &res_lib_confdb_object_iter, sizeof(res_lib_confdb_object_iter));
 }
 
-static void message_handler_req_lib_confdb_object_find (void *conn, void *message)
+static void message_handler_req_lib_confdb_object_find (void *conn,
+							const void *message)
 {
-	struct req_lib_confdb_object_find *req_lib_confdb_object_find = (struct req_lib_confdb_object_find *)message;
+	const struct req_lib_confdb_object_find *req_lib_confdb_object_find
+	  = message;
 	struct res_lib_confdb_object_find res_lib_confdb_object_find;
 	int ret = CS_OK;
 
@@ -553,13 +577,13 @@ static void message_handler_req_lib_confdb_object_find (void *conn, void *messag
 		api->object_find_create(req_lib_confdb_object_find->parent_object_handle,
 					req_lib_confdb_object_find->object_name.value,
 					req_lib_confdb_object_find->object_name.length,
-					&res_lib_confdb_object_find.find_handle);
+					m2h(&res_lib_confdb_object_find.find_handle));
 	}
 	else
 		res_lib_confdb_object_find.find_handle = req_lib_confdb_object_find->find_handle;
 
 	if (api->object_find_next(res_lib_confdb_object_find.find_handle,
-				  &res_lib_confdb_object_find.object_handle)) {
+				  m2h(&res_lib_confdb_object_find.object_handle))) {
 		ret = CS_ERR_ACCESS;
 		api->object_find_destroy(res_lib_confdb_object_find.find_handle);
 	}
@@ -572,11 +596,12 @@ static void message_handler_req_lib_confdb_object_find (void *conn, void *messag
 	api->ipc_response_send(conn, &res_lib_confdb_object_find, sizeof(res_lib_confdb_object_find));
 }
 
-static void message_handler_req_lib_confdb_write (void *conn, void *message)
+static void message_handler_req_lib_confdb_write (void *conn,
+						  const void *message)
 {
 	struct res_lib_confdb_write res_lib_confdb_write;
 	int ret = CS_OK;
-	char *error_string = NULL;
+	const char *error_string = NULL;
 
 	if (api->object_write_config(&error_string))
 		ret = CS_ERR_ACCESS;
@@ -593,12 +618,13 @@ static void message_handler_req_lib_confdb_write (void *conn, void *message)
 	api->ipc_response_send(conn, &res_lib_confdb_write, sizeof(res_lib_confdb_write));
 }
 
-static void message_handler_req_lib_confdb_reload (void *conn, void *message)
+static void message_handler_req_lib_confdb_reload (void *conn,
+						   const void *message)
 {
-	struct req_lib_confdb_reload *req_lib_confdb_reload = (struct req_lib_confdb_reload *)message;
+	const struct req_lib_confdb_reload *req_lib_confdb_reload = message;
 	struct res_lib_confdb_reload res_lib_confdb_reload;
 	int ret = CS_OK;
-	char *error_string = NULL;
+	const char *error_string = NULL;
 
 	if (api->object_reload_config(req_lib_confdb_reload->flush, &error_string))
 		ret = CS_ERR_ACCESS;
@@ -617,12 +643,12 @@ static void message_handler_req_lib_confdb_reload (void *conn, void *message)
 }
 
 static void confdb_notify_lib_of_key_change(object_change_type_t change_type,
-											  hdb_handle_t parent_object_handle,
-											  hdb_handle_t object_handle,
-											  void *object_name_pt, int object_name_len,
-											  void *key_name_pt, int key_name_len,
-											  void *key_value_pt, int key_value_len,
-											  void *priv_data_pt)
+	hdb_handle_t parent_object_handle,
+	hdb_handle_t object_handle,
+	const void *object_name_pt, size_t object_name_len,
+	const void *key_name_pt, size_t key_name_len,
+	const void *key_value_pt, size_t key_value_len,
+	void *priv_data_pt)
 {
 	struct res_lib_confdb_key_change_callback res;
 
@@ -647,9 +673,9 @@ static void confdb_notify_lib_of_key_change(object_change_type_t change_type,
 }
 
 static void confdb_notify_lib_of_new_object(hdb_handle_t parent_object_handle,
-										   hdb_handle_t object_handle,
-										   uint8_t *name_pt, int name_len,
-										   void *priv_data_pt)
+	hdb_handle_t object_handle,
+	const uint8_t *name_pt, size_t name_len,
+	void *priv_data_pt)
 {
 	struct res_lib_confdb_object_create_callback res;
 
@@ -664,9 +690,10 @@ static void confdb_notify_lib_of_new_object(hdb_handle_t parent_object_handle,
 	api->ipc_dispatch_send(priv_data_pt, &res, sizeof(res));
 }
 
-static void confdb_notify_lib_of_destroyed_object(hdb_handle_t parent_object_handle,
-											uint8_t *name_pt, int name_len,
-											void *priv_data_pt)
+static void confdb_notify_lib_of_destroyed_object(
+	hdb_handle_t parent_object_handle,
+	const uint8_t *name_pt, size_t name_len,
+	void *priv_data_pt)
 {
 	struct res_lib_confdb_object_destroy_callback res;
 
@@ -681,10 +708,11 @@ static void confdb_notify_lib_of_destroyed_object(hdb_handle_t parent_object_han
 }
 
 
-static void message_handler_req_lib_confdb_track_start (void *conn, void *message)
+static void message_handler_req_lib_confdb_track_start (void *conn,
+							const void *message)
 {
-	struct req_lib_confdb_object_track_start *req = (struct req_lib_confdb_object_track_start *)message;
-	mar_res_header_t res;
+	const struct req_lib_confdb_object_track_start *req = message;
+	coroipc_response_header_t res;
 
 	api->object_track_start(req->object_handle,
 		req->flags,
@@ -699,9 +727,10 @@ static void message_handler_req_lib_confdb_track_start (void *conn, void *messag
 	api->ipc_response_send(conn, &res, sizeof(res));
 }
 
-static void message_handler_req_lib_confdb_track_stop (void *conn, void *message)
+static void message_handler_req_lib_confdb_track_stop (void *conn,
+						       const void *message)
 {
-	mar_res_header_t res;
+	coroipc_response_header_t res;
 
 	api->object_track_stop(confdb_notify_lib_of_key_change,
 		confdb_notify_lib_of_new_object,
@@ -714,8 +743,3 @@ static void message_handler_req_lib_confdb_track_stop (void *conn, void *message
 	res.error = CS_OK;
 	api->ipc_response_send(conn, &res, sizeof(res));
 }
-
-
-
-
-
